Package: areal
Type: Package
Title: Areal Weighted Interpolation
Version: 0.1.8
Authors@R: c(
    person("Christopher", "Prener", , "chris.prener@gmail.com", c("aut", "cre"),
    comment = c(ORCID = "0000-0002-4310-9888")),
    person("Charlie","Revord", , "charlie.revord@slu.edu", c("aut")),
    person("Branson", "Fox", ,"branson.fox@slu.edu", c("aut"),
    comment = c(ORCID = "0000-0002-4361-2811"))
    )
Description: A pipeable, transparent implementation of areal weighted interpolation
    with support for interpolating multiple variables in a single function call.
    These tools provide a full-featured workflow for validation and estimation
    that fits into both modern data management (e.g. tidyverse) and spatial 
    data (e.g. sf) frameworks.
Depends: R (>= 3.4)
License: GPL-3
URL: https://chris-prener.github.io/areal/
BugReports: https://github.com/chris-prener/areal/issues
Encoding: UTF-8
LazyData: true
Imports: dplyr, glue, purrr, rlang, sf
RoxygenNote: 7.1.2
Suggests: knitr, rmarkdown, testthat, covr
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2022-05-31 00:46:59 UTC; chris
Author: Christopher Prener [aut, cre] (<https://orcid.org/0000-0002-4310-9888>),
  Charlie Revord [aut],
  Branson Fox [aut] (<https://orcid.org/0000-0002-4361-2811>)
Maintainer: Christopher Prener <chris.prener@gmail.com>
Repository: CRAN
Date/Publication: 2022-05-31 07:40:06 UTC
Built: R 4.6.0; ; 2026-04-28 23:15:22 UTC; unix

Package: areal
Type: Package
Title: Areal Weighted Interpolation
Version: 0.1.8.9000
Authors@R: c(
    person("Christopher", "Prener", , "chris.prener@gmail.com", c("aut", "cre"),
    comment = c(ORCID = "0000-0002-4310-9888")),
    person("Charlie","Revord", , "charlie.revord@slu.edu", c("aut")),
    person("Branson", "Fox", ,"branson.fox@slu.edu", c("aut"),
    comment = c(ORCID = "0000-0002-4361-2811"))
    )
Description: A pipeable, transparent implementation of areal weighted interpolation
    with support for interpolating multiple variables in a single function call.
    These tools provide a full-featured workflow for validation and estimation
    that fits into both modern data management (e.g. tidyverse) and spatial 
    data (e.g. sf) frameworks.
Depends: R (>= 3.4)
License: GPL-3
URL: https://chris-prener.github.io/areal/
BugReports: https://github.com/chris-prener/areal/issues
Encoding: UTF-8
LazyData: true
Imports:
  dplyr,
  glue,
  purrr,
  rlang,
  sf
RoxygenNote: 7.1.2
Suggests:
  knitr,
  rmarkdown,
  testthat,
  covr
VignetteBuilder: knitr

# Generated by roxygen2: do not edit by hand

export(ar_tessellate)
export(ar_validate)
export(aw_aggregate)
export(aw_calculate)
export(aw_interpolate)
export(aw_intersect)
export(aw_preview_weights)
export(aw_total)
export(aw_verify)
export(aw_weight)
importFrom(dplyr,"%>%")
importFrom(dplyr,arrange)
importFrom(dplyr,as_tibble)
importFrom(dplyr,group_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,summarize)
importFrom(glue,glue)
importFrom(rlang,":=")
importFrom(rlang,enquo)
importFrom(rlang,quo)
importFrom(rlang,quo_name)
importFrom(rlang,sym)
importFrom(sf,st_area)
importFrom(sf,st_collection_extract)
importFrom(sf,st_crs)
importFrom(sf,st_geometry)
importFrom(sf,st_intersection)
importFrom(sf,st_is_longlat)
importFrom(sf,st_make_grid)
importFrom(sf,st_sf)
importFrom(sf,st_union)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# areal <img src="man/figures/arealLogo.png" align="right" />

[![R build
status](https://github.com/chris-prener/areal/workflows/R-CMD-check/badge.svg)](https://github.com/chris-prener/areal/actions)
[![Coverage
status](https://codecov.io/gh/chris-prener/areal/branch/master/graph/badge.svg)](https://codecov.io/github/chris-prener/areal?branch=main)
[![CRAN_status_badge](https://www.r-pkg.org/badges/version/areal)](https://cran.r-project.org/package=areal)
[![cran
checks](https://cranchecks.info/badges/worst/areal)](https://cran.r-project.org/web/checks/check_results_areal.html)
[![Downloads](https://cranlogs.r-pkg.org/badges/areal?color=brightgreen)](https://www.r-pkg.org/pkg/areal)
[![DOI](https://zenodo.org/badge/152279647.svg)](https://zenodo.org/badge/latestdoi/152279647)
[![DOI](https://joss.theoj.org/papers/10.21105/joss.01221/status.svg)](https://doi.org/10.21105/joss.01221)

Areal interpolation is the process making estimates from a source set of
polygons to an overlapping but incongruent set of target polygons. One
challenge with areal interpolation is that, while the processes
themselves are well documented in the academic literature, implementing
them often involves “reinventing the wheel” by re-creating the process
in the analyst’s tool choice.

While the `R` package `sf` does offer a basic interface for areal
weighted interpolation (`st_interpolate_aw`), it lacks some features
that we use in our work. The `areal` package contains a suite tools for
validation and estimation, providing a full-featured workflow that fits
into both modern data management (e.g. `tidyverse`) and spatial data
(e.g. `sf`) frameworks.

### *Joural of Open Souce Software* Article

An [article](https://joss.theoj.org/papers/10.21105/joss.01221)
describing `areal`’s approach to areal weighted interpolation has been
published in the [*The Journal of Open Source
Software*](https://joss.theoj.org/). The article includes benchmarking
of `areal` performance on several data sets. Please [cite the
paper](/inst/CITATION) if you use `areal` in your work!

## Installation

The easiest way to get `areal` is to install it from CRAN:

``` r
install.packages("areal")
```

The development version of `areal` can be accessed from GitHub with
`remotes`:

``` r
# install.packages("remotes")
remotes::install_github("chris-prener/areal")
```

Note that installations that require `sf` to be built from *source* will
require additional software regardless of operating system. You should
check the [`sf` package website](https://r-spatial.github.io/sf/) for
the latest details on installing dependencies for that package.
Instructions vary significantly by operating system.

## Usage

Two function prefixes are used in `areal` to allow users to take
advantage of RStudio’s auto complete functionality:

-   `ar_` - data and functions that are used for multiple interpolation
    methods
-   `aw_` - functions that are used specifically for areal weighted
    interpolation

### Data

The package contains four overlapping data sets:

-   `ar_stl_race` (2017 ACS demographic counts at the census tract
    level; *n* = 106)
-   `ar_stl_asthma` (2017 asthma rates at the census tract level; *n*
    = 106)
-   `ar_stl_wards` (the 2010 political subdivisions in St. Louis; *n* =
    28).
-   `ar_stl_wardsClipped` (the 2010 political subdivisions in St. Louis
    clipped to the Mississippi River shoreline; *n* = 28).

These can be used to illustrate the core functionality of the package.
The following examples assume:

``` r
> library(areal)
>
> race <- ar_stl_race
> asthma <- ar_stl_asthma
> wards <- ar_stl_wards
```

### Areal Weighted Interpolation

`areal` currently implements an approach to interpolation known as areal
weighted interpolation. It is arguably the simplest and most common
approach to areal interpolation, though it does have some drawbacks (see
the [areal weighted interpolation
vignette](https://chris-prener.github.io/areal/articles/areal-weighted-interpolation.html)
for details). The basic usage of `areal` is through the
`aw_interpolate()` function. This is a pipe-able function that allows
for the simultaneous interpolation of multiple values.

In this first example, the total estimated population (`TOTAL_E`) of
each ward is calculated from its overlapping census tracts:

``` r
aw_interpolate(wards, tid = WARD, source = race, sid = "GEOID", 
               weight = "sum", output = "sf", extensive = "TOTAL_E")
#> old-style crs object detected; please recreate object with a recent sf::st_crs()
#> old-style crs object detected; please recreate object with a recent sf::st_crs()
#> old-style crs object detected; please recreate object with a recent sf::st_crs()
#> old-style crs object detected; please recreate object with a recent sf::st_crs()
#> old-style crs object detected; please recreate object with a recent sf::st_crs()
#> old-style crs object detected; please recreate object with a recent sf::st_crs()
#> old-style crs object detected; please recreate object with a recent sf::st_crs()
#> old-style crs object detected; please recreate object with a recent sf::st_crs()
#> old-style crs object detected; please recreate object with a recent sf::st_crs()
#> old-style crs object detected; please recreate object with a recent sf::st_crs()
#> old-style crs object detected; please recreate object with a recent sf::st_crs()
#> old-style crs object detected; please recreate object with a recent sf::st_crs()
#> old-style crs object detected; please recreate object with a recent sf::st_crs()
#> Simple feature collection with 28 features and 4 fields
#> Geometry type: POLYGON
#> Dimension:     XY
#> Bounding box:  xmin: 733361.8 ymin: 4268336 xmax: 746157.7 ymax: 4295504
#> Projected CRS: NAD83 / UTM zone 15N
#> First 10 features:
#>    OBJECTID WARD      AREA   TOTAL_E                       geometry
#> 1         1    1  46138761  7991.565 POLYGON ((740184.2 4286431,...
#> 2         2    2 267817711 12145.021 POLYGON ((742392.1 4289178,...
#> 3         3    3  66291644  7344.287 POLYGON ((742956.1 4284113,...
#> 4         4    4  53210707  8457.672 POLYGON ((739557.6 4284080,...
#> 5         5    5  60462396  8783.377 POLYGON ((744883.8 4281632,...
#> 6         6    6  64337271 14050.399 POLYGON ((742501.6 4279976,...
#> 7         7    7 101268146 15840.086 POLYGON ((745618.6 4279867,...
#> 8         8    8  45966410 12188.131 POLYGON ((739842.8 4277724,...
#> 9         9    9  73993891 14217.149 POLYGON ((742619.4 4276734,...
#> 10       10   10  62915358 11239.213 POLYGON ((737257.7 4277050,...
```

This example outputs a simple features (`sf`) object and uses one of two
options for calculating weights. All of these arguments are documented
both within the package (use `?aw_interpolate`) and on the [package’s
website](https://chris-prener.github.io/areal/).

What results from `aw_interpolate()` is mapped below. Total population
per census tract in St. Louis is mapped on the left in panel A. Using
`aw_interpolate()` as we did in the previous example, we estimate
population counts for Wards in St. Louis from those census tract values.
These estimated values are mapped on the right in panel B.

<img src="man/figures/exampleMap.png" width="100%" />

Both extensive and intensive data can be interpolated simultaneously by
using both the `extensive` and `intensive` arguments. In this second
example, the asthma and race data are combined, and estimates for both
the population values and asthma rates are calculated for each ward from
its overlapping census tracts:

``` r
# remove sf geometry
st_geometry(race) <- NULL

# create combined data
race %>%
  select(GEOID, TOTAL_E, WHITE_E, BLACK_E) %>%
  left_join(asthma, ., by = "GEOID") -> combinedData
#> old-style crs object detected; please recreate object with a recent sf::st_crs()

# interpolate
wards %>%
  select(-OBJECTID, -AREA) %>%
  aw_interpolate(tid = WARD, source = combinedData, sid = "GEOID", 
               weight = "total", output = "tibble", 
               extensive = c("TOTAL_E", "WHITE_E", "BLACK_E"),
               intensive = "ASTHMA")
#> old-style crs object detected; please recreate object with a recent sf::st_crs()
#> old-style crs object detected; please recreate object with a recent sf::st_crs()
#> old-style crs object detected; please recreate object with a recent sf::st_crs()
#> old-style crs object detected; please recreate object with a recent sf::st_crs()
#> old-style crs object detected; please recreate object with a recent sf::st_crs()
#> old-style crs object detected; please recreate object with a recent sf::st_crs()
#> old-style crs object detected; please recreate object with a recent sf::st_crs()
#> old-style crs object detected; please recreate object with a recent sf::st_crs()
#> old-style crs object detected; please recreate object with a recent sf::st_crs()
#> old-style crs object detected; please recreate object with a recent sf::st_crs()
#> old-style crs object detected; please recreate object with a recent sf::st_crs()
#> # A tibble: 28 × 5
#>     WARD BLACK_E TOTAL_E WHITE_E ASTHMA
#>    <int>   <dbl>   <dbl>   <dbl>  <dbl>
#>  1     1   7778.   7991.    153.  13.4 
#>  2     2  10552.  12042.   1308.  13.2 
#>  3     3   6627.   7334.    589.  14.1 
#>  4     4   8203.   8458.    160.  13.6 
#>  5     5   6971.   8689.   1518.  13.8 
#>  6     6   7418.  14022.   5833.  11.7 
#>  7     7   6544.  15645.   8123.   9.72
#>  8     8   3796.  12188.   7604.   9.82
#>  9     9   6351.  14095.   6786.  11.8 
#> 10    10   1667.  11239.   8703.   9.44
#> # … with 18 more rows
```

Another advantage of `areal` is that the interpolation process is not a
“black box”, but rather can be manually completed if necessary.
Functions for validating data, previewing the areal weights, and walking
step-by-step through the interpolation process are provided. See the
[areal weighted interpolation
vignette](https://chris-prener.github.io/areal/articles/areal-weighted-interpolation.html)
for additional details about this workflow.

## Road-map

We are planning to experiment with at least three additional techniques
for areal interpolation for possible inclusion into the package. These
include:

-   [Pycnophylactic
    method](https://github.com/chris-prener/areal/issues/1) (raster
    based, eliminates the sharp transitions in value between target
    features)
-   [Binary dasymetric
    method](https://github.com/chris-prener/areal/issues/2)
    (incorporates ancillary data so that population is not assumed to be
    evenly distributed within units)
-   [3-class regression dasymetric
    method](https://github.com/chris-prener/areal/issues/3) (allows for
    a more complex estimation based on multiple forms of ancillary data)

We do not have a timeline for these experiments, though we are planning
to begin experimenting with the pycnophylactic method in the coming
months. We will be keeping the issues (linked to above) updated with
progress. If you are interested in bringing these techniques to `R`,
please feel free to contribute to the development of `areal`. The best
place to start is bt checking in on our GitHub issues for each technique
to see what help is needed!

## Contributor Code of Conduct

Please note that this project is released with a [Contributor Code of
Conduct](https://chris-prener.github.io/areal/CODE_OF_CONDUCT.html). By
participating in this project you agree to abide by its terms.

---
output: github_document
---

<!-- README.md is generated from README.Rmd. Please edit that file -->

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.path = "man/figures/README-",
  out.width = "100%"
)

library(areal)
library(dplyr)
library(sf)

data(ar_stl_asthma, package = "areal")
asthma <- ar_stl_asthma

data(ar_stl_race, package = "areal")
race <- ar_stl_race

data(ar_stl_wards, package = "areal")
wards <- ar_stl_wards
```

# areal <img src="man/figures/arealLogo.png" align="right" />

[![R build status](https://github.com/chris-prener/areal/workflows/R-CMD-check/badge.svg)](https://github.com/chris-prener/areal/actions)
[![Coverage status](https://codecov.io/gh/chris-prener/areal/branch/master/graph/badge.svg)](https://codecov.io/github/chris-prener/areal?branch=main)
[![CRAN_status_badge](https://www.r-pkg.org/badges/version/areal)](https://cran.r-project.org/package=areal)
[![cran checks](https://cranchecks.info/badges/worst/areal)](https://cran.r-project.org/web/checks/check_results_areal.html)
[![Downloads](https://cranlogs.r-pkg.org/badges/areal?color=brightgreen)](https://www.r-pkg.org/pkg/areal)
[![DOI](https://zenodo.org/badge/152279647.svg)](https://zenodo.org/badge/latestdoi/152279647) 
[![DOI](https://joss.theoj.org/papers/10.21105/joss.01221/status.svg)](https://doi.org/10.21105/joss.01221)

Areal interpolation is the process making estimates from a source set of polygons to an overlapping but incongruent set of target polygons. One challenge with areal interpolation is that, while the processes themselves are well documented in the academic literature, implementing them often involves "reinventing the wheel" by re-creating the process in the analyst's tool choice. 

While the `R` package `sf` does offer a basic interface for areal weighted interpolation (`st_interpolate_aw`), it lacks some features that we use in our work. The `areal` package contains a suite tools for validation and estimation, providing a full-featured workflow that fits into both modern data management (e.g. `tidyverse`) and spatial data (e.g. `sf`) frameworks.

### *Joural of Open Souce Software* Article
An [article](https://joss.theoj.org/papers/10.21105/joss.01221) describing `areal`'s approach to areal weighted interpolation has been published in the [*The Journal of Open Source Software*](https://joss.theoj.org/). The article includes benchmarking of `areal` performance on several data sets. Please [cite the paper](/inst/CITATION) if you use `areal` in your work!

## Installation
The easiest way to get `areal` is to install it from CRAN:

``` r
install.packages("areal")
```

The development version of `areal` can be accessed from GitHub with `remotes`:

```r
# install.packages("remotes")
remotes::install_github("chris-prener/areal")
```

Note that installations that require `sf` to be built from *source* will require additional software regardless of operating system. You should check the [`sf` package website](https://r-spatial.github.io/sf/) for the latest details on installing dependencies for that package. Instructions vary significantly by operating system.

## Usage 
Two function prefixes are used in `areal` to allow users to take advantage of RStudio's auto complete functionality:

* `ar_` - data and functions that are used for multiple interpolation methods
* `aw_` - functions that are used specifically for areal weighted interpolation

### Data
The package contains four overlapping data sets: 

* `ar_stl_race` (2017 ACS demographic counts at the census tract level; *n* = 106) 
* `ar_stl_asthma` (2017 asthma rates at the census tract level; *n* = 106)
* `ar_stl_wards` (the 2010 political subdivisions in St. Louis; *n* = 28). 
* `ar_stl_wardsClipped` (the 2010 political subdivisions in St. Louis clipped to the Mississippi River shoreline; *n* = 28). 

These can be used to illustrate the core functionality of the package. The following examples assume:

```r
> library(areal)
>
> race <- ar_stl_race
> asthma <- ar_stl_asthma
> wards <- ar_stl_wards
```

### Areal Weighted Interpolation
`areal` currently implements an approach to interpolation known as areal weighted interpolation. It is arguably the simplest and most common approach to areal interpolation, though it does have some drawbacks (see the [areal weighted interpolation vignette](https://chris-prener.github.io/areal/articles/areal-weighted-interpolation.html) for details). The basic usage of `areal` is through the `aw_interpolate()` function. This is a pipe-able function that allows for the simultaneous interpolation of multiple values.

In this first example, the total estimated population (`TOTAL_E`) of each ward is calculated from its overlapping census tracts:

```{r iteration}
aw_interpolate(wards, tid = WARD, source = race, sid = "GEOID", 
               weight = "sum", output = "sf", extensive = "TOTAL_E")
```

This example outputs a simple features (`sf`) object and uses one of two options for calculating weights. All of these arguments are documented both within the package (use `?aw_interpolate`) and on the [package's website](https://chris-prener.github.io/areal/). 

What results from `aw_interpolate()` is mapped below. Total population per census tract in St. Louis is mapped on the left in panel A. Using `aw_interpolate()` as we did in the previous example, we estimate population counts for Wards in St. Louis from those census tract values. These estimated values are mapped on the right in panel B.

```{r exampleMap, echo=FALSE, out.width = '100%'}
knitr::include_graphics("man/figures/exampleMap.png")
```

Both extensive and intensive data can be interpolated simultaneously by using both the `extensive` and `intensive` arguments. In this second example, the asthma and race data are combined, and estimates for both the population values and asthma rates are calculated for each ward from its overlapping census tracts:

```{r mixed}
# remove sf geometry
st_geometry(race) <- NULL

# create combined data
race %>%
  select(GEOID, TOTAL_E, WHITE_E, BLACK_E) %>%
  left_join(asthma, ., by = "GEOID") -> combinedData

# interpolate
wards %>%
  select(-OBJECTID, -AREA) %>%
  aw_interpolate(tid = WARD, source = combinedData, sid = "GEOID", 
               weight = "total", output = "tibble", 
               extensive = c("TOTAL_E", "WHITE_E", "BLACK_E"),
               intensive = "ASTHMA")
```

Another advantage of `areal` is that the interpolation process is not a "black box", but rather can be manually completed if necessary. Functions for validating data, previewing the areal weights, and walking step-by-step through the interpolation process are provided. See the [areal weighted interpolation vignette](https://chris-prener.github.io/areal/articles/areal-weighted-interpolation.html) for additional details about this workflow.

## Road-map 
We are planning to experiment with at least three additional techniques for areal interpolation for possible inclusion into the package. These include:

- [Pycnophylactic method](https://github.com/chris-prener/areal/issues/1) (raster based, eliminates the sharp transitions in value between target features)
- [Binary dasymetric method](https://github.com/chris-prener/areal/issues/2) (incorporates ancillary data so that population is not assumed to be evenly distributed within units)
- [3-class regression dasymetric method](https://github.com/chris-prener/areal/issues/3) (allows for a more complex estimation based on multiple forms of ancillary data)

We do not have a timeline for these experiments, though we are planning to begin experimenting with the pycnophylactic method in the coming months. We will be keeping the issues (linked to above) updated with progress. If you are interested in bringing these techniques to `R`, please feel free to contribute to the development of `areal`. The best place to start is bt checking in on our GitHub issues for each technique to see what help is needed!

## Contributor Code of Conduct
Please note that this project is released with a [Contributor Code of Conduct](https://chris-prener.github.io/areal/CODE_OF_CONDUCT.html). By participating in this project you agree to abide by its terms.

devtools::load_all()

# target data
data(ar_stl_wards, package = "areal")

# source data
data(ar_stl_race, package = "areal")

# calculate intersection
ar_stl_wards %>%
  aw_intersect(source = ar_stl_race, areaVar = "...area") %>%
  aw_total(source = ar_stl_race, id = GEOID, areaVar = "...area",
           totalVar = "...totalArea", weight = "sum", type = "extensive") %>%
  aw_weight(areaVar = "...area", totalVar = "...totalArea", areaWeight = "...areaWeight") %>%
  aw_calculate(value = "TOTAL_E", areaWeight = "...areaWeight") -> intersect

aw_aggregate(intersect, target = ar_stl_wards, tid = ham, interVar = "TOTAL_E")

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

library(areal)
library(dplyr)
library(sf)

data(ar_stl_asthma, package = "areal")
data(ar_stl_race, package = "areal")
data(ar_stl_wards, package = "areal")
data(ar_stl_wardsClipped, package = "areal")

## ----load-data----------------------------------------------------------------
library(areal)

# load data into enviornment
race <- ar_stl_race                 # census tract population estimates
asthma <- ar_stl_asthma             # census tract asthma rate estimates
wards <- ar_stl_wards               # political boundaries
wardsClipped <- ar_stl_wardsClipped # political boundaries clipped to river

## ----featureMap, echo=FALSE, out.width = '100%'-------------------------------
knitr::include_graphics("../man/figures/featureMap.png")

## ----feature-count------------------------------------------------------------
# print number of features in source
nrow(race)

# print number of features in target
nrow(wards)

# create intersect for example purposes
nrow(suppressWarnings(sf::st_intersection(race, wards)))

## ----intersectMap, echo=FALSE, out.width = '100%'-----------------------------
knitr::include_graphics("../man/figures/intersectMap.png")

## ----data-by-hand, echo=FALSE-------------------------------------------------
as_tibble(
  data.frame(
    GEOID = c(29510101100, 29510101100, 29510101200, 29510101200),
    TOTAL_E = c(2510, 2510, 3545, 3545),
    WARD = c(11, 12, 12, 13)
  )
) %>% 
  knitr::kable(caption = "First Four Rows of Intersected Data")

## ----weight-by-hand, echo=FALSE-----------------------------------------------
as_tibble(
  data.frame(
    GEOID = c(29510101100, 29510101100, 29510101200, 29510101200),
    TOTAL_E = c(2510, 2510, 3545, 3545),
    WARD = c(11, 12, 12, 13),
    Ai = c(355702.9, 901331.1, 875554.7, 208612.1),
    Aj = c(1257034.0, 1257034.0, 1084166.8, 1084166.8),
    Wi = c(0.28297, 0.71703, 0.807583, 0.192417)
  )
) %>% 
  knitr::kable(caption = "First Four Rows of Intersected Data")

## ----calculate-by-hand, echo=FALSE--------------------------------------------
as_tibble(
  data.frame(
    GEOID = c(29510101100, 29510101100, 29510101200, 29510101200),
    TOTAL_E = c(2510, 2510, 3545, 3545),
    WARD = c(11, 12, 12, 13),
    Ai = c(355702.9, 901331.1, 875554.7, 208612.1),
    Aj = c(1257034.0, 1257034.0, 1084166.8, 1084166.8),
    Wi = c(0.28297, 0.71703, 0.807583, 0.192417),
    EST = c(710.2547, 1799.745, 2862.882, 682.1182)
  )
) %>% 
  knitr::kable(caption = "First Four Rows of Intersected Data")

## ----aggregate-by-hand, echo=FALSE--------------------------------------------
as_tibble(
  data.frame(
    WARD = c(11, 12, 13),
    EST = c(710.2547, 4662.627, 682.1182)
  )
) %>% 
  knitr::kable(caption = "Resulting Target Data")

## ----extensive----------------------------------------------------------------
aw_interpolate(wards, tid = WARD, source = race, sid = GEOID, 
               weight = "sum", output = "tibble", extensive = "TOTAL_E")

## ----extensive-vector---------------------------------------------------------
aw_interpolate(wards, tid = WARD, source = race, sid = GEOID, 
               weight = "sum", output = "tibble", 
               extensive = c("TOTAL_E", "WHITE_E", "BLACK_E"))

## ----extensive-weights--------------------------------------------------------
aw_preview_weights(wards, tid = WARD, source = race, sid = GEOID, 
                   type = "extensive")

## ----verify-true--------------------------------------------------------------
result <- aw_interpolate(wards, tid = WARD, source = race, sid = GEOID, 
               weight = "sum", output = "tibble", extensive = "TOTAL_E")

aw_verify(source = race, sourceValue = TOTAL_E, 
          result = result, resultValue = TOTAL_E)

## ----verify-fail--------------------------------------------------------------
result <- aw_interpolate(wards, tid = WARD, source = race, sid = GEOID, 
               weight = "total", output = "tibble", extensive = "TOTAL_E")

aw_verify(source = race, sourceValue = TOTAL_E, 
          result = result, resultValue = TOTAL_E)

## ----overlapMap, echo=FALSE, out.width = '100%'-------------------------------
knitr::include_graphics("../man/figures/overlapMap.png")

## ----extensive-weights-overlap------------------------------------------------
aw_preview_weights(wardsClipped, tid = WARD, source = race, sid = GEOID, 
                   type = "extensive")

## ----invenstive-weights-------------------------------------------------------
aw_preview_weights(wards, tid = WARD, source = asthma, sid = GEOID, 
                   type = "intensive")

## ----intensive----------------------------------------------------------------
aw_interpolate(wards, tid = WARD, source = asthma, sid = GEOID, 
               weight = "sum", output = "tibble", intensive = "ASTHMA")

## ----mixed--------------------------------------------------------------------
# remove sf geometry
st_geometry(race) <- NULL

# create combined data
race %>%
  select(GEOID, TOTAL_E, WHITE_E, BLACK_E) %>%
  left_join(asthma, ., by = "GEOID") -> combinedData

# interpolate
aw_interpolate(wards, tid = WARD, source = combinedData, sid = GEOID, 
               weight = "sum", output = "tibble", intensive = "ASTHMA",
               extensive = c("TOTAL_E", "WHITE_E", "BLACK_E"))

## ----constraints--------------------------------------------------------------
# re-load data
race <- ar_stl_race

# create combined data
race %>%
  select(GEOID, WHITE_E, BLACK_E) %>%
  mutate(
    TOTAL = WHITE_E+BLACK_E,
    WHITE_PCT = WHITE_E/TOTAL,
    BLACK_PCT = BLACK_E/TOTAL,
    TOTAL_PCT = WHITE_PCT+BLACK_PCT
  ) -> constrainedData

# interpolate
result2 <- aw_interpolate(ar_stl_wards, tid = WARD, 
               source = constrainedData, sid = GEOID, 
               weight = "sum", output = "tibble", 
               intensive = c("WHITE_PCT", "BLACK_PCT", "TOTAL_PCT"),
               extensive = c("TOTAL", "WHITE_E", "BLACK_E"))

# calculate new percentages
result2 %>%
  mutate(
    WHITE_PCT_2 = WHITE_E/TOTAL,
    BLACK_PCT_2 = BLACK_E/TOTAL,
    TOTAL_PCT_2 = WHITE_PCT_2+BLACK_PCT_2
  ) -> result2

# display
result2 %>%
  select(WHITE_PCT, WHITE_PCT_2, BLACK_PCT, BLACK_PCT_2, TOTAL_PCT, TOTAL_PCT_2)

## ----ouput--------------------------------------------------------------------
aw_interpolate(wards, tid = WARD, source = asthma, sid = GEOID, 
               weight = "sum", output = "sf", intensive = "ASTHMA")

## ----piped-input--------------------------------------------------------------
wards %>%
  select(-OBJECTID, -AREA) %>%
  aw_interpolate(tid = WARD, source = asthma, sid = GEOID, 
                 weight = "sum", output = "tibble", intensive = "ASTHMA")

## ----quoted-input-------------------------------------------------------------
wards %>%
  select(-OBJECTID, -AREA) %>%
  aw_interpolate(tid = "WARD", source = asthma, sid = "GEOID", 
                 weight = "sum", output = "tibble", intensive = "ASTHMA")

## ----manual-subset------------------------------------------------------------
race <- select(ar_stl_race, GEOID, TOTAL_E)
wards <- select(wards, -OBJECTID, -AREA)

## ----aw-intersect-------------------------------------------------------------
wards %>%
  aw_intersect(source = race, areaVar = "area") -> intersect

intersect

## ----aw-total-----------------------------------------------------------------
intersect %>%
  aw_total(source = race, id = GEOID, areaVar = "area", totalVar = "totalArea",
             type = "extensive", weight = "sum") -> intersect

intersect

## ----aw-weight----------------------------------------------------------------
intersect %>%
  aw_weight(areaVar = "area", totalVar = "totalArea", 
            areaWeight = "areaWeight") -> intersect

intersect

## ----aw-calculate-------------------------------------------------------------
intersect %>%
  aw_calculate(value = TOTAL_E, areaWeight = "areaWeight") -> intersect

intersect

## ----aw-aggregate-------------------------------------------------------------
intersect %>%
  aw_aggregate(target = wards, tid = WARD, interVar = TOTAL_E) -> result

result

# Create spatial data for examples and testing

library(dplyr)
library(readr)
library(sf)
library(tidycensus)
library(tigris)
library(usethis)

get_acs(geography = "tract", state = 29, county = 510, year = 2017, table = "B02001", output = "wide") %>%
  select(-NAME) %>%
  rename(TOTAL_E = B02001_001E, TOTAL_M = B02001_001M,
         WHITE_E = B02001_002E, WHITE_M = B02001_002M,
         BLACK_E = B02001_003E, BLACK_M = B02001_003M,
         AIAN_E = B02001_004E, AIAN_M = B02001_004M,
         ASIAN_E = B02001_005E, ASIAN_M = B02001_005M,
         NHPI_E = B02001_006E, NHPI_M = B02001_006M,
         OTHER_E = B02001_007E, OTHER_M = B02001_007M,
         TWOPLUS_E = B02001_008E, TWOPLUS_M = B02001_008M) %>%
  select(-B02001_009E, -B02001_009M, -B02001_010E, -B02001_010M) -> stlRace

tracts(state = 29, county = 510, class = "sf") %>%
  select(GEOID, STATEFP, COUNTYFP, TRACTCE, NAMELSAD, ALAND, AWATER) -> stlTracts

left_join(stlTracts, stlRace, by = "GEOID") %>%
  st_transform(crs = "ESRI:102296") -> ar_stl_race

st_read("inst/extdata/STL_POLITICS_Wards10.shp", stringsAsFactors = FALSE) %>%
  select(-Shape_Leng) %>%
  rename(AREA = Shape_Area,
         WARD = WARD10) %>%
  st_transform(crs = "ESRI:102296") -> ar_stl_wards

read_csv("inst/extdata/STL_HEALTH_Asthma.csv") %>%
  mutate(GEOID = as.character(geoID)) %>%
  rename(ASTHMA = pctAsthma) %>%
  select(GEOID, ASTHMA) %>%
  left_join(ar_stl_race, ., by = "GEOID") %>%
  select(GEOID, STATEFP, COUNTYFP, TRACTCE, NAMELSAD, ALAND, AWATER, ASTHMA) -> ar_stl_asthma

st_read("inst/extdata/STL_POLITICS_WardsClipped.shp", stringsAsFactors = FALSE) %>%
  select(-OBJECTID) %>%
  st_transform(crs = "ESRI:102296") -> ar_stl_wardsClipped

use_data(ar_stl_race, overwrite = TRUE)
use_data(ar_stl_wards, overwrite = TRUE)
use_data(ar_stl_wardsClipped, overwrite = TRUE)
use_data(ar_stl_asthma, overwrite = TRUE)

rm(stlRace, stlTracts, ar_stl_race, ar_stl_wards, ar_stl_asthma, ar_stl_wardsClipped)

totalCompare1 <- aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_race, sid = GEOID,
                               weight = "sum", output = "sf", extensive = "TOTAL_E")

save(totalCompare1, file = "inst/testdata/totalCompare1.rda", version = 2)

# create map for README, areal vignette

library(areal)
library(ggplot2)
library(cowplot)
library(viridis)
library(prener)
library(here)

result <- aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_race, sid = "GEOID",
                         weight = "sum", output = "sf", extensive = "TOTAL_E")

p1 <- ggplot() +
  geom_sf(ar_stl_race, mapping = aes(fill = TOTAL_E)) +
  scale_fill_viridis() +
  labs(
    title = "Total Population per Tract",
    subtitle = "St. Louis, MO (2017)"
  ) +
  cp_sequoiaTheme(base_size = 14, background = "white", map = TRUE) +
  theme(legend.key.size = unit(1, units="cm"))

p2 <- ggplot() +
  geom_sf(result, mapping = aes(fill = TOTAL_E)) +
  scale_fill_viridis(option="plasma") +
  labs(
    title = "Total Population per Ward",
    subtitle = "St. Louis, MO (2017)"
  ) +
  cp_sequoiaTheme(base_size = 14, background = "white", map = TRUE) +
  theme(legend.key.size = unit(1, units="cm"))

theme_set(theme_cowplot(font_family = "sans", font_size = 14))

exampleMap <- plot_grid(p1, p2, labels = c("(A)", "(B)"))

ggplot2::ggsave(filename = here("man", "figures", "exampleMap.png"), exampleMap,
       width = cp_points(800, units = "in"), height = cp_points(450, units = "in"), dpi = 72)

# create map for README, areal vignette

library(areal)
library(sf)
library(ggplot2)
library(cowplot)
library(prener)
library(here)

p1 <- ggplot() +
  geom_sf(data = ar_stl_race, fill = "#000000", color = "#ffffff") +
  labs(
    title = "Census Tract Features",
    subtitle = "St. Louis, MO"
  ) +
  cp_sequoiaTheme(base_size = 14, background = "white", map = TRUE) +
  theme(legend.position="none")

p2 <- ggplot() +
  geom_sf(data = ar_stl_wards, fill = "#000000", color = "#ffffff") +
  labs(
    title = "2010 Ward Features",
    subtitle = "St. Louis, MO"
  ) +
  cp_sequoiaTheme(base_size = 14, background = "white", map = TRUE) +
  theme(legend.position="none")

theme_set(theme_cowplot(font_family = "sans", font_size = 14))

exampleMap <- plot_grid(p1, p2, labels = c("(A)", "(B)"))

ggplot2::ggsave(filename = here("man", "figures", "featureMap.png"), exampleMap,
       width = cp_points(800, units = "in"), height = cp_points(450, units = "in"), dpi = 72)

intersect <- st_intersection(ar_stl_race, ar_stl_wards)

p3 <- ggplot() +
  geom_sf(data = intersect, fill = "#000000", color = "#ffffff") +
  labs(
    title = "Intersected Features",
    subtitle = "St. Louis, MO"
  ) +
  cp_sequoiaTheme(base_size = 14, background = "white", map = TRUE) +
  theme(legend.position="none")

intersectMap <- plot_grid(p3, NULL, labels = c("(C)"))

ggplot2::ggsave(filename = here("man", "figures", "intersectMap.png"), intersectMap,
                width = cp_points(800, units = "in"), height = cp_points(450, units = "in"), dpi = 72)

p4 <- ggplot() +
  geom_sf(data = ar_stl_race, fill = "#ff0000", color = "#000000") +
  geom_sf(data = ar_stl_wardsClipped, fill = "#ffffff", color = "#000000") +
  labs(
    title = "Clipped Wards Overlaid on Tracts",
    subtitle = "St. Louis, MO"
  ) +
  cp_sequoiaTheme(base_size = 14, background = "white", map = TRUE) +
  theme(legend.position="none")

overlapMap <- plot_grid(p4, NULL, labels = c("(D)"))

ggplot2::ggsave(filename = here("man", "figures", "overlapMap.png"), overlapMap,
                width = cp_points(800, units = "in"), height = cp_points(450, units = "in"), dpi = 72)

library(tigris)
library(sf)
library(dplyr)

area_water(state = 29, county = 510, class = "sf") %>%
  select(HYDROID, FULLNAME) %>%
  filter(FULLNAME == "Mississippi Riv") %>%
  mutate(COUNTY = "St Louis City") %>%
  st_transform(crs = 26915) -> stlWater

area_water(state = 29, county = "St. Louis County", class = "sf") %>%
  select(HYDROID, FULLNAME) %>%
  filter(HYDROID == "1101561067896" | HYDROID == "1103035479819") %>%
  mutate(COUNTY = "St Louis County") %>%
  st_transform(crs = 26915) -> slcWater

area_water(state = 17, county = "St. Clair", class = "sf") %>%
  select(HYDROID, FULLNAME) %>%
  filter(FULLNAME == "Mississippi Riv") %>%
  mutate(COUNTY = "St Clair") %>%
  st_transform(crs = 26915) -> stcWater

area_water(state = 17, county = "Madison", class = "sf") %>%
  select(HYDROID, FULLNAME) %>%
  filter(FULLNAME == "Mississippi Riv") %>%
  mutate(COUNTY = "Madison") %>%
  st_transform(crs = 26915) -> madWater

rbind(stlWater, slcWater) %>%
  rbind(., madWater) %>%
  rbind(., stcWater) -> regionWater

st_write(regionWater, "inst/extdata/regionWater.shp", delete_dsn = TRUE)

#' Aggregate Estimates Based on Target ID
#'
#' @description \code{aw_aggregate} sums the new estimates produced by \link{aw_calculate}
#'     based on the target id. These are then joined with the target data. This is
#'     the fourth step in the interpolation process after \link{aw_weight}.
#'
#' @usage aw_aggregate(.data, target, tid, interVar, newVar)
#'
#' @param .data A given intersected dataset
#' @param target A \code{sf} object that data should be interpolated to
#' @param tid A unique identification number within \code{target}
#' @param interVar A variable containing an interpolated value created by \code{aw_calculate}
#' @param newVar Optional; a new field name to store the interpolated value in. If not specified,
#'     the \code{interVar} argument will be used as the new field name.
#'
#' @return A \code{sf} object with the interpolated value added to it.
#'
#' @examples
#' library(dplyr)
#'
#' race <- select(ar_stl_race, GEOID, TOTAL_E)
#' wards <- select(ar_stl_wards, WARD)
#'
#' wards %>%
#'     aw_intersect(source = race, areaVar = "area") %>%
#'     aw_total(source = race, id = GEOID, areaVar = "area", totalVar = "totalArea",
#'              weight = "sum", type = "extensive") %>%
#'     aw_weight(areaVar = "area", totalVar = "totalArea", areaWeight = "areaWeight") %>%
#'     aw_calculate(value = "TOTAL_E", areaWeight = "areaWeight") -> intersect
#'
#' aw_aggregate(intersect, target = wards, tid = WARD, interVar = TOTAL_E)
#'
#' @importFrom dplyr group_by
#' @importFrom dplyr left_join
#' @importFrom dplyr summarize
#' @importFrom glue glue
#' @importFrom rlang :=
#' @importFrom rlang enquo
#' @importFrom rlang quo
#' @importFrom rlang quo_name
#' @importFrom rlang sym
#' @importFrom sf st_geometry
#'
#' @export
aw_aggregate <- function(.data, target, tid, interVar, newVar){

  # save parameters to list
  paramList <- as.list(match.call())

  # check for missing parameters
  if (missing(.data)) {
    stop("A sf object containing intersected data must be specified for the '.data' argument.")
  }

  if (missing(target)) {
    stop("A sf object must be specified for the 'target' argument.")
  }

  if (missing(tid)) {
    stop("A variable name must be specified for the 'tid' argument.")
  }

  if (missing(interVar)) {
    stop("A variable name must be specified for the 'interVar' argument.")
  }

  # nse
  if (!is.character(paramList$tid)) {
    tidQ <- rlang::enquo(tid)
  } else if (is.character(paramList$tid)) {
    tidQ <- rlang::quo(!! rlang::sym(tid))
  }

  tidQN <- rlang::quo_name(rlang::enquo(tid))

  if (!is.character(paramList$interVar)) {
    interVarQ <- rlang::enquo(interVar)
  } else if (is.character(paramList$interVar)) {
    interVarQ <- rlang::quo(!! rlang::sym(interVar))
  }

  interVarQN <- rlang::quo_name(rlang::enquo(interVarQ))

  if (missing(newVar)){

    newVarQN <- interVarQN

  } else if (!missing(newVar)){

    if (!is.character(paramList$newVar)) {
      newVarQ <- rlang::enquo(newVar)
    } else if (is.character(paramList$newVar)) {
      newVarQ <- rlang::quo(!! rlang::sym(newVar))
    }

    newVarQN <- rlang::quo_name(rlang::enquo(newVarQ))

  }

  # check variables
  if(!!tidQN %in% colnames(target) == FALSE) {
    stop(glue::glue("Variable '{var}', given for the target ID ('tid'), cannot be found in the given target object.",
                    var = tidQN))
  }

  # remove geometry
  sf::st_geometry(.data) <- NULL

  # calculate total area
  .data %>%
    dplyr::group_by(!!tidQ) %>%
    dplyr::summarize(!!newVarQN := base::sum(!!interVarQ)) -> sum

  # join to input data
  out <- dplyr::left_join(target, sum, by = tidQN)

  # return output
  return(out)

}

context("test aw_aggregate function")

# load test data ------------------------------------------------

# target data
data(ar_stl_wards, package = "areal")

# source data
data(ar_stl_race, package = "areal")

# calculate intersection
ar_stl_wards %>%
  aw_intersect(source = ar_stl_race, areaVar = "...area") %>%
  aw_total(source = ar_stl_race, id = GEOID, areaVar = "...area",
           totalVar = "...totalArea", weight = "sum", type = "extensive") %>%
  aw_weight(areaVar = "...area", totalVar = "...totalArea", areaWeight = "...areaWeight") %>%
  aw_calculate(value = "TOTAL_E", areaWeight = "...areaWeight") -> intersect

# test errors ------------------------------------------------

test_that("errors with missing objects", {
  expect_error(aw_aggregate(ham, target = ar_stl_wards, tid = WARD, interVar = "TOTAL_E"),
               "object 'ham' not found")
  expect_error(aw_aggregate(intersect, target = ham, tid = WARD, interVar = "TOTAL_E"),
               "object 'ham' not found")
})

test_that("errors with missing parameters", {
  expect_error(aw_aggregate(target = ar_stl_wards, tid = WARD, interVar = "TOTAL_E"),
               "A sf object containing intersected data must be specified for the '.data' argument.")
  expect_error(aw_aggregate(intersect, tid = WARD, interVar = "TOTAL_E"),
               "A sf object must be specified for the 'target' argument.")
  expect_error(aw_aggregate(intersect, target = ar_stl_wards, interVar = "TOTAL_E"),
               "A variable name must be specified for the 'tid' argument.")
  expect_error(aw_aggregate(intersect, target = ar_stl_wards, tid = WARD),
               "A variable name must be specified for the 'interVar' argument.")
})

test_that("errors with objects and id variables that do not exist", {
  expect_error(aw_aggregate(intersect, target = ar_stl_wards, tid = ham, interVar = "TOTAL_E"),
               "Variable 'ham', given for the target ID \\('tid'\\), cannot be found in the given target object.")
})

# test inputs ------------------------------------------------

test_that("correctly specified functions execute without error", {
  expect_error(aw_aggregate(intersect, target = ar_stl_wards, tid = "WARD", interVar = "TOTAL_E"), NA)
  expect_error(aw_aggregate(intersect, target = ar_stl_wards, tid = WARD, interVar = TOTAL_E), NA)
  expect_error(aw_aggregate(intersect, target = ar_stl_wards, tid = WARD, interVar = TOTAL_E, newVar = "ham"), NA)
  expect_error(aw_aggregate(intersect, target = ar_stl_wards, tid = WARD, interVar = TOTAL_E, newVar = ham), NA)
})

#' Calculate Areal Weight
#'
#' @description \code{aw_weight} creates an area weight field by dividing the area
#'     field by the total area field. This is the third step in the interpolation
#'     process after \link{aw_weight}.
#'
#' @usage aw_weight(.data, areaVar, totalVar, areaWeight)
#'
#' @param .data A \code{sf} object that has been intersected using \link{aw_intersect}
#' @param areaVar The name of the variable measuring a feature's area
#' @param totalVar The name of the variable containing total area field by \code{source} id
#' @param areaWeight The name of a new area weight field to be calculated
#'
#' @return A \code{sf} object with the intersected data and new area weight field.
#'
#' @examples
#' library(dplyr)
#'
#' race <- select(ar_stl_race, GEOID, TOTAL_E)
#' wards <- select(ar_stl_wards, WARD)
#'
#' wards %>%
#'     aw_intersect(source = race, areaVar = "area") %>%
#'     aw_total(source = race, id = GEOID, areaVar = "area", totalVar = "totalArea",
#'              weight = "sum", type = "extensive") -> intersect
#'
#' aw_weight(intersect, areaVar = "area", totalVar = "totalArea", areaWeight = "areaWeight")
#'
#' @importFrom dplyr mutate
#' @importFrom glue glue
#' @importFrom rlang :=
#' @importFrom rlang enquo
#' @importFrom rlang quo
#' @importFrom rlang quo_name
#' @importFrom rlang sym
#'
#' @export
aw_weight <- function(.data, areaVar, totalVar, areaWeight){

  # save parameters to list
  paramList <- as.list(match.call())

  # check for missing parameters
  if (missing(.data)) {
    stop("A sf object containing intersected data must be specified for the '.data' argument.")
  }

  if (missing(areaVar)) {
    stop("A variable name must be specified for the 'areaVar' argument.")
  }

  if (missing(totalVar)) {
    stop("A variable name must be specified for the 'totalVar' argument.")
  }

  if (missing(areaWeight)) {
    stop("A variable name must be specified for the 'areaWeight' argument.")
  }

  # nse
  if (!is.character(paramList$areaVar)) {
    areaVarQ <- rlang::enquo(areaVar)
  } else if (is.character(paramList$areaVar)) {
    areaVarQ <- rlang::quo(!! rlang::sym(areaVar))
  }

  areaVarQN <- rlang::quo_name(rlang::enquo(areaVar))

  if (!is.character(paramList$totalVar)) {
    totalVarQ <- rlang::enquo(totalVar)
  } else if (is.character(paramList$totalVar)) {
    totalVarQ <- rlang::quo(!! rlang::sym(totalVar))
  }

  totalVarQN <- rlang::quo_name(rlang::enquo(totalVar))

  if (!is.character(paramList$areaWeight)) {
    areaWeightQ <- rlang::enquo(areaWeight)
  } else if (is.character(paramList$areaWeight)) {
    areaWeightQ <- rlang::quo(!! rlang::sym(areaWeight))
  }

  areaWeightQN <- rlang::quo_name(rlang::enquo(areaWeight))

  # check variables
  if (!!areaVarQN != "...area"){

    if(!!areaVarQN %in% colnames(.data) == FALSE) {
      stop(glue::glue("Variable '{var}', given for the area, cannot be found in the given intersected object.",
                      var = areaVarQN))
    }

  }

  if (!!totalVarQN != "...totalArea"){

    if(!!totalVarQN %in% colnames(.data) == FALSE) {
      stop(glue::glue("Variable '{var}', given for the total area, cannot be found in the given intersected object.",
                      var = totalVarQN))
    }

  }

  # calculate area weight of intersection slivers
  out <- dplyr::mutate(.data, !!areaWeightQN := !!areaVarQ / !!totalVarQ)

  # return output
  return(out)

}

context("test aw_calculate function")

# load test data ------------------------------------------------

# target data
data(ar_stl_wards, package = "areal")

# source data
data(ar_stl_race, package = "areal")

# calculate intersection
ar_stl_wards %>%
  aw_intersect(source = ar_stl_race, areaVar = "...area") %>%
  aw_total(source = ar_stl_race, id = GEOID, areaVar = "...area",
           totalVar = "...totalArea", weight = "sum", type = "extensive") %>%
  aw_weight(areaVar = "...area", totalVar = "...totalArea", areaWeight = "...areaWeight") -> intersect

# test errors ------------------------------------------------

test_that("errors with missing objects", {
  expect_error(aw_calculate(ham, value = "TOTAL_E", areaWeight = "...areaWeight"),
               "object 'ham' not found")
})

test_that("errors with missing parameters", {
  expect_error(aw_calculate(value = "TOTAL_E", areaWeight = "...areaWeight"),
               "A sf object containing intersected data must be specified for the '.data' argument.")
  expect_error(aw_calculate(intersect, areaWeight = "...areaWeight"),
               "A variable name must be specified for the 'value' argument.")
  expect_error(aw_calculate(intersect, value = "TOTAL_E"),
               "A variable name must be specified for the 'areaWeight' argument.")
})

test_that("errors with misspecified parameters", {
  expect_error(aw_calculate(intersect, value = "ham", areaWeight = "...areaWeight"),
               "Variable 'ham', given for the value, cannot be found in the given intersected object.")
  expect_error(aw_calculate(intersect, value = "...area", areaWeight = "ham"),
               "Variable 'ham', given for the area weight, cannot be found in the given intersected object.")
})

# test inputs ------------------------------------------------

test_that("correctly specified functions execute without error", {
  expect_error(aw_calculate(intersect, value = "TOTAL_E", areaWeight = "...areaWeight"), NA)
  expect_error(aw_calculate(intersect, value = "TOTAL_E", areaWeight = "...areaWeight", newVar = "TOTAL_E"), NA)
  expect_error(aw_calculate(intersect, value = TOTAL_E, areaWeight = ...areaWeight, newVar = TOTAL_E), NA)
})

context("test aw_weight function")

# load test data ------------------------------------------------

# target data
data(ar_stl_wards, package = "areal")

# source data
data(ar_stl_race, package = "areal")

# calculate intersection
ar_stl_wards %>%
  aw_intersect(source = ar_stl_race, areaVar = "...area") %>%
  aw_total(source = ar_stl_race, id = GEOID, areaVar = "...area",
           totalVar = "...totalArea", weight = "sum", type = "extensive")-> intersect

# test errors ------------------------------------------------

test_that("errors with missing objects", {
  expect_error(aw_weight(ham, areaVar = "...area", totalVar = "...totalArea", areaWeight = "...areaWeight"),
               "object 'ham' not found")
})

test_that("errors with missing parameters", {
  expect_error(aw_weight(areaVar = "...area", totalVar = "...totalArea", areaWeight = "...areaWeight"),
               "A sf object containing intersected data must be specified for the '.data' argument.")
  expect_error(aw_weight(intersect, totalVar = "...totalArea", areaWeight = "...areaWeight"),
               "A variable name must be specified for the 'areaVar' argument.")
  expect_error(aw_weight(intersect, areaVar = "...area", areaWeight = "...areaWeight"),
               "A variable name must be specified for the 'totalVar' argument.")
  expect_error(aw_weight(intersect, areaVar = "...area", totalVar = "...totalArea"),
               "A variable name must be specified for the 'areaWeight' argument.")
})

test_that("errors with misspecified parameters", {
  expect_error(aw_weight(intersect, areaVar = "ham", totalVar = "...totalArea", areaWeight = "...areaWeight"),
               "Variable 'ham', given for the area, cannot be found in the given intersected object.")
  expect_error(aw_weight(intersect, areaVar = "...area", totalVar = "ham", areaWeight = "...areaWeight"),
               "Variable 'ham', given for the total area, cannot be found in the given intersected object.")
})

# test inputs ------------------------------------------------

test_that("correctly specified functions execute without error", {
  expect_error(aw_weight(intersect, areaVar = "...area", totalVar = "...totalArea",
                         areaWeight = "...areaWeight"), NA)
  expect_error(aw_weight(intersect, areaVar = ...area, totalVar = ...totalArea,
                         areaWeight = ...areaWeight), NA)
})

#' Clipped Ward Boundaries in St. Louis, 2010
#'
#' @description A simple features data set containing the 2010 Ward boundaries, which
#'     are used as districts for Alderpersons who serve as elected representatives.
#'     This version of the ward boundary has been modified so that the wards only
#'     extend to the Mississippi River shoreline.
#'
#' @docType data
#'
#' @usage data(ar_stl_wardsClipped)
#'
#' @format A data frame with 28 rows and 2 variables:
#' \describe{
#'   \item{WARD}{Ward number}
#'   \item{geometry}{simple features geometry}
#'   }
#'
#' @source City of St. Louis
#'
#' @examples
#' str(ar_stl_wardsClipped)
#' head(ar_stl_wardsClipped)
#'
"ar_stl_wardsClipped"

#' Ward Boundaries in St. Louis, 2010
#'
#' @description A simple features data set containing the 2010 Ward boundaries, which
#'     are used as districts for Alderpersons who serve as elected representatives.
#'     The \code{OBJECTID} and \code{AREA} columns are included to simulate "real"
#'     data that may have superfluous or unclear columns.
#'
#' @docType data
#'
#' @usage data(ar_stl_wards)
#'
#' @format A data frame with 28 rows and 4 variables:
#' \describe{
#'   \item{OBJECTID}{Artifact from ESRI data creation}
#'   \item{WARD}{Ward number}
#'   \item{AREA}{area of each ward}
#'   \item{geometry}{simple features geometry}
#'   }
#'
#' @source City of St. Louis
#'
#' @examples
#' str(ar_stl_wards)
#' head(ar_stl_wards)
#' summary(ar_stl_wards$AREA)
#'
"ar_stl_wards"

#' Intersect Source and Target Data
#'
#' @description \code{aw_intersect} intersects the source and target datasets and
#'     computes a new area field for the intersected data using the units associated
#'     with whatever project the data are currently in. This is the first step in the
#'     interpolation process after data validation and subsetting.
#'
#' @usage aw_intersect(.data, source, areaVar)
#'
#' @param .data A \code{sf} object that data should be interpolated to
#' @param source A \code{sf} object with data to be interpolated
#' @param areaVar The name of the new area variable to be calculated.
#'
#' @return A \code{sf} object with the intersected data and new area field.
#'
#' @examples
#' library(dplyr)
#'
#' race <- select(ar_stl_race, GEOID, TOTAL_E)
#' wards <- select(ar_stl_wards, WARD)
#'
#' aw_intersect(wards, source = race, areaVar = "area")
#'
#' @importFrom dplyr %>% mutate rename
#' @importFrom glue glue
#' @importFrom rlang :=
#' @importFrom rlang enquo
#' @importFrom rlang quo
#' @importFrom rlang quo_name
#' @importFrom rlang sym
#' @importFrom sf st_area st_collection_extract st_intersection
#' @importFrom dplyr rename
#' @importFrom dplyr mutate
#'
#' @export
aw_intersect <- function(.data, source, areaVar) {

  # save parameters to list
  paramList <- as.list(match.call())

  # check for missing parameters
  if (missing(.data)) {
    stop("A sf object containing target data must be specified for the '.data' argument.")
  }

  if (missing(source)) {
    stop("A sf object containing source data must be specified for the 'source' argument.")
  }

  if (missing(areaVar)) {
    stop("A variable name must be specified for the 'areaVar' argument.")
  }

  # nse
  if (!is.character(paramList$areaVar)) {
    areaVarQ <- rlang::enquo(areaVar)
  } else if (is.character(paramList$areaVar)) {
    areaVarQ <- rlang::quo(!! rlang::sym(areaVar))
  }

  areaVarQN <- rlang::quo_name(rlang::enquo(areaVarQ))

  # preform intersection
  intersection <- suppressWarnings(sf::st_intersection(source, .data))

  # if a geometry collection is returned, extract it
  if(any(grepl("GEOMETRY", sf::st_geometry_type(intersection)))) {
    intersection <- sf::st_collection_extract(intersection)
  }

  # calculate area
  intersection <- aw_area(intersection, areaVar = !!areaVarQ)

  # return output
  return(intersection)

}

# Calculate area
#
# @description Calculate the area of a feature in the units of the current
#     coordinate system. This is called by \code{aw_intersect}.
#
# @param .data A \code{sf} object that data should be interpolated to
# @param areaVar The name of the new area variable to be calculated.
#
# @return A \code{sf} object with the new area field.
#
aw_area <- function(.data, areaVar){

  # undefined global variables note
  geometry = NULL

  # save parameters to list
  paramList <- as.list(match.call())

  # nse
  areaVarQN <- rlang::quo_name(rlang::enquo(areaVar))

  # calculate area
  calculated_area <- unclass(sf::st_area(.data))

  # join
  out <- cbind(.data, calculated_area)

  # rename
  out <- dplyr::rename(out, !!areaVarQN := calculated_area)

  # return output
  return(out)

}

context("test aw_intersect function")

# load test data ------------------------------------------------

# target data
data(ar_stl_wards, package = "areal")

# source data
data(ar_stl_race, package = "areal")

# test errors ------------------------------------------------

test_that("errors with missing objects", {
  expect_error(aw_intersect(ham, source = ar_stl_race, areaVar = "area"),
               "object 'ham' not found")
  expect_error(aw_intersect(ar_stl_wards, source = ham, areaVar = "area"),
               "object 'ham' not found")
})

test_that("errors with missing parameters", {
  expect_error(aw_intersect(source = ar_stl_race, areaVar = "area"),
               "A sf object containing target data must be specified for the '.data' argument.")
  expect_error(aw_intersect(ar_stl_wards, areaVar = "area"),
               "A sf object containing source data must be specified for the 'source' argument.")
  expect_error(aw_intersect(ar_stl_wards, source = ar_stl_race),
               "A variable name must be specified for the 'areaVar' argument.")
})

# test inputs ------------------------------------------------

test_that("correctly specified functions execute without error", {
  expect_error(aw_intersect(ar_stl_wards, source = ar_stl_race, areaVar = "area"), NA)
  expect_error(aw_intersect(ar_stl_wards, source = ar_stl_race, areaVar = area), NA)
})

# test bugs ------------------------------------------------

load(system.file("testdata", "geometryCollection.rda", package = "areal", mustWork = TRUE))

test_that("GEOMETRY check casts to MULTIPOLYGON when necessary", {
  intersection <- aw_intersect(si_zcta, si_tract, "area")
  expect_equal(all(sf::st_geometry_type(intersection) == "MULTIPOLYGON"), TRUE)
})

context("test aw_total function")

# load test data ------------------------------------------------

# target data
data(ar_stl_wards, package = "areal")

# source data
data(ar_stl_race, package = "areal")

# calculate intersection
ar_stl_wards %>%
  aw_intersect(source = ar_stl_race, areaVar = "...area") -> intersect

# test errors ------------------------------------------------

test_that("errors with missing objects", {
  expect_error(aw_total(ham, source = ar_stl_race, id = GEOID, areaVar = "...area",
                            totalVar = "...totalArea", weight = "sum", type = "extensive"),
               "object 'ham' not found")
})

test_that("errors with missing parameters", {
  expect_error(aw_total(source = ar_stl_race, id = GEOID, areaVar = "...area",
                        totalVar = "...totalArea", weight = "sum", type = "extensive"),
               "A sf object containing intersected data must be specified for the '.data' argument.")
  expect_error(aw_total(intersect, id = GEOID, areaVar = "...area",
                        totalVar = "...totalArea", weight = "sum", type = "extensive"),
               "A sf object containing souce data must be specified for the 'source' argument.")
  expect_error(aw_total(intersect, source = ar_stl_race, areaVar = "...area",
                        totalVar = "...totalArea", weight = "sum", type = "extensive"),
               "A variable name must be specified for the 'id' argument.")
  expect_error(aw_total(intersect, source = ar_stl_race, id = GEOID,
                        totalVar = "...totalArea", weight = "sum", type = "extensive"),
               "A variable name must be specified for the 'areaVar' argument.")
  expect_error(aw_total(intersect, source = ar_stl_race, id = GEOID, areaVar = "...area",
                        weight = "sum", type = "extensive"),
               "A variable name must be specified for the 'totalVar' argument.")
  expect_error(aw_total(intersect, source = ar_stl_race, id = GEOID, areaVar = "...area",
                        totalVar = "...totalArea", type = "extensive"),
               "A weight type \\(either 'sum' or 'total'\\) must be specified for the 'weight' argument.")
  expect_error(aw_total(intersect, source = ar_stl_race, id = GEOID, areaVar = "...area",
                        totalVar = "...totalArea", weight = "sum"),
               "An interpolation type \\(either 'extensive' or 'intensive'\\) must be specified for the 'type' argument.")
})

test_that("errors with weight and output", {
  expect_error(aw_total(intersect, source = ar_stl_race, id = GEOID, areaVar = "...area",
                        totalVar = "...totalArea", weight = "ham", type = "extensive"),
               "The given weight type 'ham' is not valid. 'weight' must be either 'sum' or 'total'.")
  expect_error(aw_total(intersect, source = ar_stl_race, id = GEOID, areaVar = "...area",
                        totalVar = "...totalArea", weight = "total", type = "intensive"),
               "Spatially intensive interpolations should be caclulated using 'sum' for 'weight'.")
})

test_that("errors with misspecified parameters", {
  expect_error(aw_total(intersect, source = ar_stl_race, id = ham, areaVar = "...area",
                        totalVar = "...totalArea", weight = "sum", type = "extensive"),
               "Variable 'ham', given for the ID \\('id'\\), cannot be found in the given intersected object.")
  expect_error(aw_total(intersect, source = ar_stl_race, id = GEOID, areaVar = "ham",
                        totalVar = "...totalArea", weight = "sum", type = "extensive"),
               "Variable 'ham', given for the area, cannot be found in the given intersected object.")
})

# test inputs ------------------------------------------------

test_that("correctly specified functions execute without error", {
  expect_error(aw_total(intersect, source = ar_stl_race, id = GEOID, areaVar = ...area,
                        totalVar = ...totalArea, weight = "sum", type = "extensive"), NA)
  expect_error(aw_total(intersect, source = ar_stl_race, id = "GEOID", areaVar = "...area",
                        totalVar = "...totalArea", weight = "sum", type = "extensive"), NA)
})

#' Calculate Total Area
#'
#' @description \code{aw_total} produces a new total area field that contains
#'     the total area by \code{source} id. This is the second step in the
#'     interpolation process after \link{aw_intersect}.
#'
#' @usage aw_total(.data, source, id, areaVar, totalVar, type, weight)
#'
#' @param .data A \code{sf} object that has been intersected using \link{aw_intersect}
#' @param source A \code{sf} object with data to be interpolated
#' @param id A unique identification number
#' @param areaVar The name of the variable measuring a feature's area, which is
#'     created as part of \link{aw_intersect}
#' @param totalVar The name of a new total area field to be calculated
#' @param type One of \code{"intensive"} or \code{"extensive"}
#' @param weight One of \code{"sum"} or \code{"total"}
#'
#' @return A \code{sf} object with the intersected data and new total area field.
#'
#' @examples
#' library(dplyr)
#'
#' race <- select(ar_stl_race, GEOID, TOTAL_E)
#' wards <- select(ar_stl_wards, WARD)
#'
#' wards %>%
#'     aw_intersect(source = race, areaVar = "area") -> intersect
#'
#' aw_total(intersect, source = race, id = GEOID, areaVar = "area",
#'          totalVar = "totalArea", weight = "sum", type = "extensive")
#'
#' @importFrom dplyr %>%
#' @importFrom dplyr group_by
#' @importFrom dplyr left_join
#' @importFrom dplyr mutate
#' @importFrom dplyr select
#' @importFrom dplyr summarize
#' @importFrom glue glue
#' @importFrom rlang :=
#' @importFrom rlang enquo
#' @importFrom rlang quo
#' @importFrom rlang quo_name
#' @importFrom rlang sym
#' @importFrom sf st_geometry
#'
#' @export
aw_total <- function(.data, source, id, areaVar, totalVar, type, weight){

  # save parameters to list
  paramList <- as.list(match.call())

  # global binding
  geometry = NULL

  # check for missing parameters
  if (missing(.data)) {
    stop("A sf object containing intersected data must be specified for the '.data' argument.")
  }

  if (missing(source)) {
    stop("A sf object containing souce data must be specified for the 'source' argument.")
  }

  if (missing(id)) {
    stop("A variable name must be specified for the 'id' argument.")
  }

  if (missing(areaVar)) {
    stop("A variable name must be specified for the 'areaVar' argument.")
  }

  if (missing(totalVar)) {
    stop("A variable name must be specified for the 'totalVar' argument.")
  }

  if (missing(type)) {
    stop("An interpolation type (either 'extensive' or 'intensive') must be specified for the 'type' argument.")
  }

  if (missing(weight)) {
    stop("A weight type (either 'sum' or 'total') must be specified for the 'weight' argument.")
  }

  # check for misspecified parameters
  if (weight %in% c("sum", "total") == FALSE){
    stop(glue::glue("The given weight type '{var}' is not valid. 'weight' must be either 'sum' or 'total'.",
                    var = weight))
  }

  if (type == "intensive" & weight == "total"){
    stop("Spatially intensive interpolations should be caclulated using 'sum' for 'weight'.")
  }

  # nse
  if (!is.character(paramList$id)) {
    idQ <- rlang::enquo(id)
  } else if (is.character(paramList$id)) {
    idQ <- rlang::quo(!! rlang::sym(id))
  }

  idQN <- rlang::quo_name(rlang::enquo(id))

  if (!is.character(paramList$areaVar)) {
    areaVarQ <- rlang::enquo(areaVar)
  } else if (is.character(paramList$areaVar)) {
    areaVarQ <- rlang::quo(!! rlang::sym(areaVar))
  }

  areaVarQN <- rlang::quo_name(rlang::enquo(areaVar))

  totalVarQN <- rlang::quo_name(rlang::enquo(totalVar))

  # check variables
  if(!!idQN %in% colnames(.data) == FALSE) {
    stop(glue::glue("Variable '{var}', given for the ID ('id'), cannot be found in the given intersected object.",
                    var = idQN))
  }

  if (!!areaVarQN != "...area"){

    if(!!areaVarQN %in% colnames(.data) == FALSE) {
      stop(glue::glue("Variable '{var}', given for the area, cannot be found in the given intersected object.",
                      var = areaVarQN))
    }

  }

  if (type == "intensive" | (type == "extensive" & weight == "sum")){

    # remove geometry
    df <- .data
    sf::st_geometry(df) <- NULL

    # calculate sum of source area
    df %>%
      dplyr::group_by(!!idQ) %>%
      dplyr::summarize(!!totalVarQN := base::sum(!!areaVarQ)) -> sum

    # join to input data
    out <- dplyr::left_join(.data, sum, by = idQN)

  } else if (type == "extensive" & weight == "total"){

    # calculate total source area
    source %>%
      dplyr::select(!!idQ) %>%
      dplyr::mutate(!!totalVarQN := unclass(sf::st_area(geometry))) -> total

    sf::st_geometry(total) <- NULL

    # join to input data
    out <- dplyr::left_join(.data, total, by = idQN)

  }

  # return output
  return(out)

}

#' Validating Data for Interpolation
#'
#' @description \code{ar_validate} executes a series of logic tests for \code{sf} object status,
#'     shared coordinates between source and target data, appropriate project, and absence of
#'     variable name conflicts.
#'
#' @usage ar_validate(source, target, varList, method = "aw", verbose = FALSE)
#'
#' @param source A \code{sf} object with data to be interpolated
#' @param target A \code{sf} object that data should be interpolated to
#' @param varList A vector of variable names to be added to the \code{target} object
#' @param method The areal interpolation method validation is being performed for. This
#'     should be set to \code{"aw"}. Additional functionality will be added as the package
#'     adds new interpolation techniques.
#' @param verbose A logical scalar; if \code{TRUE}, a tibble with test results is returned
#'
#' @return If \code{verbose} is \code{FALSE}, a logical scalar is returned that is \code{TRUE}
#'     is all tests are passed and \code{FALSE} if one or more tests is failed. If \code{verbose}
#'     is \code{TRUE}, a tibble with detailed test results is returned.
#'
#' @seealso \link{c}
#'
#' @examples
#' ar_validate(source = ar_stl_asthma, target = ar_stl_wards, varList = "ASTHMA")
#'
#' ar_validate(source = ar_stl_asthma, target = ar_stl_wards, varList = "ASTHMA", verbose = TRUE)
#'
#' @importFrom glue glue
#' @importFrom sf st_crs st_is_longlat
#'
#' @export
ar_validate <- function(source, target, varList, method = "aw", verbose = FALSE){

  # check for missing parameters
  if (missing(source)) {
    stop("A sf object containing source data must be specified for the 'source' argument.")
  }

  if (missing(target)) {
    stop("A sf object containing target data must be specified for the 'target' argument.")
  }

  if (missing(varList)) {
    stop("A variable name or vector of variable names must be specified for the 'varList' argument.")
  }

  if (verbose != TRUE & verbose != FALSE){
    stop("The 'verbose' argument must be either 'TRUE' or 'FALSE'.")
  }

  if (method != "aw"){
    stop("The 'method' argument must be 'aw'.")
  }

  # store results from primary validate subfunctions
  sf_result <- ar_validate_sf(source, target)

  # execute additional tests if both are sf, otherwise set results to NA
  if (sf_result == FALSE){

    crs_result <- NA
    longlat_result <- NA
    polygon_result <- NA
    vars_exist_result <- NA
    vars_conflict_result <- NA


  } else if (sf_result == TRUE){

    # do both source and target have same CRS?
    crs_result <- ar_validate_crs(source, target)

    # are both source and target CRS values in planar?
    longlat_result1 <- ar_validate_longlat(source)
    longlat_result2 <- ar_validate_longlat(target)

    longlat_result <- all(longlat_result1, longlat_result2)

    # are both data sets polygon data?
    polygon_result1 <- ar_validate_polygon(source)
    polygon_result2 <- ar_validate_polygon(target)

    polygon_result <- all(polygon_result1, polygon_result2)

    # are there no conflicts with target variable names?
    vars_conflict_result <- ar_validate_vars_conflict(target, varList = varList)
    vars_exist_result <- ar_validate_vars_exist(source, varList = varList)

  }

  # determine if overall test is passed
  if(sf_result == "TRUE" & crs_result == "TRUE" & longlat_result == "TRUE" &
     polygon_result == "TRUE" &
     vars_exist_result == "TRUE" & vars_conflict_result == "TRUE") {

    result <- TRUE

  } else {

    result <- FALSE

  }

  # conditional code if verbose is assigned FALSE
  if(verbose == FALSE){

    out <- result

  }

  # conditional code if verbose is assigned TRUE
  else if (verbose == TRUE){

    table <- data.frame(
      test = c("sf Objects", "CRS Match", "CRS is Planar", "Polygon Geometries",
               "Variables Exist in Source", "No Variable Conflicts in Target",
               "Overall Evaluation"),
      result = c(sf_result, crs_result, longlat_result, polygon_result,
                 vars_exist_result, vars_conflict_result, result),
      stringsAsFactors = FALSE)

    out <- as_tibble(table)
  }

  # return output
  return(out)

}

# Lite Version of Validation for aw_preview_weights
#
# @description \code{aw_validate_preview} is designed to be called by
#     \code{aw_preview_weights} before the weights are calculated. It
#     lacks the variable validation functionality of \code{ar_validate}.
#
# @param source A \code{sf} object with data to be interpolated
# @param target A \code{sf} object that data should be interpolated to
#
# @return If \code{verbose} is \code{FALSE}, a logical scalar is returned that is \code{TRUE}
#     is all tests are passed and \code{FALSE} if one or more tests is failed. If \code{verbose}
#     is \code{TRUE}, a tibble with detailed test results is returned.
#
aw_validate_preview <- function(source, target){

  # store results from primary validate subfunctions
  sf_result <- ar_validate_sf(source, target)

  # execute additional tests if both are sf, otherwise set results to NA
  if (sf_result == FALSE){

    crs_result <- NA
    longlat_result <- NA
    polygon_result <- NA

  } else if (sf_result == TRUE){

    # do both source and target have same CRS?
    crs_result <- ar_validate_crs(source, target)

    # are both source and target CRS values in planar?
    longlat_result1 <- ar_validate_longlat(source)
    longlat_result2 <- ar_validate_longlat(target)

    longlat_result <- all(longlat_result1, longlat_result2)

    # are both data sets polygon data?
    polygon_result1 <- ar_validate_polygon(source)
    polygon_result2 <- ar_validate_polygon(target)

    polygon_result <- all(polygon_result1, polygon_result2)

  }

  # determine if overall test is passed
  if(sf_result == "TRUE" & crs_result == "TRUE" & longlat_result == "TRUE" &
     polygon_result == "TRUE") {

    out <- TRUE

  } else {

    out <- FALSE

  }

  # return output
  return(out)

}

# Testing for sf object status for source and target data
#
# @description \code{ar_validate_sf} conducts a logic test for shared coordinate
#     coordinate systems, which are a requirement for interpolation.
#
# @param source A \code{sf} object with data to be interpolated
# @param target A \code{sf} object that data should be interpolated to
#
# @return A logical scalar; if \code{TRUE}, the test is passed.
#
ar_validate_sf <- function(source, target){

  # identify sf object in class
  source_sf <- "sf" %in% class(source)
  target_sf <- "sf" %in% class(target)

  if(source_sf == TRUE & target_sf == TRUE){

    # if both objects are sf
    out <- TRUE

  } else if(source_sf == FALSE | target_sf == FALSE){

    # if one or both are not sf
    out <- FALSE

  }

  # return result output
  return(out)

}

# Testing for shared coordinates for source and target data
#
# @description \code{awrvalidate_crs} conducts a logic test for shared coordinate
#     coordinate systems, which are a requirement for interpolation.
#
# @param source A \code{sf} object with data to be interpolated
# @param target A \code{sf} object that data should be interpolated to
#
# @return A logical scalar; if \code{TRUE}, the test is passed.
#
ar_validate_crs <- function(source, target){

  if(sf::st_crs(source) == sf::st_crs(target)) {

    # if both objects share crs
    out <- TRUE

  } else if(sf::st_crs(source) != sf::st_crs(target)) {

    # if objects have different crs
    out <- FALSE
  }

  # return result output
  return(out)

}

# Testing for type of coordinates
#
# @description \code{ar_validate_longlat} conducts a logic test for
#     whether or not the data are in planar format.
#
# @param .data A sf object
#
# @return A logical scalar; if \code{TRUE}, the test is passed
#
ar_validate_longlat <- function(.data){

  result <- sf::st_is_longlat(.data)

  if (result == TRUE){

    # if object is in lat long
    out <- FALSE

  } else if (result == FALSE){

    # if object is in planar
    out <- TRUE

  }

  # return result output
  return(out)

}

# Testing for Variable Conflicts in Target
#
# @description \code{ar_validate_vars_conflict} conducts a logic test for
#     whether or not any of the variables to be created in the target
#     data already exist as named columns.
#
# @param .data A sf object
# @param varList A vector of variables to be created
#
# @return A logical scalar; if \code{TRUE}, the test is passed
#
ar_validate_vars_conflict <- function(.data, varList){

  # create logical vector
  resultVector <- varList %in% colnames(.data)
  result <- any(resultVector)

  if (result == TRUE){

    # if at least one variable name is in target
    out <- FALSE

  } else if (result == FALSE){

    # if no existing variable names are in target
    out <- TRUE

  }

  # return result output
  return(out)

}

# Testing for Variables Existing in Source
#
# @description \code{ar_validate_vars_exist} conducts a logic test for
#     whether or not all variables exist in the source data.
#
# @param .data A sf object
# @param varList A vector of variables assumed to exist.
#
# @return A logical scalar; if \code{TRUE}, the test is passed
#
ar_validate_vars_exist <- function(.data, varList){

  # create logical vector
  resultVector <- varList %in% colnames(.data)
  out <- all(resultVector)

  # return result output
  return(out)

}

# Testing Geometry
ar_validate_polygon <- function(.data){

  # create logical vector
  out <- any(sf::st_geometry_type(.data) %in% c("POLYGON", "MULTIPOLYGON"))

  # return result output
  return(out)

}

context("test ar_validate function")

# load test data ------------------------------------------------

# target data
data(ar_stl_wards, package = "areal")

# source data
data(ar_stl_race, package = "areal")

# test errors ------------------------------------------------

test_that("errors with missing objects", {
  expect_error(ar_validate(source = ham, target = ar_stl_wards, varList = "TOTAL_E", method = "aw"),
               "object 'ham' not found")
  expect_error(ar_validate(source = ar_stl_race, target = ham, varList = "TOTAL_E", method = "aw"),
               "object 'ham' not found")
})

test_that("errors with missing parameters", {
  expect_error(ar_validate(target = ar_stl_wards, varList = "TOTAL_E", method = "aw"),
               "A sf object containing source data must be specified for the 'source' argument.")
  expect_error(ar_validate(source = ar_stl_race, varList = "TOTAL_E", method = "aw"),
               "A sf object containing target data must be specified for the 'target' argument.")
  expect_error(ar_validate(source = ar_stl_race, target = ar_stl_wards, method = "aw"),
               "A variable name or vector of variable names must be specified for the 'varList' argument.")
})

test_that("errors with misspecified parameters", {
  expect_error(ar_validate(source = ar_stl_race, target = ar_stl_wards, varList = "TOTAL_E",
                           method = "aw", verbose = "ham"),
               "The 'verbose' argument must be either 'TRUE' or 'FALSE'.")
  expect_error(ar_validate(source = ar_stl_race, target = ar_stl_wards, varList = "TOTAL_E",
                           method = "ham", verbose = FALSE),
               "The 'method' argument must be 'aw'.")
})

# test inputs ------------------------------------------------

test_that("correctly specified functions execute without error", {
  expect_equal(ar_validate(source = ar_stl_race, target = ar_stl_wards, method = "aw",
                           varList = c("TOTAL_E", "BLACK_E")), TRUE)
})

# test validation failures ------------------------------------------------

wardsdf <- ar_stl_wards
sf::st_geometry(wardsdf) <- NULL

racedf <- ar_stl_race
sf::st_geometry(racedf) <- NULL

wards83 <- sf::st_transform(ar_stl_wards, crs = 4269)
race83 <- sf::st_transform(ar_stl_race, crs = 4269)

wards_points <- suppressWarnings(sf::st_centroid(ar_stl_wards))

ar_stl_wards_conflict <- dplyr::mutate(ar_stl_wards, WHITE_E = FALSE)

test_that("validation result is false", {
  expect_equal(ar_validate(source = racedf, target = wardsdf, varList = "TOTAL_E", method = "aw"), FALSE)
  expect_equal(ar_validate(source = ar_stl_race, target = wardsdf, varList = "TOTAL_E", method = "aw"), FALSE)
  expect_equal(ar_validate(source = racedf, target = ar_stl_wards, varList = "TOTAL_E", method = "aw"), FALSE)
  expect_equal(ar_validate(source = ar_stl_race, target = wards83, varList = "TOTAL_E", method = "aw"), FALSE)
  expect_equal(ar_validate(source = race83, target = ar_stl_wards, varList = "TOTAL_E", method = "aw"), FALSE)
  expect_equal(ar_validate(source = ar_stl_race, target = ar_stl_wards, varList = "WARD", method = "aw"), FALSE)
  expect_equal(ar_validate(source = ar_stl_race, target = ar_stl_wards_conflict, varList = "WHITE_E"), FALSE)
})

test_that("verbose option creates tibble", {
  expect_equal(class(ar_validate(source = ar_stl_race, target = ar_stl_wards, varList = "TOTAL_E",
                                 method = "aw", verbose = TRUE))[1], "tbl_df")
})

invalidV1 <- ar_validate(source = racedf, target = wardsdf, varList = "TOTAL_E", method = "aw", verbose = TRUE)
invalidV2 <- ar_validate(source = ar_stl_race, target = wardsdf, varList = "TOTAL_E", method = "aw", verbose = TRUE)
invalidV3 <- ar_validate(source = racedf, target = ar_stl_wards, varList = "TOTAL_E", method = "aw", verbose = TRUE)

invalidSF <- c(FALSE, NA, NA, NA, NA, NA, FALSE)

test_that("invalid sf objects return appropriate verbose output", {
  expect_equal(invalidV1$result, invalidSF)
  expect_equal(invalidV2$result, invalidSF)
  expect_equal(invalidV2$result, invalidSF)
})

invalidV4 <- ar_validate(source = ar_stl_race, target = wards83, varList = "TOTAL_E", method = "aw", verbose = TRUE)
invalidV5 <- ar_validate(source = race83, target = ar_stl_wards, varList = "TOTAL_E", method = "aw", verbose = TRUE)

invalidCRS <- c(TRUE, FALSE, FALSE, TRUE, TRUE, TRUE, FALSE)

test_that("invalid crs returns appropriate verbose output", {
  expect_equal(invalidV4$result, invalidCRS)
  expect_equal(invalidV5$result, invalidCRS)
})

invalidV6 <- ar_validate(source = ar_stl_race, target = ar_stl_wards, varList = "HAM", method = "aw", verbose = TRUE)
invalidSourceVars <- c(TRUE, TRUE, TRUE, TRUE, FALSE, TRUE, FALSE)

invalidV7 <- ar_validate(source = ar_stl_race, target = ar_stl_wards, varList = "WARD", method = "aw", verbose = TRUE)
invalidTargetVars <- c(TRUE, TRUE, TRUE, TRUE, FALSE, FALSE, FALSE)

invalidV8 <- ar_validate(source = ar_stl_race, target = ar_stl_wards_conflict, varList = "WHITE_E", method = "aw",
                         verbose = TRUE)
invalidVarsConflict <- c(TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, FALSE)

test_that("invalid crs returns appropriate verbose output", {
  expect_equal(invalidV6$result, invalidSourceVars)
  expect_equal(invalidV7$result, invalidTargetVars)
  expect_equal(invalidV8$result, invalidVarsConflict)
})

invalidV9 <- ar_validate(source = ar_stl_race, target = wards_points, varList = "WHITE_E", method = "aw",
                         verbose = TRUE)
invalidPolygon <- c(TRUE, TRUE, TRUE, FALSE, TRUE, TRUE, FALSE)

test_that("point data appropriate verbose output", {
  expect_equal(invalidV9$result, invalidPolygon)
})

#' Race in St. Louis by Census Tract, 2017
#'
#' @description A simple features data set containing the geometry and associated attributes
#'     for the 2013-2017 American Community Survey estimates for race in St. Louis.
#'
#' @docType data
#'
#' @usage data(ar_stl_race)
#'
#' @format A data frame with 106 rows and 24 variables:
#' \describe{
#'   \item{GEOID}{full GEOID string}
#'   \item{STATEFP}{state FIPS code}
#'   \item{COUNTYFP}{county FIPS code}
#'   \item{TRACTCE}{tract FIPS code}
#'   \item{NAMELSAD}{tract name}
#'   \item{ALAND}{area of tract land, square meters}
#'   \item{AWATER}{area of tract water, square meters}
#'   \item{TOTAL_E}{total populaton count, estimated}
#'   \item{TOTAL_M}{total populaton count, margin of error}
#'   \item{WHITE_E}{white populaton count, estimated}
#'   \item{WHITE_M}{white populaton count, margin of error}
#'   \item{BLACK_E}{black populaton count, estimated}
#'   \item{BLACK_M}{black populaton count, margin of error}
#'   \item{AIAN_E}{american indian and alskan native populaton count, estimated}
#'   \item{AIAN_M}{american indian and alskan native populaton count, margin of error}
#'   \item{ASIAN_E}{asian populaton count, estimated}
#'   \item{ASIAN_M}{asian populaton count, margin of error}
#'   \item{NHPI_E}{native hawaiian and pacific islander populaton count, estimated}
#'   \item{NHPI_M}{native hawaiian and pacific islander populaton count, margin of error}
#'   \item{OTHER_E}{other race populaton count, estimated}
#'   \item{OTHER_M}{other race populaton count, margin of error}
#'   \item{TWOPLUS_E}{two or more races populaton count, estimated}
#'   \item{TWOPLUS_M}{two or more races populaton count, margin of error}
#'   \item{geometry}{simple features geometry}
#'   }
#'
#' @source \code{tidycensus} package
#'
#' @examples
#' str(ar_stl_race)
#' head(ar_stl_race)
#' summary(ar_stl_race$ALAND)
#'
"ar_stl_race"

#' Create Tessellations From SF Object
#'
#' @usage ar_tessellate(.data, shape = "square", size = 1)
#'
#' @param .data An object of class \code{sf} to tessellate from
#' @param shape One of 'square' or 'hexagon', the shape to make tessellations from
#' @param size Numeric multiplier for size of tessellations, default is one kilometer
#'
#' @return A \code{sf} object
#'
#' @examples
#' ar_tessellate(ar_stl_wards)
#'
#' ar_tessellate(ar_stl_wards, shape = "hexagon", size = .75)
#'
#' @importFrom sf st_crs st_make_grid st_intersection st_union st_sf
#'
#' @export
ar_tessellate <- function(.data, shape = "square", size = 1){

  # error for wrong or missing data
  if(missing(.data) || !"sf" %in% class(.data)){
    stop("An sf object must be specified for `.data`")
  }

  # error for shape
  if(!shape %in% c("square", "hexagon")){
    stop("The shape argument must be one of 'square' or 'hexagon'")
  }

  # find units of object
  units <- sf::st_crs(.data, parameters = TRUE)
  units <- units$units_gdal

  # error if not projected units
  if(units == "degree"){
    stop("Data must be projected in order to tessellate")
  }

  # error if unrecognized units
  if(!units %in% c("Foot_US","feet", "Feet", "Meter", "meter", "Metre", "metre")){
    stop("Unit '", units ,"' of projection is unrecognized or not yet implemented")
  }

  # create cellsize vector, at 1 sq KM
  cellsize <- switch (units,
                      "feet"=,"Feet"=,
                      "Foot_US" = c(3280.84,3280.84),
                      "meter"=,"Metre"=,"metre"=,
                      "Meter" = c(1000, 1000)
              )

  # add scaling factor
  cellsize <- cellsize * size

  # convert shape argument
  shape <- switch(shape, "square" = TRUE, "hexagon" = FALSE)

  # unionize original boundary
  suppressWarnings(boundary <- sf::st_union(.data))

  # make tessellation, clip to original boundary and convert to sf/df
  grid <- sf::st_make_grid(.data, cellsize, square = shape)
  tess <- sf::st_intersection(grid, boundary)

  # coerce to sf from sfc


  tess <- sf::st_sf(tess)


  return(tess)
}

context("test ar_tessellate function")

# load test data ------------------------------------------------

data(ar_stl_wards, package = "areal")

# unprojected data
unproj <- sf::st_transform(ar_stl_wards, 4326)
# non-sf data
nonsf <- ar_stl_wards
sf::st_geometry(nonsf) <- NULL
# output
out <- ar_tessellate(ar_stl_wards)

# test errors ------------------------------------------------

test_that("errors with missing or non-sf data", {
  expect_error(ar_tessellate(), "An sf object must be specified for `.data`")
  expect_error(ar_tessellate(nonsf),"An sf object must be specified for `.data`")
})

test_that("errors for invalid shape", {
  expect_error(ar_tessellate(ar_stl_wards, shape = "ham"), "The shape argument must be one of 'square' or 'hexagon'")
})

test_that("errors for unprojected data", {
  expect_error(ar_tessellate(unproj), "Data must be projected in order to tessellate")
})

# test output ------------------------------------------------

test_that("outputs object of class sf", {
  expect_s3_class(out, "sf")
})

#' Verify Correct Extensive-Sum Interpolation
#'
#' @details \code{aw_verify} ensures that the sum of the resulting interpolated
#'     value is equal to the sum of the original source value. This functionality
#'     only works for interpolations that are extensive and use the \code{sum}
#'     approach to calculating areal weights.
#'
#' @usage aw_verify(source, sourceValue, result, resultValue)
#'
#' @param source A \code{sf} object with data to be interpolated
#' @param sourceValue A column within \code{source} to be interpolated
#' @param result A \code{sf} object with interpolated data
#' @param resultValue A column within \code{result} with the interpolated values
#'
#' @return A logical scalar; if \code{TRUE}, these two values are equal.
#'
#' @examples
#' result <- aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_race, sid = GEOID,
#'                          weight = "sum", output = "tibble", extensive = "TOTAL_E")
#'
#' aw_verify(source = ar_stl_race, sourceValue = TOTAL_E, result = result, resultValue = TOTAL_E)
#'
#' @importFrom glue glue
#'
#' @export
aw_verify <- function(source, sourceValue, result, resultValue){

  # save parameters to list
  paramList <- as.list(match.call())

  # check for missing parameters
  if (missing(source)) {
    stop("A sf object containing source data must be specified for the 'source' argument.")
  }

  if (missing(sourceValue)) {
    stop("A variable name must be specified for the 'sourceValue' argument.")
  }

  if (missing(result)) {
    stop("A sf object containing interpolated data must be specified for the 'result' argument.")
  }


  if (missing(resultValue)) {
    stop("A variable name must be specified for the 'resultValue' argument.")
  }

  # nse
  if (!is.character(paramList$sourceValue)) {
    sourceValueQ <- rlang::enquo(sourceValue)
  } else if (is.character(paramList$sourceValue)) {
    sourceValueQ <- rlang::quo(!! rlang::sym(sourceValue))
  }

  sourceValueQN <- rlang::quo_name(rlang::enquo(sourceValue))

  if (!is.character(paramList$resultValue)) {
    resultValueQ <- rlang::enquo(resultValue)
  } else if (is.character(paramList$resultValue)) {
    resultValueQ <- rlang::quo(!! rlang::sym(resultValue))
  }

  resultValueQN <- rlang::quo_name(rlang::enquo(resultValue))

  # check variables
  if(!!sourceValueQN %in% colnames(source) == FALSE) {
    stop(glue::glue("Variable '{var}', given for the source value, cannot be found in the given source object. Make sure only one existing variable is given.",
                    var = sourceValueQN))
  }

  if(!!resultValueQN %in% colnames(result) == FALSE) {
    stop(glue::glue("Variable '{var}', given for the result value, cannot be found in the given result object. Make sure only one existing variable is given.",
                    var = resultValueQN))
  }

  # store sum of original and interpolated values
  sourceSum <- sum(source[[sourceValueQN]])
  resultSum <- sum(result[[resultValueQN]])

  # logic test
  out <- sourceSum == resultSum

  # return output
  return(out)

}

context("test aw_verify function")

# load test data ------------------------------------------------

# target data
data(ar_stl_wards, package = "areal")

# source data
data(ar_stl_race, package = "areal")

# interpolate data
totalResult1 <- aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_race, sid = GEOID,
                               weight = "sum", output = "sf", extensive = "TOTAL_E")

totalResult2 <- aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_race, sid = GEOID,
                              weight = "total", output = "sf", extensive = "TOTAL_E")

# test errors ------------------------------------------------

test_that("errors with missing objects", {
  expect_error(aw_verify(source = ham, sourceValue = TOTAL_E, result = totalResult1, resultValue = TOTAL_E),
               "object 'ham' not found")
  expect_error(aw_verify(source = ar_stl_race, sourceValue = TOTAL_E, result = ham, resultValue = TOTAL_E),
               "object 'ham' not found")
})

test_that("errors with missing parameters", {
  expect_error(aw_verify(sourceValue = TOTAL_E, result = totalResult1, resultValue = TOTAL_E),
               "A sf object containing source data must be specified for the 'source' argument.")
  expect_error(aw_verify(source = ar_stl_race, result = totalResult1, resultValue = TOTAL_E),
               "A variable name must be specified for the 'sourceValue' argument.")
  expect_error(aw_verify(source = ar_stl_race, sourceValue = TOTAL_E, resultValue = TOTAL_E),
               "A sf object containing interpolated data must be specified for the 'result' argument.")
  expect_error(aw_verify(source = ar_stl_race, sourceValue = TOTAL_E, result = totalResult1),
               "A variable name must be specified for the 'resultValue' argument.")
})

test_that("errors with misspecified parameters", {
  expect_error(aw_verify(source = ar_stl_race, sourceValue = c("TOTAL_E", "WHITE_E"),
                         result = totalResult1, resultValue = TOTAL_E),
               "Variable 'c\\(\"TOTAL_E\", \"WHITE_E\"\\)', given for the source value, cannot be found in the given source object. Make sure only one existing variable is given.")
  expect_error(aw_verify(source = ar_stl_race, sourceValue = TOTAL_E,
                         result = totalResult1, resultValue = c("TOTAL_E", "WHITE_E")),
               "Variable 'c\\(\"TOTAL_E\", \"WHITE_E\"\\)', given for the result value, cannot be found in the given result object. Make sure only one existing variable is given.")
})

# test result ------------------------------------------------

test_that("correctly returns logic test results", {
  expect_equal(aw_verify(source = ar_stl_race, sourceValue = "TOTAL_E",
                         result = totalResult2, resultValue = "TOTAL_E"), FALSE)
})

#' Calculate Estimated Population
#'
#' @description \code{aw_calculate} multiplies the given \code{value} by the area weight. This
#'     is the fourth step in the interpolation process after \link{aw_weight}.
#'
#' @usage aw_calculate(.data, value, areaWeight, newVar)
#'
#' @param .data A given intersected dataset
#' @param value A column within \code{source} to be interpolated
#' @param areaWeight The name of the variable containing area weight per feature
#' @param newVar Optional; a new field name to store the interpolated value in. If not specified,
#'     the \code{value} argument will be used as the new field name.
#'
#' @return An intersected file of class sf with a new field of interest recalculated with area weight
#'
#' @examples
#' library(dplyr)
#'
#' race <- select(ar_stl_race, GEOID, TOTAL_E)
#' wards <- select(ar_stl_wards, WARD)
#'
#' wards %>%
#'     aw_intersect(source = race, areaVar = "area") %>%
#'     aw_total(source = race, id = GEOID, areaVar = "area", totalVar = "totalArea",
#'              weight = "sum", type = "extensive") %>%
#'     aw_weight(areaVar = "area", totalVar = "totalArea", areaWeight = "areaWeight") -> intersect
#'
#' aw_calculate(intersect, value = "TOTAL_E", areaWeight = "areaWeight")
#'
#' @importFrom dplyr mutate
#' @importFrom glue glue
#' @importFrom rlang :=
#' @importFrom rlang enquo
#' @importFrom rlang quo
#' @importFrom rlang quo_name
#' @importFrom rlang sym
#'
#' @export
aw_calculate <- function(.data, value, areaWeight, newVar){

  # save parameters to list
  paramList <- as.list(match.call())

  # check for missing parameters
  if (missing(.data)) {
    stop("A sf object containing intersected data must be specified for the '.data' argument.")
  }

  if (missing(value)) {
    stop("A variable name must be specified for the 'value' argument.")
  }

  if (missing(areaWeight)) {
    stop("A variable name must be specified for the 'areaWeight' argument.")
  }

  # nse
  if (!is.character(paramList$areaWeight)) {
    areaWeightQ <- rlang::enquo(areaWeight)
  } else if (is.character(paramList$areaWeight)) {
    areaWeightQ <- rlang::quo(!! rlang::sym(areaWeight))
  }

  areaWeightQN <- rlang::quo_name(rlang::enquo(areaWeight))

  if (!is.character(paramList$value)) {
    valsQ <- rlang::enquo(value)
  } else if (is.character(paramList$value)) {
    valsQ <- rlang::quo(!! rlang::sym(value))
  }

  valsQN <- rlang::quo_name(rlang::enquo(value))

  if (missing(newVar)) {
    newFieldQN <- valsQN
  } else if (!missing(newVar)){
    newFieldQN <- rlang::quo_name(rlang::enquo(newVar))
  }

  # check variables
  if(!!valsQN %in% colnames(.data) == FALSE) {
    stop(glue::glue("Variable '{var}', given for the value, cannot be found in the given intersected object.",
                    var = valsQN))
  }

  if (!!areaWeightQN != "...areaWeight"){

    if(!!areaWeightQN %in% colnames(.data) == FALSE) {
      stop(glue::glue("Variable '{var}', given for the area weight, cannot be found in the given intersected object.",
                      var = areaWeightQN))
    }

  }

  # recalculate source values of interest using area weight and assign as new field
  out <- dplyr::mutate(.data, !!newFieldQN := !!valsQ * !!areaWeightQ)

  # return output
  return(out)

}

#' Asthma in St. Louis by Census Tract, 2017
#'
#' @description A simple features data set containing the geometry and asthma estimates
#'     from the Centers for Disease Control for St. Louis.
#'
#' @docType data
#'
#' @usage data(ar_stl_asthma)
#'
#' @format A data frame with 106 rows and 24 variables:
#' \describe{
#'   \item{GEOID}{full GEOID string}
#'   \item{STATEFP}{state FIPS code}
#'   \item{COUNTYFP}{county FIPS code}
#'   \item{TRACTCE}{tract FIPS code}
#'   \item{NAMELSAD}{tract name}
#'   \item{ALAND}{area of tract land, square meters}
#'   \item{AWATER}{area of tract water, square meters}
#'   \item{ASTHMA}{percent of residents with current asthma diagnosis, estimated}
#'   \item{geometry}{simple features geometry}
#'   }
#'
#' @source Centers for Disease Control's 500 Cities Data
#'
#' @examples
#' str(ar_stl_asthma)
#' head(ar_stl_asthma)
#' summary(ar_stl_asthma$ASTHMA)
#'
"ar_stl_asthma"

#' Preview Areal Weights
#'
#' @description Provides a preview of the weight options for areal weighted interpolation.
#'     This can be useful for selecting the final specification for \code{aw_interpolate}
#'     without having to construct a pipeline of all of the subfunctions manually.
#'
#' @usage aw_preview_weights(.data, tid, source, sid, type)
#'
#' @param .data A \code{sf} object that data should be interpolated to (this is referred
#'     to as the \code{target} elsewhere in the package).
#' @param tid A unique identification number within \code{target}
#' @param source A \code{sf} object with data to be interpolated
#' @param sid A unique identification number within \code{source}
#' @param type One of either \code{"extensive"} (if the data are spatitally extensive e.g.
#'     population counts), \code{"intensive"} (if the data are spatially intensive e.g.
#'     population density), or \code{"mixed"} (if the data include both extensive and
#'     intensive values). If \code{"extensive"}, the sum is returned for the interpolated
#'     value. If \code{"intensive"}, the mean is returned for the interpolated value.
#'     If \code{"mixed"}, vectors named \code{"extensive"} and \code{"intensive"} containing
#'     the relevant variable names should be specified in the dots.
#'
#' @return A tibble with the areal weights that would be used for interpolation if \code{type}
#'     is either \code{"extensive"} or \code{"intensive"}. If it is mixed, two tibbles (one for
#'     \code{"extensive"} and one for \code{"intensive"}) are returned as a list.
#'
#' @examples
#' aw_preview_weights(ar_stl_wards, tid = WARD, source = ar_stl_race, sid = GEOID,
#'                    type = "extensive")
#'
#' aw_preview_weights(ar_stl_wards, tid = WARD, source = ar_stl_asthma, sid = GEOID,
#'                    type = "intensive")
#'
#' @importFrom dplyr arrange group_by left_join summarize
#' @importFrom glue glue
#' @importFrom rlang enquo
#' @importFrom rlang quo
#' @importFrom rlang quo_name
#' @importFrom rlang sym
#' @importFrom sf st_geometry
#'
#' @export
aw_preview_weights <- function(.data, tid, source, sid, type){

  # save parameters to list
  paramList <- as.list(match.call())

  # check for missing parameters
  if (missing(.data)) {
    stop("A sf object containing target data must be specified for the '.data' argument.")
  }

  if (missing(tid)) {
    stop("A variable name must be specified for the 'tid' argument.")
  }

  if (missing(source)) {
    stop("A sf object must be specified for the 'source' argument.")
  }

  if (missing(sid)) {
    stop("A variable name must be specified for the 'sid' argument.")
  }

  if (missing(type)) {
    stop("An interpolation type must be specified for the 'type' argument.")
  }

  # check for misspecified parameters
  if (type %in% c("extensive", "intensive", "mixed") == FALSE){
    stop(glue::glue("The given interpolation type '{var}' is not valid. 'type' must be one of 'extensive', 'intensive', or 'mixed'.",
                    var = type))
  }

  # nse
  if (!is.character(paramList$sid)) {
    sidQ <- rlang::enquo(sid)
  } else if (is.character(paramList$sid)) {
    sidQ <- rlang::quo(!! rlang::sym(sid))
  }

  sidQN <- rlang::quo_name(rlang::enquo(sid))

  if (!is.character(paramList$tid)) {
    tidQ <- rlang::enquo(tid)
  } else if (is.character(paramList$tid)) {
    tidQ <- rlang::quo(!! rlang::sym(tid))
  }

  tidQN <- rlang::quo_name(rlang::enquo(tid))

  # check variables
  if(!!sidQN %in% colnames(source) == FALSE) {
    stop(glue::glue("Variable '{var}', given for the source ID ('sid'), cannot be found in the given source object.",
                    var = sidQN))
  }

  if(!!tidQN %in% colnames(.data) == FALSE) {
    stop(glue::glue("Variable '{var}', given for the target ID ('tid'), cannot be found in the given target object.",
                    var = tidQN))
  }

  # validate source and target data
  if (aw_validate_preview(source = source, target = .data) == FALSE){

    stop("Data validation failed. Use ar_validate with verbose = TRUE to identify concerns.")

  }

  # strip source and target dataframes
  sourceS <- aw_strip_df(source, id = sidQN)
  targetS <- aw_strip_df(.data, id = tidQN)

  # caclulate extensive weights
  if (type == "extensive" | type == "mixed"){

    sum <- aw_calculate_weight(targetS, source = sourceS, id = !!sidQ, item = "extensive_sum")
    total <- aw_calculate_weight(targetS, source = sourceS, id = !!sidQ, item = "extensive_total")
    exOut <- dplyr::left_join(sum, total, by = sidQN)

  }

  if (type == "intensive" | type == "mixed"){

    inOut <- aw_calculate_weight(targetS, source = sourceS, id = !!tidQ, item = "intensive")

  }

  # create output
  if (type == "extensive"){

    out <- exOut

  } else if (type == "intensive"){

    out <- inOut

  } else if (type == "mixed"){

    out <- list("extensive" = exOut, "intensive" = inOut)

  }

  # return output
  return(out)

}


# Caclulate Weights
#
# @description Subfunction of aw_preview_weight for calculating individual weights
#
# @param .data A \code{sf} object that data should be interpolated to (this is referred
#     to as the \code{target} elsewhere in the package).
# @param source A \code{sf} object with data to be interpolated
# @param id A unique identification number in either the source or target data
# @param item One of \code{"extensive_sum"}, \code{"extensive_total"}, or \code{"intensive"}
#
aw_calculate_weight <- function(.data, source, id, item){

  # save parameters to list
  paramList <- as.list(match.call())

  # global binding
  ...areaWeight = NULL

  # nse
  idQ <- rlang::enquo(id)

  # create type and weight from item
  if (item == "extensive_sum"){

    type <- "extensive"
    weight <- "sum"
    newVar <- "extensiveSum"

  } else if (item == "extensive_total"){

    type <- "extensive"
    weight <- "total"
    newVar <- "extensiveTotal"

  } else if (item == "intensive"){

    type <- "intensive"
    weight <- "sum"
    newVar <- "intensive"

  }

  # caclulate weight
  .data %>%
    aw_intersect(source = source, areaVar = "...area") %>%
    aw_total(source = source, id = !!idQ, areaVar = "...area", totalVar = "...totalArea",
           type = type, weight = weight) %>%
    aw_weight(areaVar = "...area", totalVar = "...totalArea", areaWeight = "...areaWeight") -> result

  # remove geometry
  sf::st_geometry(result) <- NULL

  # summarize
  result %>%
    dplyr::group_by(!!idQ) %>%
    dplyr::summarize(!!newVar := sum(...areaWeight)) %>%
    dplyr::arrange(!!idQ) -> out

  # return output
  return(out)

}

context("test aw_preview_weights function")

# load test data ------------------------------------------------

# target data
data(ar_stl_wards, package = "areal")

# source data
data(ar_stl_race, package = "areal")
data(ar_stl_asthma, package = "areal")

## create modified asthma data
ar_stl_asthma %>%
  dplyr::select(-STATEFP, -COUNTYFP, -TRACTCE, -NAMELSAD, -ALAND, -AWATER) -> asthma

## create combined data
### remove sf geometry
race <- ar_stl_race
sf::st_geometry(race) <- NULL

### create combined data
race %>%
  dplyr::select(GEOID, TOTAL_E, WHITE_E, BLACK_E) %>%
  dplyr::left_join(asthma, ., by = "GEOID") -> combinedData

# test errors ------------------------------------------------

test_that("errors with missing objects", {
  expect_error(aw_preview_weights(ham, tid = WARD, source = ar_stl_race, sid = GEOID, type = "extensive"),
               "object 'ham' not found")
  expect_error(aw_preview_weights(ar_stl_wards, tid = WARD, source = ham, sid = GEOID, type = "extensive"),
               "object 'ham' not found")
})

test_that("errors with missing parameters", {
  expect_error(aw_preview_weights(tid = WARD, source = ar_stl_race, sid = GEOID, type = "extensive"),
               "A sf object containing target data must be specified for the '.data' argument.")
  expect_error(aw_preview_weights(ar_stl_wards, source = ar_stl_race, sid = GEOID, type = "extensive"),
               "A variable name must be specified for the 'tid' argument.")
  expect_error(aw_preview_weights(ar_stl_wards, tid = WARD, sid = GEOID, type = "extensive"),
               "A sf object must be specified for the 'source' argument.")
  expect_error(aw_preview_weights(ar_stl_wards, tid = WARD, source = ar_stl_race, type = "extensive"),
               "A variable name must be specified for the 'sid' argument.")
  expect_error(aw_preview_weights(ar_stl_wards, tid = WARD, source = ar_stl_race, sid = GEOID),
               "An interpolation type must be specified for the 'type' argument.")
})

test_that("errors with misspecified parameters", {
  expect_error(aw_preview_weights(ar_stl_wards, tid = WARD, source = ar_stl_race, sid = GEOID, type = "ham"),
               "The given interpolation type 'ham' is not valid. 'type' must be one of 'extensive', 'intensive', or 'mixed'.")
  expect_error(aw_preview_weights(ar_stl_wards, tid = WARD, source = ar_stl_race, sid = ham, type = "extensive"),
               "Variable 'ham', given for the source ID \\('sid'\\), cannot be found in the given source object.")
  expect_error(aw_preview_weights(ar_stl_wards, tid = ham, source = ar_stl_race, sid = GEOID, type = "extensive"),
               "Variable 'ham', given for the target ID \\('tid'\\), cannot be found in the given target object.")
})

# test inputs ------------------------------------------------

test_that("correctly specified functions execute without error", {
  expect_error(aw_preview_weights(ar_stl_wards, tid = WARD, source = ar_stl_race, sid = GEOID, type = "extensive"), NA)
  expect_error(aw_preview_weights(ar_stl_wards, tid = "WARD", source = ar_stl_asthma, sid = "GEOID", type = "intensive"), NA)
  expect_error(aw_preview_weights(ar_stl_wards, tid = WARD, source = combinedData, sid = GEOID, type = "mixed"), NA)
})

# test validation failures ------------------------------------------------

wardsdf <- ar_stl_wards
sf::st_geometry(wardsdf) <- NULL

racedf <- ar_stl_race
sf::st_geometry(racedf) <- NULL

wards83 <- sf::st_transform(ar_stl_wards, crs = 4269)
race83 <- sf::st_transform(ar_stl_race, crs = 4269)

wards_points <- suppressWarnings(sf::st_centroid(ar_stl_wards))

test_that("validation result is false", {
  expect_error(aw_preview_weights(wardsdf, tid = WARD, source = ar_stl_race, sid = GEOID, type = "extensive"),
               "Data validation failed. Use ar_validate with verbose = TRUE to identify concerns.")
  expect_error(aw_preview_weights(ar_stl_wards, tid = WARD, source = racedf, sid = GEOID, type = "extensive"),
               "Data validation failed. Use ar_validate with verbose = TRUE to identify concerns.")
  expect_error(aw_preview_weights(wards83, tid = WARD, source = ar_stl_race, sid = GEOID, type = "extensive"),
               "Data validation failed. Use ar_validate with verbose = TRUE to identify concerns.")
  expect_error(aw_preview_weights(ar_stl_wards, tid = WARD, source = race83, sid = GEOID, type = "extensive"),
               "Data validation failed. Use ar_validate with verbose = TRUE to identify concerns.")
  expect_error(aw_preview_weights(wards_points, tid = WARD, source = ar_stl_race, sid = GEOID, type = "extensive"),
               "Data validation failed. Use ar_validate with verbose = TRUE to identify concerns.")
})

# test output ------------------------------------------------

extensive <- aw_preview_weights(ar_stl_wards, tid = WARD, source = ar_stl_race, sid = GEOID, type = "extensive")
intensive <- aw_preview_weights(ar_stl_wards, tid = WARD, source = ar_stl_asthma, sid = GEOID, type = "intensive")
mixed <- aw_preview_weights(ar_stl_wards, tid = WARD, source = combinedData, sid = GEOID, type = "mixed")

test_that("objects created as expected", {
  expect_equal(class(extensive)[1], "tbl_df")
  expect_equal(class(intensive)[1], "tbl_df")
  expect_equal(class(mixed)[1], "list")
  expect_equal(class(mixed$extensive)[1], "tbl_df")
  expect_equal(class(mixed$intensive)[1], "tbl_df")
})

test_that("weights calculated as expected", {
  expect_equal(names(extensive), c("GEOID", "extensiveSum", "extensiveTotal"))
  expect_equal(names(mixed$extensive), c("GEOID", "extensiveSum", "extensiveTotal"))
  expect_equal(names(intensive), c("WARD", "intensive"))
  expect_equal(names(mixed$intensive), c("WARD", "intensive"))
})

#' Interpolate Values
#'
#' @description This is the core function within the package for areal weighted
#'     interpolation. It validates both data sources before interpolating one or more
#'     listed values from the source data into the target data.
#'
#' @usage aw_interpolate(.data, tid, source, sid, weight = "sum", output = "sf", extensive,
#'     intensive)
#'
#' @details Areal weighted interpolation can be used for generating demographic
#'     estimates for overlapping but incongruent polygon features. It assumes that
#'     individual members of a population are evenly dispersed within the source features
#'     (an assumption not likely to hold in the real world). It also functions best
#'     when data are in a projected coordinate system, like the UTM coordinate system.
#'
#' @param .data A \code{sf} object that data should be interpolated to (this is referred
#'     to as the \code{target} elsewhere in the package).
#' @param tid A unique identification number within \code{target}
#' @param source A \code{sf} object with data to be interpolated
#' @param sid A unique identification number within \code{source}
#' @param weight For \code{"extensive"} interpolations, should be either \code{"total"} or
#'     \code{"sum"}. For \code{"intensive"} interpolations, should be \code{"sum"}. For mixed
#'     interpolations, this will only impact the calculation of the extensive variables.
#' @param output One of either \code{"sf"} or \code{"tibble"}
#' @param extensive A vector of quoted variable names to be treated as spatially extensive
#'     (e.g. population counts); optional if \code{intensive} is specified
#' @param intensive A vector of quoted variable names to be treated as spatially intensive
#'     (e.g. population density); optional if \code{extensive} is specified
#'
#' @return A \code{sf} object or a \code{tibble} with the value or values interpolated into
#'     the \code{target} data.
#'
#' @seealso \link{c}
#'
#' @examples
#' aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_race, sid = GEOID, weight = "sum",
#'     output = "sf", extensive = "TOTAL_E")
#'
#' aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_asthma, sid = GEOID, weight = "sum",
#'     output = "tibble", intensive = "ASTHMA")
#'
#' @importFrom dplyr as_tibble
#' @importFrom dplyr left_join
#' @importFrom dplyr rename
#' @importFrom dplyr select
#' @importFrom glue glue
#' @importFrom rlang enquo
#' @importFrom rlang quo
#' @importFrom rlang quo_name
#' @importFrom rlang sym
#' @importFrom sf st_geometry
#'
#' @export
aw_interpolate <- function(.data, tid, source, sid, weight = "sum", output = "sf", extensive, intensive){

  # save parameters to list
  paramList <- as.list(match.call())

  # check for missing parameters
  if (missing(.data)) {
    stop("A sf object containing target data must be specified for the '.data' argument.")
  }

  if (missing(tid)) {
    stop("A variable name must be specified for the 'tid' argument.")
  }

  if (missing(source)) {
    stop("A sf object must be specified for the 'source' argument.")
  }

  if (missing(sid)) {
    stop("A variable name must be specified for the 'sid' argument.")
  }

  # determine extensive and intensive
  if (missing(extensive) & missing(intensive)){
    stop("Either 'extensive' or 'intenstive' must be specified with an accompanying list of variables to interpolate.")
  }

  if (missing(intensive) & !missing(extensive)){
    type <- "extensive"
  } else if (!missing(intensive) & missing(extensive)){
    type <- "intensive"
  } else if (!missing(intensive) & !missing(extensive)){
    type <- "mixed"
  }

  # check for misspecified parameters
  if (weight %in% c("sum", "total") == FALSE){
    stop(glue::glue("The given weight type '{var}' is not valid. 'weight' must be either 'sum' or 'total'.",
                    var = weight))
  }

  if (type == "intensive" & weight == "total"){
    stop("Spatially intensive interpolations should be caclulated using 'sum' for 'weight'.")
  }

  if (output %in% c("sf", "tibble") == FALSE){
    stop(glue::glue("The given output type '{var}' is not valid. 'output' must be either 'sf' or 'tibble'.",
                    var = output))
  }

  # nse
  if (!is.character(paramList$sid)) {
    sidQ <- rlang::enquo(sid)
  } else if (is.character(paramList$sid)) {
    sidQ <- rlang::quo(!! rlang::sym(sid))
  }

  sidQN <- rlang::quo_name(rlang::enquo(sid))

  if (!is.character(paramList$tid)) {
    tidQ <- rlang::enquo(tid)
  } else if (is.character(paramList$tid)) {
    tidQ <- rlang::quo(!! rlang::sym(tid))
  }

  tidQN <- rlang::quo_name(rlang::enquo(tid))

  # check variables
  if(!!sidQN %in% colnames(source) == FALSE) {
    stop(glue::glue("Variable '{var}', given for the source ID ('sid'), cannot be found in the given source object.",
                    var = sidQN))
  }

  if(!!tidQN %in% colnames(.data) == FALSE) {
    stop(glue::glue("Variable '{var}', given for the target ID ('tid'), cannot be found in the given target object.",
                    var = tidQN))
  }

  # check for matching tid and sid variable names
  if (tidQN == sidQN){

    # store conflict indicator
    nameConflict <- TRUE

    # store original tid name for later
    tidOrig <- tidQN

    # rename tid to ...tid
    .data <- dplyr::rename(.data, ...tid = !!tidQN)
    tidQN <- "...tid"
    tidQ <- rlang::quo(!! rlang::sym(tidQN))

  } else {

    # store conflict indicator
    nameConflict <- FALSE

  }

  # create variable lists
  if (type == "extensive"){
    vars <- extensive
  } else if (type == "intensive"){
    vars <- intensive
  } else if (type == "mixed"){
    vars <- c(extensive, intensive)
  }

  # validate source and target data
  if (ar_validate(source = source, target = .data, varList = vars, method = "aw") == FALSE){
    stop("Data validation failed. Use ar_validate with verbose = TRUE to identify concerns.")
  }

  # call aw_interpolater
  if ((type == "extensive" | type == "intensive") & length(vars) == 1) {

    # nse
    valueQ <- rlang::quo(!! rlang::sym(vars))

    # interpolate
    data <- aw_interpolate_single(source = source, sid = !!sidQ, value = !!valueQ, target = .data,
                                 tid = !!tidQ, type = type, weight = weight)

  } else if ((type == "extensive" | type == "intensive") & length(vars) > 1) {

    # interpolate
    data <- aw_interpolate_multiple(source = source, sid = !!sidQ, values = vars, target = .data,
                                 tid = !!tidQ, type = type, weight = weight)

  } else if (type == "mixed"){

    # conduct spatially extensive interpolations
    if (length(extensive) == 1){

      # nse
      valueQ <- rlang::quo(!! rlang::sym(extensive))

      # interpolate
      exresults <- aw_interpolate_single(source = source, sid = !!sidQ, value = !!valueQ, target = .data,
                                   tid = !!tidQ, type = "extensive", weight = weight)

    } else if (length(extensive) > 1){

      # interpolate
      exresults <- aw_interpolate_multiple(source = source, sid = !!sidQ, values = extensive, target = .data,
                                      tid = !!tidQ, type = "extensive", weight = weight)

    }

    # conduct spatially intensive interpolations
    if (length(intensive) == 1){

      # nse
      valueQ <- rlang::quo(!! rlang::sym(intensive))

      # interpolate
      inresults <- aw_interpolate_single(source = source, sid = !!sidQ, value = !!valueQ, target = .data,
                                         tid = !!tidQ, type = "intensive", weight = "sum")

    } else if (length(intensive) > 1){

      # interpolate
      inresults <- aw_interpolate_multiple(source = source, sid = !!sidQ, values = intensive, target = .data,
                                           tid = !!tidQ, type = "intensive", weight = "sum")

    }

    # combine spatially extensive and intensive data
    data <- dplyr::left_join(exresults, inresults, by = tidQN)

  }

  # structure output
  if (output == "sf"){

    # left join with target data
    out <- dplyr::left_join(.data, data, by = tidQN)

  } else if (output == "tibble"){

    # left join with target data
    data <- dplyr::left_join(.data, data, by = tidQN)

    # remove geometry
    sf::st_geometry(data) <- NULL

    # convert to tibble
    out <- dplyr::as_tibble(data)

  }

  # rename tid
  if (nameConflict == TRUE){
    out <- dplyr::rename(out, !!tidOrig := !!tidQN)
  }

  # return output
  return(out)

}

# Intermediate Function - Single Value
#
# @description Intermediate function called when there is only one variable to be interpolated.
#     This is used to simplify the code for \code{aw_interpolate}.
#
# @param source A \code{sf} object with data to be interpolated
# @param sid A unique identification number within \code{source}
# @param value A column within \code{source} to be interpolated
# @param target A \code{sf} object that data should be interpolated to
# @param tid A unique identification number within \code{target}
# @param type One of either \code{"extensive"} (if the data are spatially extensive e.g.
#     population counts), \code{"intensive"} (if the data are spatially intensive e.g.
#     population density), or \code{"mixed"} (if the data include both extensive and
#     intensive values). If \code{"extensive"}, the sum is returned for the interpolated
#     value. If \code{"intensive"}, the mean is returned for the interpolated value.
#     If \code{"mixed"}, vectors named \code{"extensive"} and \code{"intensive"} containing
#     the relevant variable names should be specified in the dots.
# @param weight For \code{"extensive"} interpolations; should be either \code{"total"} or
#     \code{"sum"}.
#
# @return A tibble with interpolated data, ready for final merge with \code{target}.
#
aw_interpolate_single <- function(source, sid, value, target, tid, type, weight){

  # save parameters to list
  paramList <- as.list(match.call())

  # nse
  sidQ <- rlang::enquo(sid)
  sidQN <- rlang::quo_name(rlang::enquo(sidQ))

  valueQ <- rlang::enquo(value)
  valueQN <- rlang::quo_name(rlang::enquo(value))

  tidQ <- rlang::enquo(tid)
  tidQN <- rlang::quo_name(rlang::enquo(tidQ))

  # strip source and target dataframes
  sourceS <- aw_strip_df(source, id = sidQN, value = valueQN)
  targetS <- aw_strip_df(target, id = tidQN)

  # interpolate
  out <- aw_interpolater(source = sourceS, sid = !!sidQ, value = !!valueQ, target = targetS,
                         tid = !!tidQ, type = type, weight = weight)

  # return output
  return(out)

}

# Intermediate Function - Multiple Values (iteration)
#
# @description Intermediate function called when are more than one variables to be interpolated.
#     This is used to simplify the code for \code{aw_interpolate}.
#
# @param source A \code{sf} object with data to be interpolated
# @param sid A unique identification number within \code{source}
# @param values A vector of columns within \code{source} to be interpolated
# @param target A \code{sf} object that data should be interpolated to
# @param tid A unique identification number within \code{target}
# @param type One of either \code{"extensive"} (if the data are spatially extensive e.g.
#     population counts), \code{"intensive"} (if the data are spatially intensive e.g.
#     population density), or \code{"mixed"} (if the data include both extensive and
#     intensive values). If \code{"extensive"}, the sum is returned for the interpolated
#     value. If \code{"intensive"}, the mean is returned for the interpolated value.
#     If \code{"mixed"}, vectors named \code{"extensive"} and \code{"intensive"} containing
#     the relevant variable names should be specified in the dots.
# @param weight For \code{"extensive"} interpolations; should be either \code{"total"} or
#     \code{"sum"}.
#
# @importFrom dplyr %>%
# @importFrom dplyr bind_cols
# @importFrom dplyr one_of
# @importFrom dplyr select
# @importFrom purrr imap
# @importFrom purrr map
# @importFrom purrr reduce
# @importFrom rlang enquo
# @importFrom rlang quo_name
#
# @return A tibble with interpolated data, ready for final merge with \code{target}.
#
aw_interpolate_multiple <- function(source, sid, values, target, tid, type, weight){

  # save parameters to list
  paramList <- as.list(match.call())

  # nse
  sidQ <- rlang::enquo(sid)
  sidQN <- rlang::quo_name(rlang::enquo(sidQ))

  tidQ <- rlang::enquo(tid)
  tidQN <- rlang::quo_name(rlang::enquo(tidQ))

  # create column list
  colNames <- c(tidQN, values)

  # strip target dataframe
  targetS <- aw_strip_df(target, id = tidQN)

  # create list of sf objects
  values %>%
    split(values) %>%
    purrr::map(~ aw_strip_df(source, id = !!sidQ, value = .x)) %>%
    purrr::imap(~ aw_interpolater(source = .x, sid = !!sidQ, value = (!! rlang::quo(!! rlang::sym(.y))),
                                  target = targetS, tid = !!tidQ, type = type, weight = weight,
                                  multiple = TRUE)) %>%
    purrr::reduce(.f = dplyr::bind_cols) -> out

  # remove geometry
  sf::st_geometry(targetS) <- NULL

  # combine
  out <- dplyr::bind_cols(targetS, out)

  # return output
  return(out)

}

# Strip dataframe of all non-essential variables
#
# @description \code{aw_strip_df} is called by \code{aw_interpolate}. It
#     strips \code{sf} objects of nonessential variables but keeps
#     variables listed in parameters.
#
# @param .data A \code{sf} object
# @param id A given source id field
# @param value Optional; the variable that estimations will be based on
#
# @return A \code{sf} object with only the \code{id} and, if provided, the
#     \code{value} column as well.
#
aw_strip_df <- function(.data, id, value){

  # save parameters to list
  paramList <- as.list(match.call())

  # nse
  idQ <- rlang::enquo(id)

  # strip variables
  if (missing(value)){

    # strip all but id
    out <- dplyr::select(.data, !!idQ)

  } else {

    # additional nse for value
    valsQ <- rlang::enquo(value)

    # strip all but id and value
    out <- dplyr::select(.data, !!idQ, !!valsQ)

  }

  # return output
  return(out)

}

# Carry Out Interpolation
#
# @description \code{aw_interpolater} performs pipeline of interpolation specific
#     calculations with \code{aw_intersect}, \code{aw_total}, \code{aw_weight},
#     \code{aw_calculate}, and \code{aw_aggregate}. The interpolated total is then
#     verified against the total calculated from the source data using \code{aw_verify}.
#
# @param source A \code{sf} object with data to be interpolated
# @param sid A unique identification number within \code{source}
# @param value A column within \code{source} to be interpolated
# @param target A \code{sf} object that data should be interpolated to
# @param tid A unique identification number within \code{target}
# @param type One of either \code{"extensive"} (if the data are spatially extensive e.g.
#     population counts), \code{"intensive"} (if the data are spatially intensive e.g.
#     population density), or \code{"mixed"} (if the data include both extensive and
#     intensive values). If \code{"extensive"}, the sum is returned for the interpolated
#     value. If \code{"intensive"}, the mean is returned for the interpolated value.
#     If \code{"mixed"}, vectors named \code{"extensive"} and \code{"intensive"} containing
#     the relevant variable names should be specified in the dots.
# @param weight For \code{"extensive"} interpolations; should be either \code{"total"} or
#     \code{"sum"}.
#
# @return A \code{sf} object or tibble with \code{value} interpolated into
#    the \code{target} data.
#
aw_interpolater <- function(source, sid, value, target, tid, type, weight, multiple = FALSE) {

  # save parameters to list
  paramList <- as.list(match.call())

  # nse
  sidQ <- rlang::enquo(sid)

  valueQ <- rlang::enquo(value)
  valueQN <- rlang::quo_name(rlang::enquo(value))

  tidQ <- rlang::enquo(tid)

  # intersect data
  target %>%
    aw_intersect(source = source, areaVar = "...area") -> intersected

  # calculate total value for areal weight
  if (type == "extensive"){

    intersected %>%
      aw_total(source = source, id = !!sidQ, areaVar = "...area", totalVar = "...totalArea",
             type = "extensive", weight = weight) -> totaled

  } else if (type == "intensive"){

    intersected %>%
      aw_total(source = source, id = !!tidQ, areaVar = "...area", totalVar = "...totalArea",
             weight = weight, type = "intensive") -> totaled

  }

  # caclulate areal weight and estimated value; aggregate
  totaled %>%
    aw_weight(areaVar = "...area", totalVar = "...totalArea", areaWeight = "...areaWeight") %>%
    aw_calculate(value = !!valueQ, areaWeight = "...areaWeight") %>%
    aw_aggregate(target = target, tid = !!tidQ, interVar = !!valueQ) -> out

  # remove sf from output
  sf::st_geometry(out) <- NULL

  # fix multiples
  if (multiple == TRUE){
    out <- dplyr::select(out, !!valueQ)
  }

  # return target output
  return(out)

}

context("test aw_interpolate function")

# load test data ------------------------------------------------

# target data
data(ar_stl_wards, package = "areal")

# source data
data(ar_stl_race, package = "areal")
data(ar_stl_asthma, package = "areal")

## create modified asthma data
ar_stl_asthma %>%
  dplyr::select(-STATEFP, -COUNTYFP, -TRACTCE, -NAMELSAD, -ALAND, -AWATER) %>%
  dplyr::mutate(ASTHMA2 = ASTHMA/2) -> asthma

## create combined data
### remove sf geometry
race <- ar_stl_race
sf::st_geometry(race) <- NULL

### create combined data
race %>%
  dplyr::select(GEOID, TOTAL_E, WHITE_E, BLACK_E) %>%
  dplyr::left_join(asthma, ., by = "GEOID") -> combinedData

# create comparison data
load(system.file("testdata", "totalCompare1.rda", package = "areal", mustWork = TRUE))
totalCompare2 <- suppressWarnings(sf::st_interpolate_aw(ar_stl_race["TOTAL_E"], ar_stl_wards, extensive = TRUE))
asthmaCompare <- suppressWarnings(sf::st_interpolate_aw(ar_stl_asthma["ASTHMA"], ar_stl_wards, extensive = FALSE))

# test errors ------------------------------------------------

test_that("errors with missing objects", {
  expect_error(aw_interpolate(ham, tid = WARD, source = ar_stl_race, sid = GEOID,
                              weight = "sum", output = "sf", extensive = "TOTAL_E"),
               "object 'ham' not found")
  expect_error(aw_interpolate(ar_stl_wards, tid = WARD, source = ham, sid = GEOID,
                              weight = "sum", output = "sf", extensive = c("TOTAL_E", "WHITE_E")),
               "object 'ham' not found")
})

test_that("errors with missing parameters", {
  expect_error(aw_interpolate(tid = WARD, source = ar_stl_race, sid = GEOID,
                              weight = "sum", output = "sf", extensive = "TOTAL_E"),
               "A sf object containing target data must be specified for the '.data' argument.")
  expect_error(aw_interpolate(ar_stl_wards, source = ar_stl_race, sid = GEOID,
                              weight = "sum", output = "sf", extensive = "TOTAL_E"),
               "A variable name must be specified for the 'tid' argument.")
  expect_error(aw_interpolate(ar_stl_wards, tid = WARD, sid = GEOID,
                              weight = "sum", output = "sf", extensive = "TOTAL_E"),
               "A sf object must be specified for the 'source' argument.")
  expect_error(aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_race,
                              weight = "sum", output = "sf", extensive = "TOTAL_E"),
               "A variable name must be specified for the 'sid' argument.")
  expect_error(aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_race, sid = GEOID,
                              weight = "sum", output = "sf"),
               "Either 'extensive' or 'intenstive' must be specified with an accompanying list of variables to interpolate.")
})

test_that("errors with objects and id variables that do not exist", {
  expect_error(aw_interpolate(ar_stl_wards, tid = ham, source = ar_stl_race, sid = GEOID,
                              weight = "sum", output = "sf", extensive = "TOTAL_E"),
               "Variable 'ham', given for the target ID \\('tid'\\), cannot be found in the given target object.")
  expect_error(aw_interpolate(ar_stl_wards, tid = "ham", source = ar_stl_race, sid = GEOID,
                              weight = "sum", output = "sf", extensive = "TOTAL_E"),
               "Variable 'ham', given for the target ID \\('tid'\\), cannot be found in the given target object.")
  expect_error(aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_race, sid = ham,
                              weight = "sum", output = "sf", extensive = "TOTAL_E"),
               "Variable 'ham', given for the source ID \\('sid'\\), cannot be found in the given source object.")
  expect_error(aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_race, sid = "ham",
                              weight = "sum", output = "sf", extensive = "TOTAL_E"),
               "Variable 'ham', given for the source ID \\('sid'\\), cannot be found in the given source object.")
})

test_that("errors with weight and output", {
  expect_error(aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_race, sid = GEOID,
                              weight = "ham", output = "sf", extensive = "TOTAL_E"),
               "The given weight type 'ham' is not valid. 'weight' must be either 'sum' or 'total'.")
  expect_error(aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_asthma, sid = GEOID,
                              weight = "total", output = "sf", intensive = "ASTHMA"),
               "Spatially intensive interpolations should be caclulated using 'sum' for 'weight'.")
  expect_error(aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_race, sid = GEOID,
                              weight = "sum", output = "ham", extensive = "TOTAL_E"),
               "The given output type 'ham' is not valid. 'output' must be either 'sf' or 'tibble'.")
})

test_that("force data validation failure", {
  expect_error(aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_race, sid = GEOID,
                              weight = "sum", output = "sf", extensive = "HAM"),
               "Data validation failed. Use ar_validate with verbose = TRUE to identify concerns.")
})

# test results ------------------------------------------------

totalResult1 <- aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_race, sid = GEOID,
                               weight = "sum", output = "sf", extensive = "TOTAL_E")

totalResult2 <- aw_interpolate(ar_stl_wards, tid = "WARD", source = ar_stl_race, sid = "GEOID",
                               weight = "total", output = "tibble", extensive = "TOTAL_E")

asthmaResult <- aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_asthma, sid = GEOID,
                               output = "sf", weight = "sum", intensive = "ASTHMA")

mixedResult <- aw_interpolate(ar_stl_wards, tid = WARD, source = combinedData, sid = "GEOID",
                              weight = "sum", output = "tibble", extensive = "TOTAL_E",
                              intensive = "ASTHMA")

test_that("interpolated values are equal", {
  expect_equal(totalCompare1$TOTAL_E, totalResult1$TOTAL_E) # failing
  expect_equal(totalCompare2$TOTAL_E, totalResult2$TOTAL_E)
  expect_equal(asthmaCompare$ASTHMA, asthmaResult$ASTHMA)
  expect_equal(totalCompare1$TOTAL_E, mixedResult$TOTAL_E) # failing
  expect_equal(asthmaCompare$ASTHMA, mixedResult$ASTHMA)
})

test_that("classes are created appropriately", {
  expect_equal("sf" %in% class(totalResult1), TRUE)
  expect_equal("tbl_df" %in% class(totalResult2), TRUE)
  expect_equal("sf" %in% class(asthmaResult), TRUE)
  expect_equal("tbl_df" %in% class(mixedResult), TRUE)
})

# test inputs not otherwise tested above ------------------------------------------------

test_that("correctly specified functions execute without error", {
  expect_error(aw_interpolate(ar_stl_wards, tid = WARD, source = combinedData, sid = GEOID,
                              weight = "sum", output = "sf", extensive = c("TOTAL_E", "WHITE_E", "BLACK_E")), NA)
  expect_error(aw_interpolate(ar_stl_wards, tid = WARD, source = combinedData, sid = GEOID,
                              weight = "sum", output = "sf", intensive = c("ASTHMA", "ASTHMA2")), NA)
  expect_error(aw_interpolate(ar_stl_wards, tid = WARD, source = combinedData, sid = GEOID,
                              weight = "sum", output = "sf", extensive = c("TOTAL_E", "WHITE_E", "BLACK_E"),
                              intensive = c("ASTHMA", "ASTHMA2")), NA)
})

race_geom <- dplyr::rename(ar_stl_race, geom = geometry)

test_that("correctly specified functions execute without error", {
  expect_error(aw_interpolate(ar_stl_wards, tid = WARD, source = race_geom, sid = GEOID,
                              weight = "sum", output = "sf", extensive = c("TOTAL_E", "WHITE_E", "BLACK_E")), NA)
})


# test for matching sid and tid ------------------------------------------------

wardsGEOID <- dplyr::rename(ar_stl_wards, GEOID = WARD)

wardsGEOID_result <- aw_interpolate(wardsGEOID, tid = GEOID, source = ar_stl_asthma,
                                    sid = GEOID, weight = "sum", output = "tibble", intensive = "ASTHMA")

test_that("interpolated values are equal", {
  expect_equal(wardsGEOID_result$ASTHMA, asthmaResult$ASTHMA)
})

---
title: "Preparing Data for Interpolation"
author: "Christopher Prener, Ph.D."
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Preparing Data for Interpolation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

library(areal)
library(dplyr)
library(sf)

data(ar_stl_asthma, package = "areal")

data(ar_stl_race, package = "areal")

data(ar_stl_wards, package = "areal")
```

Depending on the state of your spatial data, they may require some cleaning and modification before interpolation. The types of issues to address pre-interpolation fall into a few distinct categories:

1. Data are not in the right format
2. Data are not in the right coordinate system
3. Data have variable name conflicts
4. Data have too many variables or observations and therefore must be subset

Each of these conditions will be discussed below. The following examples assume:

```{r}
library(areal)
library(dplyr)   # data wrangling
library(sf)      # spatial data operations

# load data into enviornment
race <- ar_stl_race
asthma <- ar_stl_asthma
wards <- ar_stl_wards

# create example data - non-spatial data
asthmaTbl <- ar_stl_asthma
st_geometry(asthmaTbl) <- NULL

# create example data - wrong crs
race83 <- st_transform(race, crs = 4269)
```

## Validating Data
`areal` has a built-in tool for data validation, `ar_validate()`, that provides an excellent starting place for ensuring your data are ready to interpolate. The validation process covers the first three issues listed in the introduction. It can be run in a simple format:

```{r validate-simple}
ar_validate(source = asthma, target = wards, varList = "ASTHMA", method = "aw")
```

If `ar_validate()` returns a `FALSE` value, it can also be run in a verbose manner that returns a detailed tibble:

```{r validate-verbose}
ar_validate(source = asthma, target = wards, varList = "ASTHMA", method = "aw", verbose = TRUE)
```

Use the `verbose = TRUE` output as a checklist to address issues before interpolating your data.

## Format Issues
Data need to be loaded as `sf` objects in `R`. If `ar_validate()` returns `FALSE` for the first condition, data are not stored as `sf` objects:

```{r validate-non-sf}
ar_validate(source = asthmaTbl, target = wards, varList = "ASTHMA", method = "aw", verbose = TRUE)
```

### External Spatial Data
If you have external data, using `st_read()` is the best way to load them:

```r
df <- sf::st_read("data.shp", stringsAsFactors = FALSE)
```

The `st_read()` function supports a variety of the most common data sources.

### sp Data
If you are working with data that is an `sp` object in `R`, the `sf` package has a function `st_as_sf()` can be used to convert the object to `sf`:

```r
sf_object <- sf::st_as_sf(sp_object)
```

### Data From tidycensus and tigris
If you are using either the [`tidycensus`](https://walker-data.com/tidycensus/) or [`tigris`](https://github.com/walkerke/tigris) packages to access spatial data, there are options for downloading data as `sf` objects as well. In `tidycensus`, the `geometry = TRUE` option should be used. In `tigris`, the `class = "sf"` option should be used.

```r
stl_race <- tidycensus::get_acs(geography = "tract", state = 29, county = 510, year = 2017, 
                                table = "B02001", output = "wide", geometry = TRUE)
stl_tracts <- tigris::tracts(state = 29, county = 510, class = "sf")
```

### Tabular Data
If you have tabular data, you will need to merge them with a shapefile or other spatial data set that contains the geometry for the corresponding spatial features. For American users, a potentially common scenario here will be table of census geography (such as tracts) that lacks the geometry for those features. These can be downloaded using [`tigris`](https://github.com/walkerke/tigris) and then combined using [`dplyr`](https://dplyr.tidyverse.org). The following example assumes the tract data contains an identification number column named `GEOID` with the appropriate tract identifiers for St. Louis:

```r
stl_tracts <- tigris::tracts(state = 29, county = 510, class = "sf")

tract_data <- dplyr::left_join(stl_tracts, asthmaTBL, by = "GEOID")
```

## Coordinate Systems
Two coordinate system rules are enforced by `ar_validate`: both the source and the target data must be in the same [coordinate system](https://geocompr.robinlovelace.net/spatial-class.html#crs-intro) and that coordinate system must be a projected coordinate system. The `sf` package contains a function named `st_crs()` for previewing the current coordinate system of your data:

```{r race-crs}
st_crs(race83)
```

The `EPSG` value `4269` refers to a specific type of coordinate system known as as geographic coordinate system. These data use latitude-longitude values, which vary in length and thus cannot be used to calculate area (a key component of carrying out areal interpolations). These data would fail the `ar_validate()` process:

```{r validate-non-matching-crs}
ar_validate(source = race83, target = wards, varList = "TOTAL_E", method = "aw", verbose = TRUE)
```

These data fail both the matching CRS validation and the planar CRS validation because the `race` and `wards` data are in two different coordinate systems. We've already seen the CRS data for `race` above so we'll look at the `CRS data for `wards`:

```{r wards-crs}
st_crs(wards)
```

If these data fail that process, `st_crs()` can be used to identify whether the source or the target (or both) are in an inappropriate coordinate system. The `EPSG` value `26915` refers to another type of coordinate system known as as projected coordinate system (or "planar" data). Further, `26915` is a particular type of projected coordinate system known as the [Universal Transverse Mercator](https://en.wikipedia.org/wiki/Universal_Transverse_Mercator_coordinate_system) coordinate system. This is a good option for using `areal` because it is available around the world. There are a variety of tools for identifying the appropriate UTM zone, including this excellent [interactive map](https://mangomap.com/robertyoung/maps/69585/what-utm-zone-am-i-in-#). Once you have an EPSG zone, the website [epsg.io](https://epsg.io) can be used to search for the appropriate `EPSG` value.

In this case, the `wards` data are in an appropriate system but the `race` data need to be transformed. The `st_transform()` function can be used to modify them to an appropriate coordinate system:

```{r transform-crs}
raceFixed <- st_transform(race83, crs = 26915)
```

Once this is done, these data should pass the `ar_validate()` process:

```{r validate-matching-crs}
ar_validate(source = raceFixed, target = wards, varList = "TOTAL_E", method = "aw", verbose = TRUE)
```

## Variable Conflicts
If a variable does not exist in the source data, the validation process will fail:

```{r}
ar_validate(source = race, target = wards, varList = "TOTAL", method = "aw", verbose = TRUE)
```

In this case, the fix is as easy as correcting the typo in our `ar_validate()` call - the variable `TOTAL` should really be `TOTAL_E`:

```{r}
names(race)
```

Another common problem is that a variable exists in both our source and target data. For instance, we could add a `TOTAL_E` variable to our target data:

```{r}
wardsVar <- mutate(wards, TOTAL_E = seq(1:28))

ar_validate(source = race, target = wardsVar, varList = "TOTAL_E", method = "aw", verbose = TRUE)
```

Now, we fail the validation process because there is a conflict between the source and target data - `TOTAL_E` exists in both data sets. We can use the `select()` function from `dplyr` to remove the offending variable from the target data before proceeding:

```{r}
wardsFixed <- select(wardsVar, -TOTAL_E)

ar_validate(source = race, target = wardsFixed, varList = "TOTAL_E", method = "aw", verbose = TRUE)
```

Another option would be to rename the column using the `rename()` function from `dplyr` or by using another approach for renaming columns. Either option will work for `areal`'s purposes.

## Subsetting
One cautionary note when using `areal` is that the validation process will not check to see if too many variables or observations exist in the data. Cleaning the data ahead of time so that only the relevant observations exist can help improve performance - if you have tract data for an entire state but only need a particular city, everything will go faster if you subset these data using the `filter()` function from `dplyr`:

```r
countyData <- filter(stateData, COUNTY == 510)
```

Having too many columns poses an organizational problem more than anything else. We strongly encourage users to limit their interpolated data to only the needed values. For example, the `wards` data contains an `AREA` column whose units we don't know and an `OBJECTID` column that is an artifact from its creation using an ESRI product:

```{r}
names(wards)
```

We can use the `select()` function from `dplyr` to remove these before interpolation since we don't need them:

```{r}
wardsSubset <- select(wards, -OBJECTID, -AREA)
```

---
title: "Areal Interpolation in `R`"
author: "Christopher Prener, Ph.D."
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Areal Interpolation in `R`}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

Areal interpolation is the process making estimates from a source set of polygons to an overlapping but incongruent set of target polygons. This is required if, for example, a researcher wants to derive population estimates for neighborhoods in a U.S. city from the Census Bureau's census tracts. Neighborhoods do not typically align with census tract boundaries, and areal interpolation can be used to produce estimates in this situation. 

## Motivation
As helpful as this may be, areal interpolation is not widely used. Many neighborhood researchers use census tracts, census block groups, or even zip codes as proxies for neighborhoods instead of interpolating values from these underlying sources into the actual neighborhoods. A key reason many researchers do not take this second step is that desktop GIS applications have not historically offered built-in tools for completing these calculations. Thus researchers wishing to use areal interpolation must "reinvent the wheel" (Qiu et al. 2012) when it comes to developing processes for carrying out areal interpolation.

In `R`, the `sf` package does offer a basic interface for areal weighted interpolation (`st_interpolate_aw`). While this functionality is welcome and useful, its implementation requires extensive data cleaning to carry out multiple interpolations (or the use of a tool like `purrr`). The process is also a "black box" - the interpolations are produced without a means to dis-aggregate the process into its constituent elements.

`areal` is designed as an alternative to this approach. For its initial release, the package offers a specific approach to interpolation known as areal weighted interpolation with several distinct features. Iteration is built-in and does not require using the apply family of functions from `base` or tools from `purrr`. Users have multiple choices available both in terms of how interpolations are calculated and how output is structured. The key function, `aw_interpolate()` is pipe-able, meaning it fits into existing `tidyverse` workflows. Helper functions are provided to validate data and make decisions about which areal weight to implement. And finally, the areal weighted interpolation process can be dis-aggregated by applying the necessary functions in sequence. 

What results from areal interpolation is mapped below. We begin with a common unit of spatial data aggregation - the census tract. Total population per tract is mapped on the left in panel A. Using the `areal` package, we estimate population counts for wards in St. Louis from those census tract values. These estimated values are mapped on the right in panel B.

```{r exampleMap, echo=FALSE, out.width = '100%'}
knitr::include_graphics("../man/figures/exampleMap.png")
```

## Getting Started
### Installation
The easiest way to get `areal` is to install it from CRAN:

``` r
install.packages("areal")
```

Alternatively, the development version of `areal` can be accessed from GitHub with `remotes`:

```r
# install.packages("remotes")
remotes::install_github("chris-prener/areal")
```

Note that installations that require `sf` to be built from *source* will require additional software regardless of operating system. You should check the [`sf` package website](https://r-spatial.github.io/sf/) for the latest details on installing dependencies for that package. Instructions vary significantly by operating system.

### Functions
Two function prefixes are used in `areal` to allow users to take advantage of RStudio's auto complete functionality:

* `ar_` - data and functions that are used for multiple interpolation methods
* `aw_` - functions that are used specifically for areal weighted interpolation

### Data
The package contains four overlapping data sets that can be used for practice: 

* `ar_stl_race` (2017 ACS demographic counts at the census tract level; *n* = 106) 
* `ar_stl_asthma` (2017 asthma rates at the census tract level; *n* = 106)
* `ar_stl_wards` (the 2010 political subdivisions in St. Louis; *n* = 28). 
* `ar_stl_wardsClipped` (the 2010 political subdivisions in St. Louis clipped to the Mississippi River shoreline; *n* = 28). 

### Preparing Data
`areal` assumes that all data are available as `sf` objects. The `st_read()` function is useful for importing data stored in a variety of formats into `sf` objects. Creating these objects is a pre-requisite to passing the validation steps before interpolations are completed.

It is also recommended that all data are stored using a projected coordinate system as opposed to a geographic coordinate system like NAD 1983 or WGS 1984. There are many projected coordinate systems to choose from, but the key is that all of your data is projected using the same system. Here too the `sf` package has built-in functionality - the `st_transform()` function can be used for data re-projection.

Finally, we strongly suggest that you clean out unnecessary columns from your data prior to projection. `sf` objects can be cleaned using `tidyverse` tools from `dplyr`, which makes it easy to `select()` the columns and `filter()` the observations you want to focus on for interpolation. Columns can also be easily renamed using `rename()`.

Once data have been converted to `sf` objects, re-projected into the same planar coordinate systems, and cleaned, they are ready for interpolation!

The data preparation process is described in greater detail in a [dedicated vignette](https://chris-prener.github.io/areal/articles/data-preparation.html).

### Areal Weighted Interpolation
Areal weighted interpolation makes a single, albeit significant, assumption about your data - that populations are evenly distributed within the source data. Imagine you have a census tract with 3,000 residents. Areal weighted interpolation will assume that these 3,000 residents are evenly spread out within that tract. This may not matter in some tracts, but in tracts with large parks or dense housing developments alongside commercial buildings, this assumption is not likely to hold.

The primary means for conducting interpolations, and possibly the only function you may need from `areal`, is the `aw_interpolate()` function:

```r
## load dependencies
library(areal)

## interpolate values
aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_race, sid = "GEOID", 
               weight = "sum", output = "sf", extensive = "TOTAL_E")
```

The function is capable of both spatially extensive and intensive interpolations, and can use two different strategies for calculating areal weights for extensive interpolations. It can also produce output as either `tibble` or `sf` objects. These options are all documented in both the function documentation (use `?aw_interpolate`) as well as in a [dedicated vignette](https://chris-prener.github.io/areal/articles/areal-weighted-interpolation.html).

## Getting Help
* If you are new to `R` itself, welcome! Hadley Wickham's [*R for Data Science*](https://r4ds.had.co.nz) is an excellent way to get started with data manipulation in the tidyverse, which `areal` is designed to integrate seamlessly with.
* If you are new to spatial analysis in `R`, we strongly encourage you check out the excellent new [*Geocomputation in R*](https://geocompr.robinlovelace.net) by Robin Lovelace, Jakub Nowosad, and Jannes Muenchow.
* If you have questions about using `areal`, you are encouraged to use the [RStudio Community forums](https://community.rstudio.com). Please create a [`reprex`](https://reprex.tidyverse.org/) before posting. Feel free to tag Chris (`@chris.prener`) in any posts about `areal`.
* If you think you've found a bug, please create a [`reprex`](https://reprex.tidyverse.org/) and then open an issue on [GitHub](https://github.com/chris-prener/areal/issues/new/choose).

## Suggesting Features or Changes
We are planning to experiment with at least three additional techniques for areal interpolation for possible inclusion into the package. These include:

- [Pycnophylactic method](https://github.com/chris-prener/areal/issues/1) (raster based, eliminates the sharp transitions in value between target features)
- [Binary dasymetric method](https://github.com/chris-prener/areal/issues/2) (incorporates ancillary data so that population is not assumed to be evenly distributed within units)
- [3-class regression dasymetric method](https://github.com/chris-prener/areal/issues/3) (allows for a more complex estimation based on multiple forms of ancillary data)

We do not have a timeline for these experiments, though we are planning to begin experimenting with the pycnophylactic method in the coming months. We will be keeping the issues (linked to above) updated with progress. If you are interested in bringing these techniques to `R`, please feel free to contribute to the development of `areal`. The best place to start is bt checking in on our GitHub issues for each technique to see what help is needed!

If you have other features or suggestions you want to see, please open an issue on [GitHub](https://github.com/chris-prener/areal/issues/new/choose) (and ideally created a [`reprex`](https://reprex.tidyverse.org/) to go with it!). Please note that this project is released with a [Contributor Code of Conduct](https://chris-prener.github.io/areal/CODE_OF_CONDUCT.html). By participating in this project you agree to abide by its terms.

---
title: "Areal Weighted Interpolation"
author: "Christopher Prener, Ph.D."
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Areal Weighted Interpolation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

library(areal)
library(dplyr)
library(sf)

data(ar_stl_asthma, package = "areal")
data(ar_stl_race, package = "areal")
data(ar_stl_wards, package = "areal")
data(ar_stl_wardsClipped, package = "areal")
```

Areal weighted interpolation is a technique for estimating the values for overlapping but incongruent polygon features. This article describes the `areal` package's approach to areal weighted interpolation. After providing a quick introduction to the technique, the options for `aw_interpolate()` are discussed and an example of interpolating data using the manual workflow is provided.

## Introduction to Areal Weighted Interpolation
Areal weighted interpolation is the simplest approach to estimating population values for overlapping polygons. It makes a significant and important assumption - that individuals are spread out *evenly* within the source features. This assumption quickly breaks down in the real world - areas that have commercial developments mixed in with residential housing, for example, or neighborhoods with a large city park. We do not always have access to this type of contextual data, however, and so areal weighted interpolation remains a popular choice. 

Areal weighted interpolation is a multi-step process. The `areal` package contains a number of example data sets that can be used to illustrate this process:

```{r load-data}
library(areal)

# load data into enviornment
race <- ar_stl_race                 # census tract population estimates
asthma <- ar_stl_asthma             # census tract asthma rate estimates
wards <- ar_stl_wards               # political boundaries
wardsClipped <- ar_stl_wardsClipped # political boundaries clipped to river
```

The boundaries for the `race` and `asthma` the data are the same - census tracts. When mapped, this is what the census tract and ward features look like:

```{r featureMap, echo=FALSE, out.width = '100%'}
knitr::include_graphics("../man/figures/featureMap.png")
```

### Step 1: Intersection

The first step with areal weighted interpolation is to intersect the data. Imagine one shapefile (we'll call this the "target") acting as a cookie cutter - subdividing the features of the other (which we'll call the "source") based on areas of overlap such that only those overlapping areas remain (this is important - if these shapefiles do not cover identical areas, those areas only covered by one shapefile will be lost). The number of new features created is entirely dependent on the shapes of the features in the source and target data sets:

```{r feature-count}
# print number of features in source
nrow(race)

# print number of features in target
nrow(wards)

# create intersect for example purposes
nrow(suppressWarnings(sf::st_intersection(race, wards)))
```

By intersecting these two data sets, we get a new data set with *n* = 287 features. The resulting `sf` object looks like so:

```{r intersectMap, echo=FALSE, out.width = '100%'}
knitr::include_graphics("../man/figures/intersectMap.png")
```

One by-product of the intersection process is that each intersected feature takes on the attributes of both the source and target data. The population value of interest from each source feature (for example, total population per tract or `TOTAL_E`), therefore exists as an attribute for each intersected feature. The identification numbers from both the source (`GEOID`) and the target data (`WARD`) are also applied:

```{r data-by-hand, echo=FALSE}
as_tibble(
  data.frame(
    GEOID = c(29510101100, 29510101100, 29510101200, 29510101200),
    TOTAL_E = c(2510, 2510, 3545, 3545),
    WARD = c(11, 12, 12, 13)
  )
) %>% 
  knitr::kable(caption = "First Four Rows of Intersected Data")
```

### Step 2: Areal Weights

We then calculate an areal weight for each intersected feature. Let:

* ${W}_{i} = \textrm{areal weight for intersected feature i}$
* ${A}_{i} = \textrm{area of intersected feature i}$
* ${A}_{j} = \textrm{total area of source feature j}$

$$ {W}_{i} = \frac { {A}_{i} }{ {A}_{j} } $$

Since ${A}_{j}$ is calculated using the source identification number, the first two observations from table above with the first four rows of intersected data would have the same value for ${A}_{j}$, and the second two observations would also share the same ${A}_{j}$. The resulting values for ${W}_{i}$ would therefore be:

```{r weight-by-hand, echo=FALSE}
as_tibble(
  data.frame(
    GEOID = c(29510101100, 29510101100, 29510101200, 29510101200),
    TOTAL_E = c(2510, 2510, 3545, 3545),
    WARD = c(11, 12, 12, 13),
    Ai = c(355702.9, 901331.1, 875554.7, 208612.1),
    Aj = c(1257034.0, 1257034.0, 1084166.8, 1084166.8),
    Wi = c(0.28297, 0.71703, 0.807583, 0.192417)
  )
) %>% 
  knitr::kable(caption = "First Four Rows of Intersected Data")
```

### Step 3: Estimate Population

Next, we need to estimate the share of the population value that occupies the intersected feature. Let:

* ${E}_{i} = \textrm{estimated value for intersected feature } i$
* ${W}_{i} = \textrm{areal weight for intersected feature } i$
* ${V}_{j} = \textrm{population value for source feature } j$

$$ {E}_{i} = {V}_{j}*{W}_{i} $$

Using our sample data, we therefore multiply the value (`TOTAL_E`) by the weight (`Wi`) to produce our `EST` estimate column:

```{r calculate-by-hand, echo=FALSE}
as_tibble(
  data.frame(
    GEOID = c(29510101100, 29510101100, 29510101200, 29510101200),
    TOTAL_E = c(2510, 2510, 3545, 3545),
    WARD = c(11, 12, 12, 13),
    Ai = c(355702.9, 901331.1, 875554.7, 208612.1),
    Aj = c(1257034.0, 1257034.0, 1084166.8, 1084166.8),
    Wi = c(0.28297, 0.71703, 0.807583, 0.192417),
    EST = c(710.2547, 1799.745, 2862.882, 682.1182)
  )
) %>% 
  knitr::kable(caption = "First Four Rows of Intersected Data")
```

### Step 4: Summarize Data

Finally, we summarize the data based on the target identification number. Let:

* ${G}_{k} = \textrm{sum of all estimated values for target feature } k$
* ${E}_{ik} = \textrm{estimated values from intersected features in } i \textrm{ within target feature } k$

$$ {G}_{k} = \sum{{E}_{ik}} $$

With our hypothetical data, the resulting table would therefore look like:

```{r aggregate-by-hand, echo=FALSE}
as_tibble(
  data.frame(
    WARD = c(11, 12, 13),
    EST = c(710.2547, 4662.627, 682.1182)
  )
) %>% 
  knitr::kable(caption = "Resulting Target Data")
```

This process is repeated for each of the *n* = 287 observations in the intersected data - areal weights are calculated, and the product of the areal weight the source value is summed based on the target identification number.

## Extensive and Intensive Interpolations
### Extensive Interpolations
The example above is a spatially *extensive* interpolation because it involves count data. In `areal`, these estimates are obtained using the `aw_interpolate()` function:

```{r extensive}
aw_interpolate(wards, tid = WARD, source = race, sid = GEOID, 
               weight = "sum", output = "tibble", extensive = "TOTAL_E")
```

For spatially extensive interpolations, a list of variable names should be supplied for the argument `extensive`. This can be a single variable name, such as in the example above, or a vector of variable names:

```{r extensive-vector}
aw_interpolate(wards, tid = WARD, source = race, sid = GEOID, 
               weight = "sum", output = "tibble", 
               extensive = c("TOTAL_E", "WHITE_E", "BLACK_E"))
```

This ability is a key feature of `areal` - iteration is built into the package by default, eliminating the need for repeated table joins after interpolations are calculated.

### Calculating Weights for Extensive Interpolations
The `aw_interpolate` function also uses an argument `weight`. There are two options, `"sum"` and `"total"`. Each makes a different assumption about the nature of the data and the relationship between the `source` and `target` features. For *perfectly* overlapping data, the distinction between these two options should not matter. In practice, however, there are often deviations in our data even between features that should be perfectly congruous.

The `"sum"` approach to calculating weights assumes that 100% of the source data should be divided among the target features. When ${A}_{j}$ is calculated (see previous section), it is done by taking the sum of the areas for all intersected features ($i$) within a given source feature ($j$). Let:

* ${A}_{j} = \textrm{total area of source feature j}$
* ${A}_{ij} = \textrm{areas for intersected features in } i \textrm{ within source feature } j$

$$ {A}_{j} = \sum{{A}_{ij}} $$

On the other hand, the `"total"` approach to calculating weights assumes that, if a source feature is only covered by 99.88% of the target features, only 99.88% of the source target's data should be allocated to target features in the interpolation. When ${A}_{j}$ is created, the actual area of source feature $j$ is used.

#### Weights Example 1: Non-Overlap Due to Data Quality
In the example above, `race` and `wards` are products of two different agencies. The `aw_stl_wards` data is a product of the City of the St. Louis and is quite close to fully overlapping with the U.S. Census Bureau's TIGER boundaries for the city. However, there are a number of very small deviations at the edges where the ward boundaries are *smaller* than the tracts (but only just so). These deviations result in small portions of census tracts not fitting into any ward. 

We can see this in the weights that are used by `aw_interpolate()`. The `aw_preview_weights()` function can be used to return a preview of these areal weights. 

```{r extensive-weights}
aw_preview_weights(wards, tid = WARD, source = race, sid = GEOID, 
                   type = "extensive")
```

The first tract listed above has a total estimated population of 2510. The practical impact of the weights is that only `r 2510*.9988335` individuals will be allocated to wards if the `"total"` approach to calculating areal weights is used. If `"sum"` is used, on the other hand, all 2510 individuals would be allocated to wards. In this scenario, the `"sum"` approach makes more sense because, while the race and ward data do not overlap in practice, they *should* overlap since no tracts extend out of the city's boundaries. We therefore want to ensure that all individuals within each tract are allocated out to wards.

With spatially extensive interpolations that utilize the `"sum"` approach, the sum of the interpolated column should equal the sum of the original source data's column that was interpolated. This can be verified with `aw_verify()`:

```{r verify-true}
result <- aw_interpolate(wards, tid = WARD, source = race, sid = GEOID, 
               weight = "sum", output = "tibble", extensive = "TOTAL_E")

aw_verify(source = race, sourceValue = TOTAL_E, 
          result = result, resultValue = TOTAL_E)
```

This check does *not* work with the `"total"` approach to areal weights:

```{r verify-fail}
result <- aw_interpolate(wards, tid = WARD, source = race, sid = GEOID, 
               weight = "total", output = "tibble", extensive = "TOTAL_E")

aw_verify(source = race, sourceValue = TOTAL_E, 
          result = result, resultValue = TOTAL_E)
```

#### Weights Example 2: Non-Overlap Due to Differing Boundaries
We can use the `aw_stl_wardsClipped` data to illustrate a more extreme disparity between source and target data. The `aw_stl_wardsClipped` data have been modified so that the ward boundaries do not extend past the Mississippi River shoreline, which runs along the entire eastern boundary of the city. When we overlay them on the city's census tracts, all of the census tracts on the eastern side of the city extend outwards. 

```{r overlapMap, echo=FALSE, out.width = '100%'}
knitr::include_graphics("../man/figures/overlapMap.png")
```

The difference in weights in this example is more extreme:

```{r extensive-weights-overlap}
aw_preview_weights(wardsClipped, tid = WARD, source = race, sid = GEOID, 
                   type = "extensive")
```

Only 72.31% of tract `29510101800`, for example, falls within a ward. In many American cities that lie within larger counties, tract boundaries do not stop at the municipal boundaries in a way that is similar to the difference between tracts and the clipped wards here. In this scenario, we do not want to allocate every individual into our city of interest and the `"total"` approach to weights is appropriate. Not using `"total"` would result in an over-count of individuals in our city.

If, on the other hand, we believe that all of the individuals *should* be allocated into wards, using `"total"` in this case would result in a severe under-count of individuals. 

### Intensive Interpolations
Spatially *intensive* operations are used when the data to be interpolated are, for example, a percentage or density value. An example of these data can be found in `ar_stl_asthma`, which contains asthma rates for each census tract in the city. The interpolation process is very similar to the spatially extensive workflow, except with how the areal weight is calculated. Instead of using the source data's area for reference, the *target* data is used in the denominator. Let:

* ${W}_{i} = \textrm{areal weight for intersected feature i}$
* ${A}_{i} = \textrm{area of intersected feature i}$
* ${A}_{ik} = \textrm{areas for intersected features in } i \textrm{ within target feature } k$

$$ {W}_{i} = \frac { {A}_{i} }{ \sum{{A}_{ik}} } $$

Like spatially extensive interpolations that use the `"sum"` approach, the weights for intensive interpolations should always be equal to 1 as well. 

```{r invenstive-weights}
aw_preview_weights(wards, tid = WARD, source = asthma, sid = GEOID, 
                   type = "intensive")
```

We can calculate the intensive interpolation by specifying a variable name for the `intensive` argument in `aw_interpolate()` and omitting the `extensive` argument:

```{r intensive}
aw_interpolate(wards, tid = WARD, source = asthma, sid = GEOID, 
               weight = "sum", output = "tibble", intensive = "ASTHMA")
```

This gives us an estimate of the asthma rates at the ward level.

### Mixed Interpolations
`areal` also provides support for "mixed" interpolations where both spatially extensive and intensive interpolations need to be calculated. We specify a variable name or a vector of variable names for *both* the `intensive` and `extensive` arguments:

```{r mixed}
# remove sf geometry
st_geometry(race) <- NULL

# create combined data
race %>%
  select(GEOID, TOTAL_E, WHITE_E, BLACK_E) %>%
  left_join(asthma, ., by = "GEOID") -> combinedData

# interpolate
aw_interpolate(wards, tid = WARD, source = combinedData, sid = GEOID, 
               weight = "sum", output = "tibble", intensive = "ASTHMA",
               extensive = c("TOTAL_E", "WHITE_E", "BLACK_E"))
```

Users should take care to consider the implications of interpolating multiple values, such as total population and the number of African American residents (both extensive), and then calculating a spatially intensive variable from them such as percent African American. Doing so in multiple steps, and thereby treating extensive and intensive values as independent, may result in estimates that differ from a single step process where the percent of African American residents is interpoltated directly. 

```{r constraints}
# re-load data
race <- ar_stl_race

# create combined data
race %>%
  select(GEOID, WHITE_E, BLACK_E) %>%
  mutate(
    TOTAL = WHITE_E+BLACK_E,
    WHITE_PCT = WHITE_E/TOTAL,
    BLACK_PCT = BLACK_E/TOTAL,
    TOTAL_PCT = WHITE_PCT+BLACK_PCT
  ) -> constrainedData

# interpolate
result2 <- aw_interpolate(ar_stl_wards, tid = WARD, 
               source = constrainedData, sid = GEOID, 
               weight = "sum", output = "tibble", 
               intensive = c("WHITE_PCT", "BLACK_PCT", "TOTAL_PCT"),
               extensive = c("TOTAL", "WHITE_E", "BLACK_E"))

# calculate new percentages
result2 %>%
  mutate(
    WHITE_PCT_2 = WHITE_E/TOTAL,
    BLACK_PCT_2 = BLACK_E/TOTAL,
    TOTAL_PCT_2 = WHITE_PCT_2+BLACK_PCT_2
  ) -> result2

# display
result2 %>%
  select(WHITE_PCT, WHITE_PCT_2, BLACK_PCT, BLACK_PCT_2, TOTAL_PCT, TOTAL_PCT_2)
```

Note that there are a number of points of departure between the data interpolated as intensive values (`WHITE_PCT`, `BLACK_PCT`) and those that were interpolated as count data (i.e. extensive values) and then converted to intensive variables (`WHITE_PCT_2` and `BLACK_PCT_2`). 

## Output Options
All of the above examples have created a tibble for output, but `areal` also supports the creation of `sf` objects as well:

```{r ouput}
aw_interpolate(wards, tid = WARD, source = asthma, sid = GEOID, 
               weight = "sum", output = "sf", intensive = "ASTHMA")
```

## Other Features of aw_interpolate
The `sf` option will include *all* of the variables that were included in the original target data. The `aw_interpolate()` function is pipe-able, allowing for existing tidyverse workflows to be integrated into the interpolation process. For example, if we wanted to remove the `OBJECTID` and `AREA` columns because they are not needed, this can be accomplished easily with `areal` and `dplyr`:

```{r piped-input}
wards %>%
  select(-OBJECTID, -AREA) %>%
  aw_interpolate(tid = WARD, source = asthma, sid = GEOID, 
                 weight = "sum", output = "tibble", intensive = "ASTHMA")
```

All of the `areal` functions that are exported support non-standard evaluation, meaning that inputs can be either unquoted as they are above or quoted:

```{r quoted-input}
wards %>%
  select(-OBJECTID, -AREA) %>%
  aw_interpolate(tid = "WARD", source = asthma, sid = "GEOID", 
                 weight = "sum", output = "tibble", intensive = "ASTHMA")
```

This functionality is not available for the `intensive` and `extensive` arguments at this time.

## Manual Workflow
`areal` purposely exports the sub-functions that are called by `aw_interpolate()` so that the interpolation process is not a "black box" but rather can be recreated manually. This is envisioned as a diagnostic toolkit, with the final interpolations estimated using the simpler `aw_interpolate()` function once any issues have been identified and ameliorated

First, we'll prepare the data but retaining only the columns we are interested in from the source data using the `select()` function from `dplyr`:

```{r manual-subset}
race <- select(ar_stl_race, GEOID, TOTAL_E)
wards <- select(wards, -OBJECTID, -AREA)
```

We want to be careful to retain both a column with a value to be interpolated (total population in this case, `TOTAL_E`) and a column with a unique identification number (`GEOID` in this case).

### Intersect Data
As we noted above, the interpolation process begins with calculating the intersection between the source and target data. We use the function `aw_intersect()` to accomplish this:

```{r aw-intersect}
wards %>%
  aw_intersect(source = race, areaVar = "area") -> intersect

intersect
```

Note that `aw_intersect()` automatically calculates the area of the intersected feature.

### Calculate Total Area
Next, we want to apply the total area of our source features to our data using `aw_total()`. This will implement the correct areal weighting approach based on the `type` and `weight` arguments. We'll use the `"sum"` approach to areal weights here:

```{r aw-total}
intersect %>%
  aw_total(source = race, id = GEOID, areaVar = "area", totalVar = "totalArea",
             type = "extensive", weight = "sum") -> intersect

intersect
```

Changing `type` to `"intensive"` would be necessary for spatially intensive interpolations. Likewise, changing `weight` to `"total"` is necessary if areas that lack overlap should not be allocated into the target features.

### Calculate Areal Weight
With the total weight in hand, we are ready to calculate the areal weight itself using `aw_weight()`.

```{r aw-weight}
intersect %>%
  aw_weight(areaVar = "area", totalVar = "totalArea", 
            areaWeight = "areaWeight") -> intersect

intersect
```

### Calculate Estimated Population
We can then multiply the value (`TOTAL_E`) by the weight (`areaWeight`) to get a population estimate for each intersected feature using `aw_calculate()`:

```{r aw-calculate}
intersect %>%
  aw_calculate(value = TOTAL_E, areaWeight = "areaWeight") -> intersect

intersect
```

There is an optional `newVar` argument that can be used to store the estimates in a new column rather than in the existing `value` column.

### Aggregate Estimated Population by Target ID
Finally, we aggregate the estimated values by target features using `aw_aggregate()`:

```{r aw-aggregate}
intersect %>%
  aw_aggregate(target = wards, tid = WARD, interVar = TOTAL_E) -> result

result
```

---
name: Bug report
about: Create a report to help me address a problem

---

**Before you open your issue:**
* All issues and contributions are covered by the [Code of Conduct](/.github/CODE_OF_CONDUCT.md)
* Please check out the [Contribution guidelines](/.github/CONTRIBUTING.md)

**Describe the bug**
A clear and concise description of what the bug is.

**Expected behavior**
A clear and concise description of what you expected to happen.

**To Reproduce**
Please include a minimal reproducible example (AKA a reprex). If you've never
heard of a [reprex](http://reprex.tidyverse.org/) before, start by reading
<https://www.tidyverse.org/help/#reprex>. When you create your reprex, please use 
a data set available in a package.

```r
# inset reprex here
```

**Screenshots**
If applicable, add screenshots to help explain your problem.

**Desktop (please complete the following information):**
 - OS: [e.g. macOS]
 - Version of R [e.g. v3.5.1]
 - Version of RStudio [e.g. v1.1.453]

**Additional context**
Add any other context about the problem here.

# areal <img src="man/figures/arealLogo.png" align="right" />

[![R build status](https://github.com/chris-prener/areal/workflows/R-CMD-check/badge.svg)](https://github.com/chris-prener/areal/actions)
[![Coverage status](https://codecov.io/gh/chris-prener/areal/branch/master/graph/badge.svg)](https://codecov.io/github/chris-prener/areal?branch=main)
[![CRAN_status_badge](http://www.r-pkg.org/badges/version/areal)](https://cran.r-project.org/package=areal)
[![cran checks](https://cranchecks.info/badges/worst/areal)](https://cran.r-project.org/web/checks/check_results_areal.html)
[![Downloads](http://cranlogs.r-pkg.org/badges/areal?color=brightgreen)](http://www.r-pkg.org/pkg/areal)
[![DOI](https://zenodo.org/badge/152279647.svg)](https://zenodo.org/badge/latestdoi/152279647) 
[![DOI](http://joss.theoj.org/papers/10.21105/joss.01221/status.svg)](https://doi.org/10.21105/joss.01221)

Areal interpolation is the process making estimates from a source set of polygons to an overlapping but incongruent set of target polygons. One challenge with areal interpolation is that, while the processes themselves are well documented in the academic literature, implementing them often involves "reinventing the wheel." While the `R` package `sf` does offer a basic interface for areal weighted interpolation (`st_interpolate_aw`), it lacks some features that we use in our work. The `areal` package contains a suite tools for validation and estimation, providing a full-featured workflow that fits into both modern data management (e.g. `tidyverse`) and spatial data (e.g. `sf`) frameworks.

### *Joural of Open Souce Software* Article
An [article](http://joss.theoj.org/papers/10.21105/joss.01221) describing `areal`'s approach to areal weighted interpolation has been published in the [*The Journal of Open Source Software*](http://joss.theoj.org/). The article includes benchmarking of `areal` performance on several data sets. Please [cite the paper](authors.html) if you use `areal` in your work!

## Quick Start
The easiest way to get `areal` is to install it from CRAN:

``` r
install.packages("areal")
```

Alternatively, the development version of `areal` can be accessed from GitHub with `remotes`:

```r
# install.packages("remotes")
remotes::install_github("chris-prener/areal")
```

Additional details, including some tips for installing `sf`, can be found in the [Get started article](articles/areal.html#getting-started).

## Resources
In addition to instructions for installation, the main [Get started](articles/areal.html) article has:

  * a quick overview of areal interpolation,
  * some notes on preparing data for interpolation,
  * a brief introduction to the `aw_interpolate()` function,
  * tips for getting help and submitting feedback, 
  * and the `areal` package's development roadmap!
  
This site also offers dedicated articles on [data preparation](articles/data-preparation.html) and [using `areal` for areal weighted interpolation](articles/areal-weighted-interpolation.html).

**Before you open your pull request:**

* All issues and contributions are covered by the [Code of Conduct](/.github/CODE_OF_CONDUCT.md)
* Please check out the [Contribution guidelines](/.github/CONTRIBUTING.md)

**Describe your contribution** Please describe the changes you are suggestion. If these are in response to specific issues, please link to them by using the `#15` syntax where `15` is the issue number.

---
name: Feature request
about: Suggest an idea for this project

---

**Before you open your issue:**
* All issues and contributions are covered by the [Code of Conduct](/.github/CODE_OF_CONDUCT.md)
* Please check out the [Contribution guidelines](/.github/CONTRIBUTING.md)

**Is your feature request related to a problem? Please describe.**
A clear and concise description of what the problem is. Ex. I'm always frustrated when [...]

**Describe the solution you'd like**
A clear and concise description of what you want to happen.

**Provide a reprex**
Please include a minimal reproducible example (AKA a reprex). If you've never
heard of a [reprex](http://reprex.tidyverse.org/) before, start by reading
<https://www.tidyverse.org/help/#reprex>. When you create your reprex, please use 
a data set available in a package.

```r
# inset reprex here
```

**Describe alternatives you've considered**
A clear and concise description of any alternative solutions or features you've considered.

**Additional context**
Add any other context or screenshots about the feature request here.

---
name: Other inquiry
about: Ask a question or give me some feedback

---

**Before you open your issue:**
* All issues and contributions are covered by the [Code of Conduct](/.github/CODE_OF_CONDUCT.md)
* Please check out the [Contribution guidelines](/.github/CONTRIBUTING.md)

**Is your inquiry related to a problem? Please describe.**
A clear and concise description of what the problem is. Ex. I'm always frustrated when [...]

**Provide a reprex**
Please include a minimal reproducible example (AKA a reprex). If you've never
heard of a [reprex](http://reprex.tidyverse.org/) before, start by reading
<https://www.tidyverse.org/help/#reprex>. When you create your reprex, please use a 
data set available in a package.

```r
# inset reprex here
```

## Release summary
This version of `areal` is a maintenance release: it addresses some user submitted issues on Github, but is primarily focused on updating the maintainer contact information.

## Test environments
* local macOS install: R 4.1.2
* Linux ubuntu distribution (via GitHub Actions): R-devel, R-release, past four R-oldrel (4.1.3, 4.0.5, 3.6.3, 3.5.3, 3.4.4)
* macOS (via GitHub Actions): R-release
* windows (via GitHub Actions): R-release
* winbuilder: R-release, R-oldrel, R-devel

## R CMD check results
There were no ERRORs, WARNINGs, or NOTEs with local or CI checks. There is one NOTE on winbuilder:

```r
* checking CRAN incoming feasibility ... NOTE
Maintainer: 'Christopher Prener <chris.prener@gmail.com>'

New maintainer:
  Christopher Prener <chris.prener@gmail.com>
Old maintainer(s):
  Christopher Prener <chris.prener@slu.edu>
```

I am still the maintainer - I have just changed my contact information due to new employment! 

---
title: "Appendix for JOSS Paper"
author: "Christopher Prener, Ph.D."
date: '(`r format(Sys.time(), "%B %d, %Y")`)'
output: 
  github_document: default
  html_notebook: default 
---

## Introduction
This document contains replication code for the examples provided in our *Journal of Open Source Software* manuscript.

## Dependencies
This notebook requires

```{r load-packages}
# primary package
library(areal)

# tidyverse packages
library(dplyr)

# spatial packages
library(sf)
library(tidycensus)
library(tigris)

# other packages
library(gridExtra)
library(microbenchmark)
library(testthat)
```

## Comparisons with `sf`
### Produce Estimates
First, we'll create three spatially extensive estimates for comparison. Two will use the `areal` package, varying the type of weight applied to the estimate:

```{r areal-extensive}
# areal package, spatially extensive using total
areal_exT <- aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_race, sid = GEOID,
               weight = "total", output = "tibble", extensive = "TOTAL_E")

# areal package, spatially extensive using sum
areal_exS <- aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_race, sid = GEOID,
                            weight = "sum", output = "tibble", extensive = "TOTAL_E")
```

Next, we'll replicate the process using `sf`:

```{r sf-extensive}
# sf package, spatially extensive
sf_ex <- st_interpolate_aw(ar_stl_race["TOTAL_E"], ar_stl_wards, extensive = TRUE)
```

We'll also produce a spatially intensive estimate using `areal`:

```{r areal-intensive}
# areal package, spatially intensive
areal_in <- aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_asthma, sid = GEOID,
                            weight = "sum", output = "tibble", intensive = "ASTHMA")
```

And finally, we'll replicate the spatially intensive estimate using `sf`:

```{r sf-intensive}
# sf package, spatially intensive
sf_in <- st_interpolate_aw(ar_stl_asthma["ASTHMA"], ar_stl_wards, extensive = FALSE)
```

### Compile Results
First, we'll compile the extensive results:

```{r compile-extensive}
# areal, extensive sum
areal_exS <- areal_exS %>%
  select(WARD, TOTAL_E) %>%
  rename(areal_exS = TOTAL_E)

# areal, extensive total
areal_exT <- areal_exT %>%
  select(WARD, TOTAL_E) %>%
  rename(areal_exT = TOTAL_E)

# sf, extensive total
sf_ex <- sf_ex %>%
  rename(sf_ex = TOTAL_E)
st_geometry(sf_ex) <- NULL

# combine
extensive <- left_join(sf_ex, areal_exT, by = c("Group.1" = "WARD")) %>%
  left_join(., areal_exS, by = c("Group.1" = "WARD")) %>%
  mutate(delta = areal_exT-areal_exS) %>%
  rename(Ward = Group.1) %>%
  as_tibble()
```

We'll make a similar compliation of the intensive results:

```{r compile-intensive}
# areal, intensive
areal_in <- areal_in %>%
  select(WARD, ASTHMA) %>%
  rename(areal_in = ASTHMA)

# sf, intensive
sf_in <- sf_in %>%
  rename(sf_in = ASTHMA)
st_geometry(sf_in) <- NULL

# combine
intensive <- left_join(sf_in, areal_in, by = c("Group.1" = "WARD")) %>%
  rename(Ward = Group.1) %>%
  as_tibble()
```

### Print Tables
The following code chunk produces two tables for the manuscript:

```{r print-tables}
# produce rounded extensive estimates
extensiveSub <- extensive %>%
  filter(Ward >= 1 & Ward <= 10) %>%
  mutate(
    sf_ex = round(sf_ex, digits = 3),
    areal_exT = round(areal_exT, digits = 3),
    areal_exS = round(areal_exS, digits = 3),
    delta = round(delta, digits = 3)
  ) %>%
  rename(
    `sf` = sf_ex,
    `areal, total weight` = areal_exT,
    `areal, sum weight` = areal_exS
  )

# print extensive table
png(filename = "paper/extensiveTable.png", width = 480, height = 300, bg = "white", type = "cairo-png")
grid.arrange(tableGrob(extensiveSub, rows = NULL), top = "Comparison of sf and areal Output\nSpatially Extensive Interpolation")
dev.off()

# produce rounded intensive estimates
intensiveSub <- intensive %>%
  filter(Ward >= 1 & Ward <= 10) %>%
  mutate(
    sf_in = round(sf_in, digits = 3),
    areal_in = round(areal_in, digits = 3)
  ) %>%
  rename(
    `sf` = sf_in,
    `areal` = areal_in
  )

# print intensive table
png(filename = "paper/intensiveTable.png", width = 480, height = 300, bg = "white", type = "cairo-png")
grid.arrange(tableGrob(intensiveSub, rows = NULL), top = "Comparison of sf and areal Output\nSpatially Intensive Interpolation")
dev.off()
```

### Compare Results
We can verify that the `areal` workflow with `weight = "total"` matches the `sf` extensive output:

```{r verify-extensive}
expect_equal(extensive$sf_ex, extensive$areal_exT)
```

We can do the same for the intensive interpolations:

```{r verify-intensive}
expect_equal(intensive$sf_in, intensive$areal_in)
```

### Benchmark
Next, we'll benchmark the extensive estimation times:

```{r extensive-benchmark}
# compare spatially extensive interpolations
microbenchmark(
  aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_race, sid = GEOID,
                 weight = "total", output = "tibble", extensive = "TOTAL_E"),
  suppressWarnings(st_interpolate_aw(ar_stl_race["TOTAL_E"], ar_stl_wards, extensive = TRUE))
)
```

We'll repeat the process for the intensive estimations:

```{r intensive-benchmark}
# compare spatially intensive interpolations
microbenchmark(
  aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_asthma, sid = GEOID,
                 weight = "sum", output = "tibble", intensive = "ASTHMA"),
  suppressWarnings(st_interpolate_aw(ar_stl_asthma["ASTHMA"], ar_stl_wards, extensive = FALSE))
)
```

## Geometry Collections
Finally, we'll provide an example of a more intensive estimation process that also triggers the geometry collection workflow, which will add to the estimation time. We need to download several data sets using `tigris` and `tidycensus`:

```{r download-census, include=FALSE}
# county populations
moPop <- get_acs(geography = "county", variables = "B01003_001", output = "wide", state = 29, geometry = TRUE) %>%
  st_transform(crs = 26915) %>%
  select(GEOID, B01003_001E) %>%
  rename(totalPop = B01003_001E)

# block group geometry
moBlockGroups <- block_groups(state = 29, class = "sf") %>%
  st_transform(crs = 26915) %>%
  select(GEOID)
```

Here are the sample sizes for both data sets:

```{r census-n}
nrow(moPop)
nrow(moBlockGroups)
```

Here is the benchmark for the estimates produced with these data:

```{r benchmark-geo-collection}
microbenchmark(
  aw_interpolate(moBlockGroups, tid = GEOID, source = moPop, sid = GEOID,
                 weight = "sum", output = "tibble", intensive = "totalPop")
)
```

We would like to thank both @edzer and @sjsrey as well as @lheagy for the feedback and the opportunity to revise both the software and the manuscript. We believe that the manuscript has been greatly strengthened as a result of your collective feedback and are excited to resubmit it for your review. @lheagy, if there is a better way to embed tables and to handle the appendix file we've created (available [here](https://github.com/slu-openGIS/areal/blob/master/paper/appendix.md)), please let us know - we're happy to adapt both to any style that *JOSS* prefers.

## Reviewer 1 - @edzer
* The pdf of the paper shows R code on page 2, but this code is hidden where it runs off the page at the side.
    * The example has been changed to return a tibble object rather than an `sf` object. A new line was added below the example that clarifies the `sf` functionality as well.

* The manuscript claims it provides functions that "validate data suitability for areal interpolation", but does not explain what this validation does. What does it do?
    * A description of the validation process has been added to the manuscript in the section titled **The `areal` R package**.

* given that the package does little more than sf::st_interpolate_aw, and given that that function was available first, the added value of the packages would become more clear when it would mention the name of that function (rather than refer to "the existing functionality in sf"),
    * A direct reference to `sf::st_interpolate_aw` in the first paragraph of the section titled **The `areal` R package**.

* show in a side-by-side comparison that areal::aw_interpolate and sf::st_interpolate_aw give identical results for an intensive an extensive variable
    * This is included in the new **A Quick Comparison with sf** section with two figures, one for extensive interpolations and one for intensive interpolations.

* give an example where areal::aw_interpolate does something different when there is a difference in areas, and show how large the effect of doing this is
    * This is included in the new **A Quick Comparison with sf** section with a figure and a description of the difference as well as why one approach may be preferable over the other.

* the correct reference for Pebesma, E. (2018) is found here
    * This has been corrected

* change the wording of "black box", as this is not a black box. Closed source software is a black box
    * This has been changed to:

    > "for users who need to unpack the interpolation workflow"

* the areal-weighted-interpolation vignette mentions "Spatially intensive operations are used when the data to be interpolated are a ratio." I'm afraid it is not this simple: a counter example is is CO2 emissions measured in tons/year: an extensive variable when interpolated spatially.
    * This has been changed to:

    > "Spatially *intensive* operations are used when the data to be interpolated are, for example, a percentage or density value."

* Additionally, @edzer provided [feedback](https://github.com/slu-openGIS/areal/issues/18) about two packages listed as dependencies that may not be necessary.
    * Both `lwgeom` and `tibble` have been removed as dependencies

## Reviewer 2 - @sjsrey

* Scaling: the current examples are well crafted and illustrate the core functionality. At the same time n is fairly modest here and one wonders if these methods were going to be used for larger n problems, or applied iteratively over many cities , would there be any bottlenecks users should be aware of?
    * We have added benchmarks to the end a new section titled **Performance Notes** as well as a discussion of a special case that `areal` handles, which is when geometry collections are returned by `sf::st_intersection()`. The process for handling this special case is more resource intensive and results in longer processing times.

* The distinction between extensive and intensive variables is handled nicely. One case that might also be considered is the choice between interpolation of an intensive variable directly versus deriving the intensive estimates as a ratio of two extensive variables that have been estimated
    * Mindful of the short length of JOSS manuscripts, we have included a discussion of this in the vignette on areal weighted interpolation in the section entitled **Mixed Interpolations**.

* The weight argument names are a bit confusing.
    * Since the package is already on CRAN, we are reluctant to change the argument. In the paper, however, we have made two clarifications to address this. The first was to explicitly link the names for both weights to their descriptions in the second to last paragraph of the section **The areal R package**. They now read:

    > "...we offer a formula that matches the existing functionality in `sf`, which is based on the total area of the original source feature (specified with `weight = "total"`)."

    > "This uses a sum of the source feature areas remaining after the data are intersected as part of the spatial weight calculation process (specified with `weight = "sum"`)."

    We also have added a section titled **A Quick Comparison with sf** that illustrates the difference between these two approaches, and includes a brief discussion of the selection process between both in the second and third full paragraphs. It includes the following language:

    > "If we expect that our source and target data should overlap completely (but perhaps do not because of data quality issues), the `"sum"` approach will allocate all individuals into target features whereas `"total"`will not, instead allocating only a proportion of individuals relative to the overlap between the source and target features. In the example data provided in the `areal` package, this is the case - the Census tracts do not perfectly map onto the extent of the wards, and so the `"total"` approach is inappropriate."

Appendix for JOSS Paper
================
Christopher Prener, Ph.D.
(March 23, 2019)

## Introduction

This document contains replication code for the examples provided in our
*Journal of Open Source Software* manuscript.

## Dependencies

This notebook requires

``` r
# primary package
library(areal)

# tidyverse packages
library(dplyr)
```

    ## 
    ## Attaching package: 'dplyr'

    ## The following objects are masked from 'package:stats':
    ## 
    ##     filter, lag

    ## The following objects are masked from 'package:base':
    ## 
    ##     intersect, setdiff, setequal, union

``` r
# spatial packages
library(sf)
```

    ## Linking to GEOS 3.6.1, GDAL 2.1.3, PROJ 4.9.3

``` r
library(tidycensus)
library(tigris)
```

    ## To enable 
    ## caching of data, set `options(tigris_use_cache = TRUE)` in your R script or .Rprofile.

    ## 
    ## Attaching package: 'tigris'

    ## The following object is masked from 'package:graphics':
    ## 
    ##     plot

``` r
# other packages
library(gridExtra)
```

    ## 
    ## Attaching package: 'gridExtra'

    ## The following object is masked from 'package:dplyr':
    ## 
    ##     combine

``` r
library(microbenchmark)
library(testthat)
```

    ## 
    ## Attaching package: 'testthat'

    ## The following object is masked from 'package:dplyr':
    ## 
    ##     matches

## Comparisons with `sf`

### Produce Estimates

First, we’ll create three spatially extensive estimates for comparison.
Two will use the `areal` package, varying the type of weight applied to
the estimate:

``` r
# areal package, spatially extensive using total
areal_exT <- aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_race, sid = GEOID,
               weight = "total", output = "tibble", extensive = "TOTAL_E")

# areal package, spatially extensive using sum
areal_exS <- aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_race, sid = GEOID,
                            weight = "sum", output = "tibble", extensive = "TOTAL_E")
```

Next, we’ll replicate the process using `sf`:

``` r
# sf package, spatially extensive
sf_ex <- st_interpolate_aw(ar_stl_race["TOTAL_E"], ar_stl_wards, extensive = TRUE)
```

    ## Warning in st_interpolate_aw.sf(ar_stl_race["TOTAL_E"], ar_stl_wards,
    ## extensive = TRUE): st_interpolate_aw assumes attributes are constant over
    ## areas of x

We’ll also produce a spatially intensive estimate using `areal`:

``` r
# areal package, spatially intensive
areal_in <- aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_asthma, sid = GEOID,
                            weight = "sum", output = "tibble", intensive = "ASTHMA")
```

And finally, we’ll replicate the spatially intensive estimate using
`sf`:

``` r
# sf package, spatially intensive
sf_in <- st_interpolate_aw(ar_stl_asthma["ASTHMA"], ar_stl_wards, extensive = FALSE)
```

    ## Warning in st_interpolate_aw.sf(ar_stl_asthma["ASTHMA"], ar_stl_wards,
    ## extensive = FALSE): st_interpolate_aw assumes attributes are constant over
    ## areas of x

### Compile Results

First, we’ll compile the extensive results:

``` r
# areal, extensive sum
areal_exS <- areal_exS %>%
  select(WARD, TOTAL_E) %>%
  rename(areal_exS = TOTAL_E)

# areal, extensive total
areal_exT <- areal_exT %>%
  select(WARD, TOTAL_E) %>%
  rename(areal_exT = TOTAL_E)

# sf, extensive total
sf_ex <- sf_ex %>%
  rename(sf_ex = TOTAL_E)
st_geometry(sf_ex) <- NULL

# combine
extensive <- left_join(sf_ex, areal_exT, by = c("Group.1" = "WARD")) %>%
  left_join(., areal_exS, by = c("Group.1" = "WARD")) %>%
  mutate(delta = areal_exT-areal_exS) %>%
  rename(Ward = Group.1) %>%
  as_tibble()
```

We’ll make a similar compliation of the intensive results:

``` r
# areal, intensive
areal_in <- areal_in %>%
  select(WARD, ASTHMA) %>%
  rename(areal_in = ASTHMA)

# sf, intensive
sf_in <- sf_in %>%
  rename(sf_in = ASTHMA)
st_geometry(sf_in) <- NULL

# combine
intensive <- left_join(sf_in, areal_in, by = c("Group.1" = "WARD")) %>%
  rename(Ward = Group.1) %>%
  as_tibble()
```

### Print Tables

The following code chunk produces two tables for the manuscript:

``` r
# produce rounded extensive estimates
extensiveSub <- extensive %>%
  filter(Ward >= 1 & Ward <= 10) %>%
  mutate(
    sf_ex = round(sf_ex, digits = 3),
    areal_exT = round(areal_exT, digits = 3),
    areal_exS = round(areal_exS, digits = 3),
    delta = round(delta, digits = 3)
  ) %>%
  rename(
    `sf` = sf_ex,
    `areal, total weight` = areal_exT,
    `areal, sum weight` = areal_exS
  )

# print extensive table
png(filename = "paper/extensiveTable.png", width = 480, height = 300, bg = "white", type = "cairo-png")
grid.arrange(tableGrob(extensiveSub, rows = NULL), top = "Comparison of sf and areal Output\nSpatially Extensive Interpolation")
dev.off()
```

    ## quartz_off_screen 
    ##                 2

``` r
# produce rounded intensive estimates
intensiveSub <- intensive %>%
  filter(Ward >= 1 & Ward <= 10) %>%
  mutate(
    sf_in = round(sf_in, digits = 3),
    areal_in = round(areal_in, digits = 3)
  ) %>%
  rename(
    `sf` = sf_in,
    `areal` = areal_in
  )

# print intensive table
png(filename = "paper/intensiveTable.png", width = 480, height = 300, bg = "white", type = "cairo-png")
grid.arrange(tableGrob(intensiveSub, rows = NULL), top = "Comparison of sf and areal Output\nSpatially Intensive Interpolation")
dev.off()
```

    ## quartz_off_screen 
    ##                 2

### Compare Results

We can verify that the `areal` workflow with `weight = "total"` matches
the `sf` extensive output:

``` r
expect_equal(extensive$sf_ex, extensive$areal_exT)
```

We can do the same for the intensive interpolations:

``` r
expect_equal(intensive$sf_in, intensive$areal_in)
```

### Benchmark

Next, we’ll benchmark the extensive estimation times:

``` r
# compare spatially extensive interpolations
microbenchmark(
  aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_race, sid = GEOID,
                 weight = "total", output = "tibble", extensive = "TOTAL_E"),
  suppressWarnings(st_interpolate_aw(ar_stl_race["TOTAL_E"], ar_stl_wards, extensive = TRUE))
)
```

    ## Unit: milliseconds
    ##                                                                                                                                          expr
    ##  aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_race,      sid = GEOID, weight = "total", output = "tibble", extensive = "TOTAL_E")
    ##                                              suppressWarnings(st_interpolate_aw(ar_stl_race["TOTAL_E"], ar_stl_wards,      extensive = TRUE))
    ##       min       lq     mean   median       uq      max neval cld
    ##  264.6951 272.8943 279.4292 276.2390 281.6604 352.8633   100   b
    ##  236.6746 245.1094 251.3938 248.7699 251.9611 345.5775   100  a

We’ll repeat the process for the intensive estimations:

``` r
# compare spatially intensive interpolations
microbenchmark(
  aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_asthma, sid = GEOID,
                 weight = "sum", output = "tibble", intensive = "ASTHMA"),
  suppressWarnings(st_interpolate_aw(ar_stl_asthma["ASTHMA"], ar_stl_wards, extensive = FALSE))
)
```

    ## Unit: milliseconds
    ##                                                                                                                                         expr
    ##  aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_asthma,      sid = GEOID, weight = "sum", output = "tibble", intensive = "ASTHMA")
    ##                                           suppressWarnings(st_interpolate_aw(ar_stl_asthma["ASTHMA"], ar_stl_wards,      extensive = FALSE))
    ##       min       lq     mean   median       uq      max neval cld
    ##  254.0831 259.8244 267.8781 265.3997 269.5136 431.5775   100   b
    ##  237.4632 242.4974 248.1372 246.4525 250.7644 329.9287   100  a

## Geometry Collections

Finally, we’ll provide an example of a more intensive estimation process
that also triggers the geometry collection workflow, which will add to
the estimation time. We need to download several data sets using
`tigris` and `tidycensus`:

Here are the sample sizes for both data sets:

``` r
nrow(moPop)
```

    ## [1] 115

``` r
nrow(moBlockGroups)
```

    ## [1] 4506

Here is the benchmark for the estimates produced with these data:

``` r
microbenchmark(
  aw_interpolate(moBlockGroups, tid = GEOID, source = moPop, sid = GEOID,
                 weight = "sum", output = "tibble", intensive = "totalPop")
)
```

    ## Unit: seconds
    ##                                                                                                                                     expr
    ##  aw_interpolate(moBlockGroups, tid = GEOID, source = moPop, sid = GEOID,      weight = "sum", output = "tibble", intensive = "totalPop")
    ##       min       lq     mean   median       uq      max neval
    ##  17.28777 17.54022 17.68895 17.61689 17.71123 21.12038   100

# Contributing to this Lesson
Thanks for using this package. Before filing an issue or opening a pull
request, there are a few places to explore and pieces to put together to make
the process as smooth as possible.

## Getting Started
Start by making a minimal reproducible example (**reprex**) using the
[reprex](http://reprex.tidyverse.org/) package. If you haven't heard of or used
reprex before, you're in for a treat! Seriously, reprex will make all of your
R-question-asking endeavors easier (which is a pretty insane ROI for the five to
ten minutes it'll take you to learn what it's all about). For additional reprex
pointers, check out the [Get help!](https://www.tidyverse.org/help/) section of
the tidyverse site.

Armed with your reprex, the next step is to figure out [where to ask](https://www.tidyverse.org/help/#where-to-ask).

  * If it's a question: start with [community.rstudio.com](https://community.rstudio.com/).
    There are more people there to answer questions.  
  * If it's a bug: you're in the right place, file an issue.  
  * If it's a feature request: you're in the right place, file an issue.  
  * If you're not sure: let the community help you figure it out! If your
    problem _is_ a bug or a feature request, you can easily return here and
    report it. You can use our _other inquiry_ issue template to get our
    feedback.

## Before Opening an Issue
Before opening a new issue, be sure to [search issues and pull requests][issuesLink]
to make sure the bug or typo hasn't been reported and/or already fixed.
Similarly, search to make sure someone has not already made a suggestion. If
there is a suggestion and you want to express support for it, please comment
in the existing issue.

By default, the search will be pre-populated with `is:issue is:open`. You can
[edit the qualifiers](https://help.github.com/articles/searching-issues-and-pull-requests/)
(e.g. `is:pr`, `is:closed`) as needed. For example, you'd simply
remove `is:open` to search _all_ issues in the repo, open or closed.

## Issue Templates
When you go to open an issue, you are encouraged to select from one of the
three templates: bug reports, feature requests, or other inquiries. These will
help you craft an issue that is as helpful and clear as possible!

## PR process
If you've found a way to contribute, please follow the PR process described
here to fork, edit, and push your proposed changes. We recommend opening an
issue to gauge our interest before spending time working on a PR!

### Fork, clone, branch
The first thing you'll need to do is to [fork](https://help.github.com/articles/fork-a-repo/)
[this lesson][repoLink], and then clone it locally.
We recommend that you create a branch for each PR.

### Style
Match the existing code or documentation style. Be careful to only make style changes
to the code you are contributing.

### Document
Make sure to update the help files if your changes impact
any of those documents. If you add a new package dependency, please also update
the `DESCRIPTION` file so that the dependency is part of our continuous integration
testing.

### Check
Once you are done, make sure that all changed `.Rmd` documents knit without error.

### Commit
When you've made your changes, write a clear commit message describing what
you've done. If you've fixed or closed an issue, make sure to include keywords
(e.g. `fixes #101`) at the end of your commit message (not in its
title) to automatically close the issue when the PR is merged.

### Push and pull
Once you've pushed your commit(s) to a branch in _your_ fork, you're ready to
make the pull request. Pull requests should have descriptive titles to remind
reviewers/maintainers what the PR is about. You can easily view what exact
changes you are proposing using either the [Git diff](http://r-pkgs.had.co.nz/git.html#git-status)
view in RStudio, or the [branch comparison view](https://help.github.com/articles/creating-a-pull-request/)
you'll be taken to when you go to create a new PR. If the PR is related to an
issue, provide the issue number and slug in the _description_ using
auto-linking syntax (e.g. `#15`).

## Code of Conduct
Please note that this project is released with a [Contributor Code of
Conduct](CODE_OF_CONDUCT.md). By participating in this project you agree to
abide by its terms.

## Acknowledgements
These contributing guidelines are modified from those used by [Jenny Bryan](https://github.com/jennybc).

[repoLink]: https://github.com/chris-prener/areal
[issuesLink]: https://github.com/chris-prener/areal/issues

---
title: "Preparing Data for Interpolation"
author: "Christopher Prener, Ph.D."
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Preparing Data for Interpolation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

library(areal)
library(dplyr)
library(sf)

data(ar_stl_asthma, package = "areal")

data(ar_stl_race, package = "areal")

data(ar_stl_wards, package = "areal")
```

Depending on the state of your spatial data, they may require some cleaning and modification before interpolation. The types of issues to address pre-interpolation fall into a few distinct categories:

1. Data are not in the right format
2. Data are not in the right coordinate system
3. Data have variable name conflicts
4. Data have too many variables or observations and therefore must be subset

Each of these conditions will be discussed below. The following examples assume:

```{r}
library(areal)
library(dplyr)   # data wrangling
library(sf)      # spatial data operations

# load data into enviornment
race <- ar_stl_race
asthma <- ar_stl_asthma
wards <- ar_stl_wards

# create example data - non-spatial data
asthmaTbl <- ar_stl_asthma
st_geometry(asthmaTbl) <- NULL

# create example data - wrong crs
race83 <- st_transform(race, crs = 4269)
```

## Validating Data
`areal` has a built-in tool for data validation, `ar_validate()`, that provides an excellent starting place for ensuring your data are ready to interpolate. The validation process covers the first three issues listed in the introduction. It can be run in a simple format:

```{r validate-simple}
ar_validate(source = asthma, target = wards, varList = "ASTHMA", method = "aw")
```

If `ar_validate()` returns a `FALSE` value, it can also be run in a verbose manner that returns a detailed tibble:

```{r validate-verbose}
ar_validate(source = asthma, target = wards, varList = "ASTHMA", method = "aw", verbose = TRUE)
```

Use the `verbose = TRUE` output as a checklist to address issues before interpolating your data.

## Format Issues
Data need to be loaded as `sf` objects in `R`. If `ar_validate()` returns `FALSE` for the first condition, data are not stored as `sf` objects:

```{r validate-non-sf}
ar_validate(source = asthmaTbl, target = wards, varList = "ASTHMA", method = "aw", verbose = TRUE)
```

### External Spatial Data
If you have external data, using `st_read()` is the best way to load them:

```r
df <- sf::st_read("data.shp", stringsAsFactors = FALSE)
```

The `st_read()` function supports a variety of the most common data sources.

### sp Data
If you are working with data that is an `sp` object in `R`, the `sf` package has a function `st_as_sf()` can be used to convert the object to `sf`:

```r
sf_object <- sf::st_as_sf(sp_object)
```

### Data From tidycensus and tigris
If you are using either the [`tidycensus`](https://walker-data.com/tigris) or [`tigris`](https://github.com/walkerke/tigris) packages to access spatial data, there are options for downloading data as `sf` objects as well. In `tidycensus`, the `geometry = TRUE` option should be used. In `tigris`, the `class = "sf"` option should be used.

```r
stl_race <- tidycensus::get_acs(geography = "tract", state = 29, county = 510, year = 2017, 
                                table = "B02001", output = "wide", geometry = TRUE)
stl_tracts <- tigris::tracts(state = 29, county = 510, class = "sf")
```

### Tabular Data
If you have tabular data, you will need to merge them with a shapefile or other spatial data set that contains the geometry for the corresponding spatial features. For American users, a potentially common scenario here will be table of census geography (such as tracts) that lacks the geometry for those features. These can be downloaded using [`tigris`](https://github.com/walkerke/tigris) and then combined using [`dplyr`](https://dplyr.tidyverse.org). The following example assumes the tract data contains an identification number column named `GEOID` with the appropriate tract identifiers for St. Louis:

```r
stl_tracts <- tigris::tracts(state = 29, county = 510, class = "sf")

tract_data <- dplyr::left_join(stl_tracts, asthmaTBL, by = "GEOID")
```

## Coordinate Systems
Two coordinate system rules are enforced by `ar_validate`: both the source and the target data must be in the same [coordinate system](https://geocompr.robinlovelace.net/spatial-class.html#crs-intro) and that coordinate system must be a projected coordinate system. The `sf` package contains a function named `st_crs()` for previewing the current coordinate system of your data:

```{r race-crs}
st_crs(race83)
```

The `EPSG` value `4269` refers to a specific type of coordinate system known as as geographic coordinate system. These data use latitude-longitude values, which vary in length and thus cannot be used to calculate area (a key component of carrying out areal interpolations). These data would fail the `ar_validate()` process:

```{r validate-non-matching-crs}
ar_validate(source = race83, target = wards, varList = "TOTAL_E", method = "aw", verbose = TRUE)
```

These data fail both the matching CRS validation and the planar CRS validation because the `race` and `wards` data are in two different coordinate systems. We've already seen the CRS data for `race` above so we'll look at the `CRS data for `wards`:

```{r wards-crs}
st_crs(wards)
```

If these data fail that process, `st_crs()` can be used to identify whether the source or the target (or both) are in an inappropriate coordinate system. The `EPSG` value `26915` refers to another type of coordinate system known as as projected coordinate system (or "planar" data). Further, `26915` is a particular type of projected coordinate system known as the [Universal Transverse Mercator](https://en.wikipedia.org/wiki/Universal_Transverse_Mercator_coordinate_system) coordinate system. This is a good option for using `areal` because it is available around the world. There are a variety of tools for identifying the appropriate UTM zone, including this excellent [interactive map](https://mangomap.com/robertyoung/maps/69585/what-utm-zone-am-i-in-#). Once you have an EPSG zone, the website [epsg.io](https://epsg.io) can be used to search for the appropriate `EPSG` value.

In this case, the `wards` data are in an appropriate system but the `race` data need to be transformed. The `st_transform()` function can be used to modify them to an appropriate coordinate system:

```{r transform-crs}
raceFixed <- st_transform(race83, crs = 26915)
```

Once this is done, these data should pass the `ar_validate()` process:

```{r validate-matching-crs}
ar_validate(source = raceFixed, target = wards, varList = "TOTAL_E", method = "aw", verbose = TRUE)
```

## Variable Conflicts
If a variable does not exist in the source data, the validation process will fail:

```{r}
ar_validate(source = race, target = wards, varList = "TOTAL", method = "aw", verbose = TRUE)
```

In this case, the fix is as easy as correcting the typo in our `ar_validate()` call - the variable `TOTAL` should really be `TOTAL_E`:

```{r}
names(race)
```

Another common problem is that a variable exists in both our source and target data. For instance, we could add a `TOTAL_E` variable to our target data:

```{r}
wardsVar <- mutate(wards, TOTAL_E = seq(1:28))

ar_validate(source = race, target = wardsVar, varList = "TOTAL_E", method = "aw", verbose = TRUE)
```

Now, we fail the validation process because there is a conflict between the source and target data - `TOTAL_E` exists in both data sets. We can use the `select()` function from `dplyr` to remove the offending variable from the target data before proceeding:

```{r}
wardsFixed <- select(wardsVar, -TOTAL_E)

ar_validate(source = race, target = wardsFixed, varList = "TOTAL_E", method = "aw", verbose = TRUE)
```

Another option would be to rename the column using the `rename()` function from `dplyr` or by using another approach for renaming columns. Either option will work for `areal`'s purposes.

## Subsetting
One cautionary note when using `areal` is that the validation process will not check to see if too many variables or observations exist in the data. Cleaning the data ahead of time so that only the relevant observations exist can help improve performance - if you have tract data for an entire state but only need a particular city, everything will go faster if you subset these data using the `filter()` function from `dplyr`:

```r
countyData <- filter(stateData, COUNTY == 510)
```

Having too many columns poses an organizational problem more than anything else. We strongly encourage users to limit their interpolated data to only the needed values. For example, the `wards` data contains an `AREA` column whose units we don't know and an `OBJECTID` column that is an artifact from its creation using an ESRI product:

```{r}
names(wards)
```

We can use the `select()` function from `dplyr` to remove these before interpolation since we don't need them:

```{r}
wardsSubset <- select(wards, -OBJECTID, -AREA)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aw_intersect.R
\name{aw_intersect}
\alias{aw_intersect}
\title{Intersect Source and Target Data}
\usage{
aw_intersect(.data, source, areaVar)
}
\arguments{
\item{.data}{A \code{sf} object that data should be interpolated to}

\item{source}{A \code{sf} object with data to be interpolated}

\item{areaVar}{The name of the new area variable to be calculated.}
}
\value{
A \code{sf} object with the intersected data and new area field.
}
\description{
\code{aw_intersect} intersects the source and target datasets and
    computes a new area field for the intersected data using the units associated
    with whatever project the data are currently in. This is the first step in the
    interpolation process after data validation and subsetting.
}
\examples{
library(dplyr)

race <- select(ar_stl_race, GEOID, TOTAL_E)
wards <- select(ar_stl_wards, WARD)

aw_intersect(wards, source = race, areaVar = "area")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aw_verify.R
\name{aw_verify}
\alias{aw_verify}
\title{Verify Correct Extensive-Sum Interpolation}
\usage{
aw_verify(source, sourceValue, result, resultValue)
}
\arguments{
\item{source}{A \code{sf} object with data to be interpolated}

\item{sourceValue}{A column within \code{source} to be interpolated}

\item{result}{A \code{sf} object with interpolated data}

\item{resultValue}{A column within \code{result} with the interpolated values}
}
\value{
A logical scalar; if \code{TRUE}, these two values are equal.
}
\description{
Verify Correct Extensive-Sum Interpolation
}
\details{
\code{aw_verify} ensures that the sum of the resulting interpolated
    value is equal to the sum of the original source value. This functionality
    only works for interpolations that are extensive and use the \code{sum}
    approach to calculating areal weights.
}
\examples{
result <- aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_race, sid = GEOID,
                         weight = "sum", output = "tibble", extensive = "TOTAL_E")

aw_verify(source = ar_stl_race, sourceValue = TOTAL_E, result = result, resultValue = TOTAL_E)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aw_interpolate.R
\name{aw_interpolate}
\alias{aw_interpolate}
\title{Interpolate Values}
\usage{
aw_interpolate(.data, tid, source, sid, weight = "sum", output = "sf", extensive,
    intensive)
}
\arguments{
\item{.data}{A \code{sf} object that data should be interpolated to (this is referred
to as the \code{target} elsewhere in the package).}

\item{tid}{A unique identification number within \code{target}}

\item{source}{A \code{sf} object with data to be interpolated}

\item{sid}{A unique identification number within \code{source}}

\item{weight}{For \code{"extensive"} interpolations, should be either \code{"total"} or
\code{"sum"}. For \code{"intensive"} interpolations, should be \code{"sum"}. For mixed
interpolations, this will only impact the calculation of the extensive variables.}

\item{output}{One of either \code{"sf"} or \code{"tibble"}}

\item{extensive}{A vector of quoted variable names to be treated as spatially extensive
(e.g. population counts); optional if \code{intensive} is specified}

\item{intensive}{A vector of quoted variable names to be treated as spatially intensive
(e.g. population density); optional if \code{extensive} is specified}
}
\value{
A \code{sf} object or a \code{tibble} with the value or values interpolated into
    the \code{target} data.
}
\description{
This is the core function within the package for areal weighted
    interpolation. It validates both data sources before interpolating one or more
    listed values from the source data into the target data.
}
\details{
Areal weighted interpolation can be used for generating demographic
    estimates for overlapping but incongruent polygon features. It assumes that
    individual members of a population are evenly dispersed within the source features
    (an assumption not likely to hold in the real world). It also functions best
    when data are in a projected coordinate system, like the UTM coordinate system.
}
\examples{
aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_race, sid = GEOID, weight = "sum",
    output = "sf", extensive = "TOTAL_E")

aw_interpolate(ar_stl_wards, tid = WARD, source = ar_stl_asthma, sid = GEOID, weight = "sum",
    output = "tibble", intensive = "ASTHMA")

}
\seealso{
\link{c}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aw_calculate.R
\name{aw_calculate}
\alias{aw_calculate}
\title{Calculate Estimated Population}
\usage{
aw_calculate(.data, value, areaWeight, newVar)
}
\arguments{
\item{.data}{A given intersected dataset}

\item{value}{A column within \code{source} to be interpolated}

\item{areaWeight}{The name of the variable containing area weight per feature}

\item{newVar}{Optional; a new field name to store the interpolated value in. If not specified,
the \code{value} argument will be used as the new field name.}
}
\value{
An intersected file of class sf with a new field of interest recalculated with area weight
}
\description{
\code{aw_calculate} multiplies the given \code{value} by the area weight. This
    is the fourth step in the interpolation process after \link{aw_weight}.
}
\examples{
library(dplyr)

race <- select(ar_stl_race, GEOID, TOTAL_E)
wards <- select(ar_stl_wards, WARD)

wards \%>\%
    aw_intersect(source = race, areaVar = "area") \%>\%
    aw_total(source = race, id = GEOID, areaVar = "area", totalVar = "totalArea",
             weight = "sum", type = "extensive") \%>\%
    aw_weight(areaVar = "area", totalVar = "totalArea", areaWeight = "areaWeight") -> intersect

aw_calculate(intersect, value = "TOTAL_E", areaWeight = "areaWeight")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aw_total.R
\name{aw_total}
\alias{aw_total}
\title{Calculate Total Area}
\usage{
aw_total(.data, source, id, areaVar, totalVar, type, weight)
}
\arguments{
\item{.data}{A \code{sf} object that has been intersected using \link{aw_intersect}}

\item{source}{A \code{sf} object with data to be interpolated}

\item{id}{A unique identification number}

\item{areaVar}{The name of the variable measuring a feature's area, which is
created as part of \link{aw_intersect}}

\item{totalVar}{The name of a new total area field to be calculated}

\item{type}{One of \code{"intensive"} or \code{"extensive"}}

\item{weight}{One of \code{"sum"} or \code{"total"}}
}
\value{
A \code{sf} object with the intersected data and new total area field.
}
\description{
\code{aw_total} produces a new total area field that contains
    the total area by \code{source} id. This is the second step in the
    interpolation process after \link{aw_intersect}.
}
\examples{
library(dplyr)

race <- select(ar_stl_race, GEOID, TOTAL_E)
wards <- select(ar_stl_wards, WARD)

wards \%>\%
    aw_intersect(source = race, areaVar = "area") -> intersect

aw_total(intersect, source = race, id = GEOID, areaVar = "area",
         totalVar = "totalArea", weight = "sum", type = "extensive")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ar_tessellate.R
\name{ar_tessellate}
\alias{ar_tessellate}
\title{Create Tessellations From SF Object}
\usage{
ar_tessellate(.data, shape = "square", size = 1)
}
\arguments{
\item{.data}{An object of class \code{sf} to tessellate from}

\item{shape}{One of 'square' or 'hexagon', the shape to make tessellations from}

\item{size}{Numeric multiplier for size of tessellations, default is one kilometer}
}
\value{
A \code{sf} object
}
\description{
Create Tessellations From SF Object
}
\examples{
ar_tessellate(ar_stl_wards)

ar_tessellate(ar_stl_wards, shape = "hexagon", size = .75)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ar_stl_wards.R
\docType{data}
\name{ar_stl_wards}
\alias{ar_stl_wards}
\title{Ward Boundaries in St. Louis, 2010}
\format{
A data frame with 28 rows and 4 variables:
\describe{
  \item{OBJECTID}{Artifact from ESRI data creation}
  \item{WARD}{Ward number}
  \item{AREA}{area of each ward}
  \item{geometry}{simple features geometry}
  }
}
\source{
City of St. Louis
}
\usage{
data(ar_stl_wards)
}
\description{
A simple features data set containing the 2010 Ward boundaries, which
    are used as districts for Alderpersons who serve as elected representatives.
    The \code{OBJECTID} and \code{AREA} columns are included to simulate "real"
    data that may have superfluous or unclear columns.
}
\examples{
str(ar_stl_wards)
head(ar_stl_wards)
summary(ar_stl_wards$AREA)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ar_validate.R
\name{ar_validate}
\alias{ar_validate}
\title{Validating Data for Interpolation}
\usage{
ar_validate(source, target, varList, method = "aw", verbose = FALSE)
}
\arguments{
\item{source}{A \code{sf} object with data to be interpolated}

\item{target}{A \code{sf} object that data should be interpolated to}

\item{varList}{A vector of variable names to be added to the \code{target} object}

\item{method}{The areal interpolation method validation is being performed for. This
should be set to \code{"aw"}. Additional functionality will be added as the package
adds new interpolation techniques.}

\item{verbose}{A logical scalar; if \code{TRUE}, a tibble with test results is returned}
}
\value{
If \code{verbose} is \code{FALSE}, a logical scalar is returned that is \code{TRUE}
    is all tests are passed and \code{FALSE} if one or more tests is failed. If \code{verbose}
    is \code{TRUE}, a tibble with detailed test results is returned.
}
\description{
\code{ar_validate} executes a series of logic tests for \code{sf} object status,
    shared coordinates between source and target data, appropriate project, and absence of
    variable name conflicts.
}
\examples{
ar_validate(source = ar_stl_asthma, target = ar_stl_wards, varList = "ASTHMA")

ar_validate(source = ar_stl_asthma, target = ar_stl_wards, varList = "ASTHMA", verbose = TRUE)

}
\seealso{
\link{c}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ar_stl_race.R
\docType{data}
\name{ar_stl_race}
\alias{ar_stl_race}
\title{Race in St. Louis by Census Tract, 2017}
\format{
A data frame with 106 rows and 24 variables:
\describe{
  \item{GEOID}{full GEOID string}
  \item{STATEFP}{state FIPS code}
  \item{COUNTYFP}{county FIPS code}
  \item{TRACTCE}{tract FIPS code}
  \item{NAMELSAD}{tract name}
  \item{ALAND}{area of tract land, square meters}
  \item{AWATER}{area of tract water, square meters}
  \item{TOTAL_E}{total populaton count, estimated}
  \item{TOTAL_M}{total populaton count, margin of error}
  \item{WHITE_E}{white populaton count, estimated}
  \item{WHITE_M}{white populaton count, margin of error}
  \item{BLACK_E}{black populaton count, estimated}
  \item{BLACK_M}{black populaton count, margin of error}
  \item{AIAN_E}{american indian and alskan native populaton count, estimated}
  \item{AIAN_M}{american indian and alskan native populaton count, margin of error}
  \item{ASIAN_E}{asian populaton count, estimated}
  \item{ASIAN_M}{asian populaton count, margin of error}
  \item{NHPI_E}{native hawaiian and pacific islander populaton count, estimated}
  \item{NHPI_M}{native hawaiian and pacific islander populaton count, margin of error}
  \item{OTHER_E}{other race populaton count, estimated}
  \item{OTHER_M}{other race populaton count, margin of error}
  \item{TWOPLUS_E}{two or more races populaton count, estimated}
  \item{TWOPLUS_M}{two or more races populaton count, margin of error}
  \item{geometry}{simple features geometry}
  }
}
\source{
\code{tidycensus} package
}
\usage{
data(ar_stl_race)
}
\description{
A simple features data set containing the geometry and associated attributes
    for the 2013-2017 American Community Survey estimates for race in St. Louis.
}
\examples{
str(ar_stl_race)
head(ar_stl_race)
summary(ar_stl_race$ALAND)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aw_aggregate.R
\name{aw_aggregate}
\alias{aw_aggregate}
\title{Aggregate Estimates Based on Target ID}
\usage{
aw_aggregate(.data, target, tid, interVar, newVar)
}
\arguments{
\item{.data}{A given intersected dataset}

\item{target}{A \code{sf} object that data should be interpolated to}

\item{tid}{A unique identification number within \code{target}}

\item{interVar}{A variable containing an interpolated value created by \code{aw_calculate}}

\item{newVar}{Optional; a new field name to store the interpolated value in. If not specified,
the \code{interVar} argument will be used as the new field name.}
}
\value{
A \code{sf} object with the interpolated value added to it.
}
\description{
\code{aw_aggregate} sums the new estimates produced by \link{aw_calculate}
    based on the target id. These are then joined with the target data. This is
    the fourth step in the interpolation process after \link{aw_weight}.
}
\examples{
library(dplyr)

race <- select(ar_stl_race, GEOID, TOTAL_E)
wards <- select(ar_stl_wards, WARD)

wards \%>\%
    aw_intersect(source = race, areaVar = "area") \%>\%
    aw_total(source = race, id = GEOID, areaVar = "area", totalVar = "totalArea",
             weight = "sum", type = "extensive") \%>\%
    aw_weight(areaVar = "area", totalVar = "totalArea", areaWeight = "areaWeight") \%>\%
    aw_calculate(value = "TOTAL_E", areaWeight = "areaWeight") -> intersect

aw_aggregate(intersect, target = wards, tid = WARD, interVar = TOTAL_E)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aw_preview_weights.R
\name{aw_preview_weights}
\alias{aw_preview_weights}
\title{Preview Areal Weights}
\usage{
aw_preview_weights(.data, tid, source, sid, type)
}
\arguments{
\item{.data}{A \code{sf} object that data should be interpolated to (this is referred
to as the \code{target} elsewhere in the package).}

\item{tid}{A unique identification number within \code{target}}

\item{source}{A \code{sf} object with data to be interpolated}

\item{sid}{A unique identification number within \code{source}}

\item{type}{One of either \code{"extensive"} (if the data are spatitally extensive e.g.
population counts), \code{"intensive"} (if the data are spatially intensive e.g.
population density), or \code{"mixed"} (if the data include both extensive and
intensive values). If \code{"extensive"}, the sum is returned for the interpolated
value. If \code{"intensive"}, the mean is returned for the interpolated value.
If \code{"mixed"}, vectors named \code{"extensive"} and \code{"intensive"} containing
the relevant variable names should be specified in the dots.}
}
\value{
A tibble with the areal weights that would be used for interpolation if \code{type}
    is either \code{"extensive"} or \code{"intensive"}. If it is mixed, two tibbles (one for
    \code{"extensive"} and one for \code{"intensive"}) are returned as a list.
}
\description{
Provides a preview of the weight options for areal weighted interpolation.
    This can be useful for selecting the final specification for \code{aw_interpolate}
    without having to construct a pipeline of all of the subfunctions manually.
}
\examples{
aw_preview_weights(ar_stl_wards, tid = WARD, source = ar_stl_race, sid = GEOID,
                   type = "extensive")

aw_preview_weights(ar_stl_wards, tid = WARD, source = ar_stl_asthma, sid = GEOID,
                   type = "intensive")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ar_stl_wardsClipped.R
\docType{data}
\name{ar_stl_wardsClipped}
\alias{ar_stl_wardsClipped}
\title{Clipped Ward Boundaries in St. Louis, 2010}
\format{
A data frame with 28 rows and 2 variables:
\describe{
  \item{WARD}{Ward number}
  \item{geometry}{simple features geometry}
  }
}
\source{
City of St. Louis
}
\usage{
data(ar_stl_wardsClipped)
}
\description{
A simple features data set containing the 2010 Ward boundaries, which
    are used as districts for Alderpersons who serve as elected representatives.
    This version of the ward boundary has been modified so that the wards only
    extend to the Mississippi River shoreline.
}
\examples{
str(ar_stl_wardsClipped)
head(ar_stl_wardsClipped)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ar_stl_asthma.R
\docType{data}
\name{ar_stl_asthma}
\alias{ar_stl_asthma}
\title{Asthma in St. Louis by Census Tract, 2017}
\format{
A data frame with 106 rows and 24 variables:
\describe{
  \item{GEOID}{full GEOID string}
  \item{STATEFP}{state FIPS code}
  \item{COUNTYFP}{county FIPS code}
  \item{TRACTCE}{tract FIPS code}
  \item{NAMELSAD}{tract name}
  \item{ALAND}{area of tract land, square meters}
  \item{AWATER}{area of tract water, square meters}
  \item{ASTHMA}{percent of residents with current asthma diagnosis, estimated}
  \item{geometry}{simple features geometry}
  }
}
\source{
Centers for Disease Control's 500 Cities Data
}
\usage{
data(ar_stl_asthma)
}
\description{
A simple features data set containing the geometry and asthma estimates
    from the Centers for Disease Control for St. Louis.
}
\examples{
str(ar_stl_asthma)
head(ar_stl_asthma)
summary(ar_stl_asthma$ASTHMA)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aw_weight.R
\name{aw_weight}
\alias{aw_weight}
\title{Calculate Areal Weight}
\usage{
aw_weight(.data, areaVar, totalVar, areaWeight)
}
\arguments{
\item{.data}{A \code{sf} object that has been intersected using \link{aw_intersect}}

\item{areaVar}{The name of the variable measuring a feature's area}

\item{totalVar}{The name of the variable containing total area field by \code{source} id}

\item{areaWeight}{The name of a new area weight field to be calculated}
}
\value{
A \code{sf} object with the intersected data and new area weight field.
}
\description{
\code{aw_weight} creates an area weight field by dividing the area
    field by the total area field. This is the third step in the interpolation
    process after \link{aw_weight}.
}
\examples{
library(dplyr)

race <- select(ar_stl_race, GEOID, TOTAL_E)
wards <- select(ar_stl_wards, WARD)

wards \%>\%
    aw_intersect(source = race, areaVar = "area") \%>\%
    aw_total(source = race, id = GEOID, areaVar = "area", totalVar = "totalArea",
             weight = "sum", type = "extensive") -> intersect

aw_weight(intersect, areaVar = "area", totalVar = "totalArea", areaWeight = "areaWeight")

}
